<?php
/**
 * @package Blox Page Builder
 * @author UniteCMS.net
 * @copyright (C) 2017 Unite CMS, All Rights Reserved. 
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * */

defined('_JEXEC') or die('Restricted access');

class com_bloxInstallerScript
{
	
	/**
	 * Constructor
	 *
	 * @param   JAdapterInstance  $adapter  The object responsible for running this script
	 */
	public function __constructor(JAdapterInstance $adapter){
		
	}
 
			
	
	/**
	 * Called before any type of action
	 *
	 * @param   string  $route  Which action is happening (install|uninstall|discover_install)
	 * @param   JAdapterInstance  $adapter  The object responsible for running this script
	 *
	 * @return  boolean  True on success
	 */
	public function preflight($route, JAdapterInstance $adapter){
		
	}
 

	/**
	 * publish the plugin
	 */
	private function publishPlugin(){
		
		$sql = "UPDATE `#__extensions` SET `enabled`=1 where (`type`=\"plugin\" and `element`=\"blox\")";
		
		$db = JFactory::getDbo();
		$db->setQuery($sql);
		$db->query();		
	}
	
	
	/**
	 * print message about addons install
	 */
	private function printInstallAddonsMessage($message, $fullLog){
		
		if(empty($message))
			return(false);
		 
		$html = "<div style='padding-bottom:10px;'>";
		$html .= "<span style='color:green;'>$message</span>";
		
		$html .= " &nbsp; <a style='color:green;text-decoration:underline;' href='javascript:void(0);' onclick='document.getElementById(\"addons_install_details\").style.display=\"\"'>show details</a>";
		
		$html .= "<div id='addons_install_details' style='padding-top:10px;padding-bottom:10px;display:none'>";
		$html .= $fullLog;
		$html .= "</div>";
		
		$html .= "</div>";
		
		echo $html;
	}
	
	
	/**
	 * install addons
	 */
	private function installAddons(JAdapterInstance $adapter){
		
		//get addons path
		$installer = new JInstaller();
		$p_installer = $adapter->getParent();
		$pathInstaller = $p_installer->getPath("source");
		
		$pathInstallAddons = $pathInstaller."/install_addons/";
		
		//require component files
		if(class_exists("AddonLibraryJoomlaPluginsBase"))
			return(false);
		
		$pathIncludes = JPATH_ADMINISTRATOR."/components/com_blox/includes.php";
		require $pathIncludes;
		
		//import addons
		
		$objImporter = new UniteCreatorExporter();
		$fullLog = $objImporter->importAddonsFromFolder($pathInstallAddons);
		$textLog = $objImporter->getTextLogShort();
		
		$this->printInstallAddonsMessage($textLog, $fullLog);
		
	}
	
	
	/**
	 * update content id
	 */
	private function updateTables(){
		
		$db = JFactory::getDbo();
		
		//add addontype to addons table
		
		try{
			
			//alter table change
			$sql = "ALTER TABLE `#__addonlibrary_addons`
			add `addontype` varchar(255);";
		
			$db->setQuery($sql);
			$db->query();
						
		
		}catch(Exception $e){
			//throw $e;
			//skip errors
		}
		
		//add alias to addons table
		
		try{
			
			//alter table change
			$sql = "ALTER TABLE `#__addonlibrary_addons`
			add `alias` varchar(128);";
			
			$db->setQuery($sql);
			$db->query();
			
		}catch(Exception $e){
			//throw $e;
			//skip errors
		}
		
		//add 'name' to layouts table
		try{
			
			//alter table change
			$sql = "ALTER TABLE `#__addonlibrary_layouts`
			add `name` varchar(128);";
			
			$db->setQuery($sql);
			$db->query();
			
		}catch(Exception $e){
			//throw $e;
			//skip errors
		}
		
		try{
			
			//add layout type
			$sql = "ALTER TABLE `#__addonlibrary_layouts`
			add `layout_type` varchar(128);";
			
			$db->setQuery($sql);
			$db->query();
		}catch(Exception $e){
			//throw $e;
			//skip errors
		}
			
	}
	
	
	/**
	 * Called after any type of action
	 *
	 * @param   string  $route  Which action is happening (install|uninstall|discover_install)
	 * @param   JAdapterInstance  $adapter  The object responsible for running this script
	 *
	 * @return  boolean  True on success
	 */
	public function postflight($route, JAdapterInstance $adapter){
		
		try{
			
			$this->migrate();
			
			if($route == "update"){
				$this->updateTables();
			}
			
			$this->installAddons($adapter);
			
		}catch(Exception $e){
			
			//dmp($e->getMessage());
			//dmp($e->getTraceAsString());
			
			//skip errors
		}
		
	}	
	
	
	
	/**
	 * 
	 * install the modules from "modules" folder
	 */
	public function installModules(JAdapterInstance &$adapter,$type="install"){
		
		$ds = "";
		if(defined("DIRECTORY_SEPARATOR"))
			$ds = DIRECTORY_SEPARATOR;
		else
			$ds = DS;
		
		$manifest = $adapter->get("manifest");
		
		$installer = new JInstaller();
		$p_installer = $adapter->getParent();
		
		// Install modules
		if (is_object($manifest->modules->module)){	
			foreach($manifest->modules->module as $module){
				$attributes = $module->attributes();
				$modulePath = $p_installer->getPath("source") . $ds . $attributes['folder'] . $ds . $attributes['module'];
				
				if($type == "install")
					$installer->install($modulePath);
				else 
					$installer->update($modulePath);
			}
		}
		
	}

	/**
	 * 
	 * install the plugins from "plugins" folder
	 */
	public function installPlugins(JAdapterInstance &$adapter,$type="install"){
		
		$ds = "";
		if(defined("DIRECTORY_SEPARATOR"))
			$ds = DIRECTORY_SEPARATOR;
		else
			$ds = DS;
		
		$manifest = $adapter->get("manifest");
		
		$installer = new JInstaller();
		$p_installer = $adapter->getParent();
		
		//Install plugins
		if (is_object($manifest->plugins->plugins)){	
			foreach($manifest->plugins->plugin as $plugin){
				$attributes = $plugin->attributes();
				$pluginPath = $p_installer->getPath("source") . $ds . $attributes['folder'] . $ds . $attributes['plugin'];
				
				if($type == "install")
					$installer->install($pluginPath);
				else 
					$installer->update($pluginPath);
			}
		}
		
	}
	
	
	/**
	 * 
	 * install the templates from "templates" folder
	 */
	public function installTemplates(JAdapterInstance &$adapter,$type="install"){
		
		$ds = "";
		if(defined("DIRECTORY_SEPARATOR"))
			$ds = DIRECTORY_SEPARATOR;
		else
			$ds = DS;
		
		$manifest = $adapter->get("manifest");
		
		$installer = new JInstaller();
		$p_installer = $adapter->getParent();
		
		// Install templates
		if (is_object($manifest->templates->template)){	
			foreach($manifest->templates->template as $template){
				$attributes = $template->attributes();
				$templatePath = $p_installer->getPath("source") . $ds . $attributes['folder'] . $ds . $attributes['template'];
				
				if($type == "install")
					$installer->install($templatePath);
				else 
					$installer->update($templatePath);
			}
		}
	
	}
	
	
	/**
	 * delete uninsatll files of addon library
	 */
	private function migrate_deleteUninstallFiles(){
		$filepath = JPATH_ADMINISTRATOR."/components/com_addonlibrary/provider/sql/uninstall.sql";
		
		if(file_exists($filepath))
			@unlink($filepath);
	}
	
	/**
	 * unpublish addon library
	 */
	private function migrate_upublishLibrary(){
		
		$db = JFactory::getDbo();
		
		//unpublish plugin
		$sql = "UPDATE `#__extensions` SET `enabled`=0 where (`type`=\"plugin\" and `element`=\"addonlibrary\")";
		
		$db->setQuery($sql);
		$db->query();
		
		//unpublish creator plugin
		$sql = "UPDATE `#__extensions` SET `enabled`=0 where (`type`=\"plugin\" and `element`=\"addonlibrarycreator\")";
		
		$db->setQuery($sql);
		$db->query();
		
		
		//unpublish module
		$sql = "UPDATE `#__extensions` SET `enabled`=0 where (`type`=\"module\" and `element`=\"mod_addonlibrary\")";
		$db->setQuery($sql);
		$db->query();
		
		//unpublish component
		$sql = "UPDATE `#__extensions` SET `enabled`=0 where (`type`=\"component\" and `element`=\"com_addonlibrary\")";
		$db->setQuery($sql);
		$db->query();
				
		
		//rename modules
		
		$sql = "UPDATE `#__modules` SET `module`=\"mod_blox\" where (`module`=\"mod_addonlibrary\")";
		$db->setQuery($sql);
		$db->query();
		
		//replace component menus
		$sql = "UPDATE `#__menu` SET `link`=replace(link,\"com_addonlibrary&view\", \"com_blox&view\") where published=1 ";
		$db->setQuery($sql);
		$db->query();
		
	}
	
	
	/**
	 * move assets from addonlibrary to blox
	 */
	private function migrate_moveAssets(){
		
		$folderSource = JPATH_SITE."/media/com_addonlibrary/assets/";
		$isDir = is_dir($folderSource);
		
		if(is_dir($folderSource) == false)
			return(false);
		
		$folderDest = JPATH_SITE."/media/com_blox/assets/";
		
		if(is_dir($folderDest)){
			
			//check if there is folders or not
			$arrFiles = scandir($folderDest);
			if(!empty($arrFiles) && count($arrFiles) > 3)
				return(false);
			
			@unlink($folderDest."index.html");
			@rmdir($folderDest);
		}
		
		@rename($folderSource, $folderDest);
		
		@mkdir($folderSource);
	}
	
	
	/**
	* migrate from addon library
	*/
	private function migrate(){
				
		$this->migrate_deleteUninstallFiles();
				
		$this->migrate_upublishLibrary();
				
		$this->migrate_moveAssets();
		
	}
	
	
	/**
	 * Called on installation
	 *
	 * @param   JAdapterInstance  $adapter  The object responsible for running this script
	 *
	 * @return  boolean  True on success
	 */
	public function install(JAdapterInstance $adapter){
				
		$this->installModules($adapter,"install");
		$this->installPlugins($adapter,"install");
		
		//$this->installTemplates($adapter,"install");
		
		$this->publishPlugin();		
	}
 
	
	/**
	 * Called on update
	 *
	 * @param   JAdapterInstance  $adapter  The object responsible for running this script
	 *
	 * @return  boolean  True on success
	 */
	public function update(JAdapterInstance $adapter){
				
		$this->installModules($adapter,"update");
		$this->installPlugins($adapter,"update");
		$this->installTemplates($adapter,"update");
		
		$this->publishPlugin();
	}

	
	/**
	 * Called on uninstallation
	 *
	 * @param   JAdapterInstance  $adapter  The object responsible for running this script
	 */
	public function uninstall(JAdapterInstance $adapter){
		
	}
}

?>