/**
 * history object
 */
function UniteCreatorHistory(){
	
	var t = this;
	var g_arrData = [], g_objGrid;
	var g_temp = {
			max_items:30,
			handle:null,
			keyupTrashold:500,
			show_debug:true
	};
	
	var g_vars = {
		ACTION_ADD_ELEMENT: "add_element",
		ACTION_DELETE_ELEMENT: "delete_element",
		ACTION_ORDER_ELEMENTS: "order_elements",
		ACTION_GO_DEEPER: "go_deeper",
		ACTION_UPDATE_ELEMENT_SETTINGS: "update_element_settings",
		ACTION_UPDATE_ADDON: "update_addon",
		
		TYPE_ROW: "row",
		TYPE_CONTAINER: "container",
		TYPE_COLUMN: "col",
		TYPE_ADDON: "addon",
		TYPE_GRID: "grid",
	};
	
	if (!g_ucAdmin)
		var g_ucAdmin = new UniteAdminUC();
	
	
	function _________GENERAL__________(){}
	
	
	/**
	 * create action
	 */
	function createObjAction(name, objElement, params){
		
		var objAction = {};
		objAction["name"] = name;
		objAction["element"] = objElement;
		
		if(params)
			jQuery.extend(objAction, params);
		
		return(objAction);
	}
	
	
	/**
	 * run function with trashold
	 */
	function runWithTrashold(func){
		
		if(g_temp.handle)
			clearTimeout(g_temp.handle);
		
		g_temp.handle = setTimeout(function(){
			func();
		}, g_temp.keyupTrashold);
		
	};
	
	
	function _________BASIC_GETTERS__________(){}
	
	/**
	 * get last data
	 */
	function getData_lastOperation(){
		
		if(!g_arrData)
			return(null);
		
		if(g_arrData.length <= 1)
			return(null);
		
		var numItems = g_arrData.length;
		
		var objData = g_arrData[numItems-2];
				
		return(objData);
	}
	
	
	
	/**
	 * get element id
	 */
	function getElementID(element){
		
		var id = g_ucAdmin.getVal(element, "id");
		
		if(!id){
			trace(element);
			throw new Error("element don't have ID");
		}
				
		return(id);
	}
	
	
	/**
	 * get indexes array with id as a key from elements
	 */
	function getArrIndexesAssoc(arrElements){
		
		var arrIndexes = {};
		
		if(!arrElements)
			return(arrIndexes);
		
		if(arrElements.length == 0)
			return(arrElements);
		
		for(var index in arrElements){
			
			var element = arrElements[index];
			
			if(typeof index == "string")
				index = Number(index);
			
			var id = getElementID(element);
			
			arrIndexes[id] = index;
		}
		
		return(arrIndexes);
	}
	
	/**
	 * get element id's
	 */
	function getArrElementIDs(arrElements){
		
		var arrIDs = [];
		for(var index in arrElements){
			
			var objElement = arrElements[index];
			var id = getElementID(objElement);
			arrIDs.push(id);
		}
		
		return(arrIDs);
	}
	
	/**
	 * find element sibling id's, before and after the elements in elements array
	 */
	function getElementSibglingIDs(objElement, arrElements){
		
		var siblings = {};
		
		var arrIDs = getArrElementIDs(arrElements);
		
		if(arrIDs.length == 0)
			return(null);
		
		var elementID = getElementID(objElement);
		
		var index = jQuery.inArray(elementID, arrIDs);
		
		if(index > 0)
			siblings.prevElementID = arrIDs[index-1];
		
		if(index < (arrIDs.length-1))
			siblings.nextElementID = arrIDs[index+1];
		
		return(siblings);
	}
	
	/**
	 * get data children name by type
	 */
	function getDataChildrenName(type){
		
		switch(type){
			case g_vars.TYPE_GRID:
				return("rows");
			break;
			case g_vars.TYPE_ROW:
				return("containers");
			break;
			case g_vars.TYPE_CONTAINER:
				return("cols");
			break;
			case g_vars.TYPE_COLUMN:
				return("addon_data");
			break;
			case g_vars.TYPE_ADDON:
				return(null);
			break;
			default:
				throw new Error("getDataChildrenName error: Wrong data type:"+type);
			break;
		}
		
	}
	
	/**
	 * get settings name by element type
	 */
	function getSettingsNameByType(type){
		
		switch(type){
			case g_vars.TYPE_GRID:
			case g_vars.TYPE_ADDON:
				return("options");
			break;
		}
		
		return("settings");
	}
	
	/**
	 * get children type
	 */
	function getChildrenType(type){
		
		switch(type){
			case g_vars.TYPE_GRID:
				return(g_vars.TYPE_ROW);
			break;
			case g_vars.TYPE_ROW:
				return(g_vars.TYPE_CONTAINER);
			break;
			case g_vars.TYPE_CONTAINER:
				return(g_vars.TYPE_COLUMN);
			break;
			case g_vars.TYPE_COLUMN:
				return(g_vars.TYPE_ADDON);
			break;
			case g_vars.TYPE_ADDON:
				return(null);
			break;
			default:
				throw new Error("getChildrenType error: Wrong children type of:"+type);
			break;
		}
		
		
	}
	
	function ______________FIND_NEXT_ACTION________________(){}
	
	
	/**
	 * find first missing element
	 */
	function findFirstMissingElement(elementsSource, elementsDest){
		
		var arrIndexesAssocSource = getArrIndexesAssoc(elementsSource);
			
		for(var index in elementsDest){
			
			var element = elementsDest[index];
			var id = getElementID(element);
			
			//if id not exists - return the missing element
			var isIDExists = arrIndexesAssocSource.hasOwnProperty(id);
			if(isIDExists == false)
				return(element);
		}
		
		return(null);
	}
	
	
	/**
	 * find first not needed element (excess)
	 */
	function findFirstNotNeededElement(elementsSource, elementsDest){
		
		var arrDif = _.differenceBy(elementsSource, elementsDest, "id");
		
		if(_.isEmpty(arrDif))
			return(null);
		
		var element = arrDif[0];
		
		return(element);
	}
	
	
	/**
	 * check if match order between 2 sources
	 */
	function isOrderMatch(elementsSource, elementsDest){
		
		if(elementsSource.length != elementsDest.length)
			return(false);
		
		for(var index in elementsSource){
			var sourceID = elementsSource[index].id;
			var destID = elementsDest[index].id;
			
			if(sourceID !== destID)
				return(false);
		}
		
		return(true);
	}
	
	
	/**
	 * find different elements
	 */
	function findDifferentElements(elementsSource, elementsDest){
		
		if(elementsSource.length == 0)
			return(null);
		
		for(var index in elementsSource){
			
			var elementSource = elementsSource[index];
			var elementDest = elementsDest[index];
			
			var isEqual = _.isEqual(elementSource, elementDest);
						
			if(isEqual == true)
				continue;
			
			var response = {
				"dest":elementDest,
				"source":elementSource,
				"type":elementSource.type
			};
			
			return(response);
		}
		
		return(null);
	}
	
	
	/**
	 * compare elements, assuming that they not equal
	 */
	function compareElements(dataSource, dataDest, type, parent){
				
		//---- compare settings
		
		var settingsKey = getSettingsNameByType(type);
		
		var settingsSource = g_ucAdmin.getVal(dataSource, settingsKey);
		var settingsDest = g_ucAdmin.getVal(dataDest, settingsKey);
			
		var isEqual = _.isEqual(settingsSource, settingsDest);
		if(isEqual == false){
			
			var objAction = createObjAction(g_vars.ACTION_UPDATE_ELEMENT_SETTINGS, dataDest, {type: type});
			return(objAction);
		}
		
		//if it's addons, compare addon config
		
		if(type == g_vars.TYPE_ADDON){
			
			var isEqualAddonData = _.isEqual(dataSource, dataDest);
			if(isEqualAddonData == false){
				
				var params = {type: type};
				params.source_addonid = dataSource.id
				
				var objAction = createObjAction(g_vars.ACTION_UPDATE_ADDON, dataDest, params);
				return(objAction);
			}
			
		}
		
		//---- get children -----
		
		var dataChildrenName = getDataChildrenName(type);
		var typeChildren = getChildrenType(type);
		
		g_ucAdmin.validateObjProperty(dataSource, dataChildrenName, type+" children");
		g_ucAdmin.validateObjProperty(dataDest, dataChildrenName, type+" children");
		
		var arrChildrenSource = dataSource[dataChildrenName];
		var arrChildrenDest = dataDest[dataChildrenName];
		
		//compare if the elements are equal
		var isEqual = _.isEqual(arrChildrenSource, arrChildrenDest);
		if(isEqual == true)
			return(null);
		
		
		//--- check missing element - for add element action
		
		var objMissingElement = findFirstMissingElement(arrChildrenSource, arrChildrenDest);
		 
		if(objMissingElement){
			
			var params = getElementSibglingIDs(objMissingElement, arrChildrenDest);
			params.type = objMissingElement.type;
			params.parentID = parent;
			
			var objAction = createObjAction(g_vars.ACTION_ADD_ELEMENT, objMissingElement, params);
			
			return(objAction);
		}
		
		//--- check for not needed element - for delete element action
		 
		var objNotNeededElement = findFirstNotNeededElement(arrChildrenSource, arrChildrenDest);
		if(objNotNeededElement){
			
			var params = {
				type: objNotNeededElement.type
			};
			
			var objAction = createObjAction(g_vars.ACTION_DELETE_ELEMENT, objNotNeededElement);
			
			return(objAction);
		}
		
		//--- check for unsorted elements
		
		var isMatch = isOrderMatch(arrChildrenSource, arrChildrenDest);
		
		if(isMatch == false){
		 	
			var arrIDs = getArrElementIDs(arrChildrenDest);
			var params = {
				type: typeChildren
			};
			
			var objAction = createObjAction(g_vars.ACTION_ORDER_ELEMENTS, arrIDs, params);
			
			return(objAction);
		}
		
		//---- check for changes inside the elements. The elements are ordered now
		 
		var objDifferent = findDifferentElements(arrChildrenSource, arrChildrenDest);
		
		if(objDifferent){
			
			var parentID = g_ucAdmin.getVal(objDifferent.source, "id");
			
			var params = {parent: parentID};
			
			var objAction = createObjAction(g_vars.ACTION_GO_DEEPER, objDifferent, params);
									
			return(objAction);
		}
		
			
		return(null);
		
		
	}
	
	
	/**
	 * find next action
	 */
	function findNextAction(dataSource, dataDest, type, parent){
		
		if(!parent)
			var parent = null;
		
		var objAction = compareElements(dataSource, dataDest, type, parent);
				
		if(!objAction)
			return(null);
				
		if(objAction.name == g_vars.ACTION_GO_DEEPER){
			 
			if(g_temp.show_debug == true){
				trace("action: " + objAction.name);
				trace(objAction);			
			}
			
			var objAction = findNextAction(objAction.element.source, objAction.element.dest, objAction.element.type, objAction.parent);
			
			if(!objAction)
				throw new Error("Should be some action!");
						
			return(objAction);
		}
		
		
		return(objAction);
	}
		
	
	function ______________OPERATIONS________________(){}
	
	/**
	 * add grid data
	 */
	function addGridData(){
		
		var objData = g_objGrid.getGridData();
		
		//make sure it's a clone
		objData = jQuery.extend({}, objData);
		
		//compare with last item, allow only changes
		var lastData = _.last(g_arrData);
		
		var isEqualToLast = _.isEqual(lastData, objData);
		if(isEqualToLast == true)
			return(false);
		
		g_arrData.push(objData);
		
		if(g_arrData.length > g_temp.max_items)
			g_arrData.shift();
		
	}
	
	
	/**
	 * do action - add row
	 */
	function doAction_addElement(objAction){
		
		var type = objAction.type;
		var parentID = g_ucAdmin.getVal(objAction, "parentID");
		
		var elementData = objAction.element;
		
		var objSibling = null;
		var siblingDirection = null;
		var objParent = null;
		
		if(parentID)
			objParent = jQuery("#"+parentID);
		
		var prevID = g_ucAdmin.getVal(objAction, "prevElementID");
		
		if(prevID){
			var objSibling = jQuery("#"+prevID);
			if(objSibling.length == 0)
				objSibling = null;
			else
				var siblingDirection = "after";
		}
		
		if(!objSibling){
			
			var nextID = g_ucAdmin.getVal(objAction, "nextElementID");
			
			if(nextID){
				var objSibling = jQuery("#"+nextID);
				if(objSibling.length == 0)
					objSibling = null;
				else
					var siblingDirection = "before";
			}
		}
		
		if(!objSibling)
			siblingDirection = null;
		
		switch(type){
			case g_vars.TYPE_ROW:
				
				g_objGrid.initByData_row(elementData, objSibling, siblingDirection);
				
			break;
			case g_vars.TYPE_CONTAINER:
				 				
				if(!objSibling || parentID)
					g_objGrid.initByData_container(elementData, objParent, null);		//add by parent
				else
					g_objGrid.initByData_container(elementData, objSibling, siblingDirection);		//add by sibling

			break;
			case g_vars.TYPE_COLUMN:
				
				g_objGrid.initByData_col(elementData, objParent, objSibling, siblingDirection);		//add by sibling
				
			break;
			case g_vars.TYPE_ADDON:
								
				g_objGrid.addColAddon(objParent, elementData, true, null, objSibling, siblingDirection);
				
			break;
			default:
				throw new Error("doAction_addElement error: type not found: "+type);
			break;
		}
						
	}
	
	
	/**
	 * delete row
	 */
	function doAction_deleteElement(elementData){
		
		var type = elementData.type;
		
		var elementID = elementData.id;
		var objElement = jQuery("#"+elementID);
		
		g_ucAdmin.validateDomElement(objElement, type);
		
		switch(type){
			case g_vars.TYPE_ROW:
				
				g_objGrid.deleteRow(objElement);
				 
			break;
			case g_vars.TYPE_CONTAINER:
				
				g_objGrid.deleteRowContainer(objElement);
				
			break;
			case g_vars.TYPE_COLUMN:
				
				g_objGrid.deleteCol(objElement);
				
			break;
			case g_vars.TYPE_ADDON:
				
				g_objGrid.deleteColAddon(objElement);
				
			break;
			default:
				throw new Error("doAction_deleteElement error. Wrong type: "+type);
			break;
		}
		
	}
	
	/**
	 * order elements
	 */
	function doAction_orderElements(arrIDs, type){
		
		switch(type){
			case g_vars.TYPE_ROW:
				g_objGrid.orderRowsByIDs(arrIDs);
			break;
			case g_vars.TYPE_CONTAINER:
				
				g_objGrid.orderRowContainersByIDs(arrIDs);
								
			break;
			case g_vars.TYPE_COLUMN:
				
				g_objGrid.orderColumnsByIDs(arrIDs);
				
			break;
			default:
				throw new Error("doAction_orderElements error. Wrong type: "+type);
			break;		
		}
				
	}
	
	/**
	 * update grid settings
	 */
	function doAction_updateElementSettings(elementData, type){
		 
		if(type != g_vars.TYPE_GRID){
			var objElement = jQuery("#"+elementData.id);
			g_ucAdmin.validateDomElement(objElement, type);
		}
		
		//get settings object
		var settingsKey = getSettingsNameByType(type);
		var settings = g_ucAdmin.getVal(elementData, settingsKey); 
			
		
		switch(type){
			case g_vars.TYPE_GRID:		//in case of grid, elementData is the settings
								
				g_objGrid.applyGridSettingsFromData(settings);
			break;
			case g_vars.TYPE_ROW:
								
				g_objGrid.updateRowSettings(objElement, settings);
				
			break;
			case g_vars.TYPE_CONTAINER:				
				g_objGrid.updateContainerSettings(objElement, settings);
								
			break;
			case g_vars.TYPE_COLUMN:
				g_objGrid.updateColSettings(objElement, settings);
			break;
			case g_vars.TYPE_ADDON:
				
				g_objGrid.updateAddonContainerSettings(objElement, settings, true);
				
			break;
			default:
				throw new Error("doAction_updateElementSettings, wrong settings type: "+type);
			break;
		}
		
	}
	
	/**
	 * update addon
	 */
	function doAction_updateAddon(objAction){
		
		var sourceID = objAction.source_addonid;
		var objAddon = jQuery("#"+sourceID);
		var addonData = objAction.element;
		
		g_ucAdmin.validateDomElement(objAddon, "source addon");
		
		g_objGrid.redrawAddonByData(objAddon, addonData);
		
	}
	
	
	/**
	 * do action
	 */
	function doAction(objAction){
				
		if(!objAction)
			throw new Error("error action passed");
				
		var actionName = objAction.name;
		var elementData = objAction.element;
		var type = g_ucAdmin.getVal(objAction, "type");
		
		if(g_temp.show_debug == true){
			trace("do action");
			trace(actionName);
			trace(objAction);
		}
		
		switch(actionName){
		
			case g_vars.ACTION_DELETE_ELEMENT:
				
				doAction_deleteElement(elementData);
			break;
			case g_vars.ACTION_ORDER_ELEMENTS:
				
				doAction_orderElements(elementData, type);
				
			break;
			case g_vars.ACTION_UPDATE_ELEMENT_SETTINGS:
				
				doAction_updateElementSettings(elementData, type);
				
			break;
			case g_vars.ACTION_ADD_ELEMENT:
				
				doAction_addElement(objAction);
				
			break;
			case g_vars.ACTION_UPDATE_ADDON:
				
				doAction_updateAddon(objAction);
				
			break;
			default:
				throw new Error("Wrong action: " + actionName);
			break;
		}
		
	}
	
	/**
	 * return the grid to the dest data
	 */
	function returnToDest(dataDest){
		
		var dataSource = g_objGrid.getGridData();
				
		var objNextAction = findNextAction(dataSource, dataDest, g_vars.TYPE_GRID, null);
		
		if(!objNextAction){
			trace("no action");
			return(false);
		}
		
		doAction(objNextAction);
		
	}
	
	
	/**
	 * do the undo action
	 */
	this.undo = function(){
		
		var dataDest = getData_lastOperation();
		
		if(!dataDest)
			return(false);
		
		returnToDest(dataDest);
		
		g_objGrid.refreshSettingsPanel();
		
	}
	
	function ______________EVENTS________________(){}
	
	/**
	 * on change taken
	 */
	function onChangeTaken(){
		
		runWithTrashold(addGridData);
	}
	
	
	/**
	 * init events
	 */
	function initEvents(){
				
		g_objGrid.onEvent(g_objGrid.events.CHANGE_TAKEN, onChangeTaken);
		
	}
	
	
	/**
	 * init grid builder
	 */
	this.initGrid = function(gridBuilder){
		
		g_objGrid = gridBuilder;
		
		initEvents();
		
		addGridData();
	};
	
	
}
