function UniteCreatorGridBuilder() {

	this.g_objGrid = null; 
	this.g_objStyle = null;
	this.g_optionsCustom = {};
	this.g_options = null;
	this.g_objWrapper = null;
	this.g_objPageBuilderWrapper = null;
	this.g_objBodyParent = null;
	this.g_pageBuilder = null;
	
	var g_objBrowser, g_objBrowserSections; // addons
	
	this.g_objGridData = new UniteCreatorGridBuilder_gridData();
	this.g_objAddonHTML = new UniteCreatorGridBuilder_addonHTML();
	this.g_objDragDrop = new UniteCreatorGridBuilder_dragDrop();
	this.g_panel = null;
	
	
	// browser
	var g_objRowStyleContainer, g_objColStyleContainer;
	var g_gridID, g_objSettingsGrid = new UniteSettingsUC();
	var g_objAddonConfig = new UniteCreatorAddonConfig();
	var g_objSettingsRow = new UniteSettingsUC(); 
	var g_objDialogRowSettings, g_objHistory = new UniteCreatorHistory();
	var g_objBuffer = null, g_objScroll;
	
	var t = this;
	
	this.g_vars = {
		class_col : "uc-grid-col",
		class_first_col : "uc-col-first",
		class_last_col : "uc-col-last",
		class_empty : "uc-col-empty",
		class_first_row : "uc-row-first",
		class_last_row : "uc-row-last",
		class_size_prefix : "uc-colsize-",
		class_element_hover: "uc-element-hover",
		id_prefix : "uc_addon_",
		addon_conetentid_prefix : "uc_addon_contentid",
		max_cols : 6,
		serial : 0, // give serial numbers to addons
		shape_deviders : null
	};

	this.g_temp = {
		init_counter : 0,
		is_live_view : false,
		indev : false,
		is_data_inited : false,
		is_drag_mode: false,
		page_params : {},
		google_fonts : {},
		icons : {}, // settings that hold delay before change
		settings_delay_row : [ "row_container_width", "col_gutter",
				"row_padding_top", "row_padding_bottom",
				"space_between_addons", "padding_top_mobile",
				"padding_bottom_mobile", "row_class", "row_css",
				"row_container_css", "container_css" ],
		settings_placeholders_grid : [ "row_padding_top", "row_padding_bottom",
				"col_gutter", "space_between_addons" ],
		settings_placeholders : [ "padding_top", "padding_bottom",
				"padding_left", "padding_right", "margin_top", "margin_bottom",
				"margin_left", "margin_right", "shape_devider_top_height",
				"shape_devider_bottom_height" ],
		sizes : [ "tablet", "mobile" ],
		addontype_bgaddon : "bg_addon",
		value_empty_array : "[[uc_empty_array]]"
	};

	this.events = {
		ROW_COLUMNS_UPDATED : "ROW_COLUMNS_UPDATED",
		ROWS_UPDATED : "ROWS_UPDATED", // add, remove, reorder row
		ROW_ADDED : "ROW_ADDED",
		COL_ADDED : "COL_ADDED",
		COL_ADDONS_UPDATED : "ADDONS_UPDATED", // updated col addons

		ROW_CONTAINER_DUPLICATED : "CONTAINER_DUPLICATED",
		ROW_CONTAINER_ADDED : "CONTAINER_ADDED", // updated containers
													// (sort,delete,remove,add)
		ROW_CONTAINERS_UPDATED : "CONTAINERS_UPDATES", // updated containers
														// (sort,delete,remove,add)
		ROW_CONTAINER_SETTINGS_UPDATED : "CONTAINER_SETTINGS_UPDATED", // updated
																		// container
																		// settings

		BEFORE_REMOVE_ADDON : "BEFORE_REMOVE_ADDON",
		ELEMENT_REMOVED : "ELEMENT_REMOVED", // on some addons removed
		ADDON_ADDED : "ADDON_ADDED", // before duplicated addon data inserted
		ADDON_DUPLICATED : "ADDON_DUPLICATED", // before duplicated addon data
												// inserted
		COL_DUPLICATED : "COL_DUPLICATED", // before duplicated column data
											// added
		ROW_DUPLICATED : "ROW_DUPLICATED", // before duplicated row data added
		GRID_SETTINGS_UPDATED : "GRID_SETTINGS_UPDATED", // on row settings
															// updated
		PAGE_PARAMS_UPDATED : "PAGE_PARAMS_UPDATED",
		ROW_SETTINGS_UPDATED : "ROW_SETTINGS_UPDATED", // on row settings
														// updated
		COL_SETTINGS_UPDATED : "COL_SETTINGS_UPDATED", // on col settings
														// updated
		ADDON_CONTAINER_SETTINGS_UPDATED : "ADDON_CONTAINER_SETTINGS_UPDATED", // on
																				// addon
																				// container
																				// updated
		ADDON_SETTINGS_UPDATED : "ADDON_SETTINGS_UPDATED", // on addon settings
															// updated
		CHANGE_TAKEN : "CHANGE_TAKEN", // on some sort change taken (enable
										// save)
		DATA_INITED : "DATA_INITED", // when the grid data inited
		BODY_CLICK : "BODY_CLICK"
	};

	if (!g_ucAdmin)
		var g_ucAdmin = new UniteAdminUC();

	this.____________GENERAL______________ = function() {}

	/**
	 * get icon, check that it's exists
	 */
	this.getIcon = function(name) {

		var icon = g_ucAdmin.getVal(t.g_temp.icons, name);

		if (!icon) {
			trace("those are available icons: ");
			trace(t.g_temp.icons);
			throw new Error("icon + " + name + " not found");
		}

		return (icon);
	}

	/**
	 * get element type - column, addon, row, undefined
	 */
	this.getElementType = function(element) {

		if (!element)
			return ("undefined");

		if (element.hasClass("uc-grid-col"))
			return ("column");

		if (element.hasClass("uc-grid-col-addon"))
			return ("addon");

		if (element.hasClass("uc-grid-row"))
			return ("row");

		if (element.hasClass("uc-grid-row-container"))
			return ("container");

		if (element.hasClass("uc-grid-builder"))
			return ("grid");

		if (element.hasClass("uc-grid-bg-addon"))
			return ("bgaddon");

		return ("undefined");
	}

	/**
	 * get element list that the given element
	 */
	this.getBrothersElements = function(objElementInTheList, type) {

		if (!(type))
			var type = t.getElementType(objElement);

		if (type == "undefined")
			throw new Error("getBrothersElements: the type can't be undefined");

		switch (type) {
		case "row":
			var objRows = t.getRows();
			return (objRows);
			break;
		case "container":
			var objParentRow = t.getParentRow(objElement);
			var objContainers = t.getRowContainersAll(objParentRow);
			return (objContainers);
			break;
		case "column":
			var objParentContainer = t.getParentRowContainer(objElement);
			var objColumns = t.getCols(objParentContainer);
			return (objColumns);
			break;
		case "addon":
			var objParentColumn = t.getParentCol(objElement);
			var objAddons = t.getColAddons(objParentColumn);
			return (objAddons);
			break;
		default:
			throw new Error("wrong object type: " + type);
			break;
		}

	}

	/**
	 * get previous element
	 */
	this.getPrevElement = function(objElement, type) {

		if (!(type))
			var type = t.getElementType(objElement);

		var objElements = t.getBrothersElements(objElement, type);
		var index = objElement.index();
		if (index == 0)
			return (null);

		var objElementBefore = jQuery(objElements[index - 1]);

		return (objElementBefore);
	}

	/**
	 * get previous element
	 */
	this.getNextElement = function(objElement, type) {

		if (!(type))
			var type = t.getElementType(objElement);

		var objElements = t.getBrothersElements(objElement, type);
		var index = objElement.index();
		var numElements = objElements.length;

		if (index >= (numElements - 1))
			return (null);

		var objElementAfter = jQuery(objElements[index + 1]);

		return (objElementAfter);
	}

	/**
	 * get the parent of the element
	 */
	this.getElementParent = function(objElement, type) {

		if (!(type))
			var type = t.getElementType(objElement);

		switch (type) {
		case "grid":
			return (null);
			break;
		case "row":

			return (t.g_objGrid);
			break;
		case "container":
			var objRow = t.getParentRow(objElement);
			return (objRow);
			break;
		case "column":
			var objContainer = t.getParentRowContainer(objElement);
			return (objContainer);
			break;
		case "addon":
			var objCol = t.getParentCol(objElement);
			return (objCol);
			break;
		default:
			throw new Error("Wrong element type: " + type);
			break;
		}

	}

	/**
	 * do grid action
	 */
	this.doGridAction = function(action, parentRow, params) {

		switch (action) {
		case "open_grid_settings":
			t.openGridSettingsPanel();
			break;
		case "open_page_params_panel":
			t.openPageParamsPanel();
			break;
		case "add_row":
			t.addRow(parentRow);
			break;
		case "to_view_mode": // no extras mode
			t.g_objGrid.addClass("uc-view-mode");
			break;
		case "to_regular_mode": // no extras mode
			t.g_objGrid.removeClass("uc-view-mode");
			break;
		case "play_panelobject_animation":
			t.playPanelObjectAnimation();
			break;
		case "play_panelobject_section_animation":
			t.playPanelObjectSectionAnimation();
			break;
		case "undo":
			t.undo();
			break;
		default:
			throw new Error("Wrong grid action: " + action);
			break;
		}

	}

	/**
	 * get element addons (grid, row, col)
	 */
	this.getElementAddons = function(objElement) {

		var objAddons = objElement.find(".uc-grid-col-addon");
		
		return (objAddons);
	}

	/**
	 * show error message from the parent object
	 */
	this.showErrorMessage = function(message) {

		if (t.g_pageBuilder)
			t.g_pageBuilder.showErrorMessage(message);
		else {

			if (t.g_objWrapper && t.g_objWrapper.length) {
				t.g_objWrapper.html("<div class='unite_error_message'>" + message
						+ "</div>");
			} else
				alert(message);
		}

	}

	/**
	 * add mobile placeholder for some object (row,column,addon)
	 */
	this.addMobilePlaceholder = function(objPlaceholders, mobileName,
			parentName, objParentSettings, objTopParentSettings, defaultValue) {

		var parentSettingValue = g_ucAdmin
				.getVal(objParentSettings, parentName);
		if (parentSettingValue !== "") {
			objPlaceholders[mobileName] = parentSettingValue;
			return (objPlaceholders);
		}

		var parentTopSettingValue = g_ucAdmin.getVal(objTopParentSettings,
				parentName);
		if (parentTopSettingValue !== "") {

			objPlaceholders[mobileName] = parentTopSettingValue;
			return (objPlaceholders);
		}

		if (typeof defaultValue !== "undefined")
			objPlaceholders[mobileName] = defaultValue;

		return (objPlaceholders);
	}

	/**
	 * get placeholders object for all the sizes
	 */
	this.getObjSizePlaceholders = function(arrSettings, objPlaceholders,
			objOptions, objParentOptions, prefix, defaultValue) {

		var arrSizes = t.g_temp.sizes;

		var arrParentSizeValues = {};

		for ( var indexSize in arrSizes) {

			var size = arrSizes[indexSize];

			for ( var indexSetting in arrSettings) {
				var settingName = arrSettings[indexSetting];

				var isPrefix = false;
				if (prefix)
					isPrefix = true;

				if (settingName.indexOf("noprefix_") === 0) {
					settingName = settingName.replace("noprefix_", "");
					isPrefix = false;
				}

				var settingNameSize = settingName + "_" + size;

				if (isPrefix == true)
					settingName = prefix + "_" + settingName;

				objPlaceholders = t.addMobilePlaceholder(objPlaceholders,
						settingNameSize, settingName, objOptions,
						objParentOptions, defaultValue);

				// take placeholder from parent size if available (mobile from
				// tablet)
				var parentValue = g_ucAdmin.getVal(arrParentSizeValues,
						settingName);
				if (parentValue !== "")
					objPlaceholders[settingNameSize] = parentValue;

				// save parent setting size for child size values placeholders
				var sizeValue = g_ucAdmin.getVal(objOptions, settingNameSize);
				if (sizeValue !== "")
					arrParentSizeValues[settingName] = sizeValue;

			}

		}

		return (objPlaceholders);
	}

	/**
	 * get new element ID
	 */
	this.getNewElementID = function(type) {

		t.g_vars.serial++;
		var elementID = g_gridID + "_" + type + "_" + t.g_vars.serial;
		elementID = elementID.replace("#", "");

		return (elementID);
	}

	/**
	 * get element data settings key
	 */
	this.getElementDataSettingsKey = function(objElement) {

		var type = t.getElementType(objElement);

		switch (type) {
		case "addon":
			return ("addon_settings");
			break;
		case "grid":
			return ("options");
			break;
		default:
			return ("settings");
			break;
		}

	}

	/**
	 * get element settings
	 */
	this.getElementSettings = function(objElement) {

		var dataKey = t.getElementDataSettingsKey(objElement);

		var objSettings = objElement.data(dataKey);
		if (!objSettings)
			objSettings = {};

		return (objSettings);
	}
	
	
	
	/**
	 * update element setting (not addon data)
	 */
	this.updateElementSetting = function(objElement, settingName, settingValue) {

		var dataKey = t.getElementDataSettingsKey(objElement);

		var objSettings = objElement.data(dataKey);
		if (!objSettings)
			objSettings = {};

		objSettings[settingName] = settingValue;
		
		objElement.data(dataKey, objSettings);

		t.updatePanelSettingIfActive(objElement, settingName, settingValue);
	}
	
	
	/**
	 * update panel settings if active
	 */
	this.updatePanelSettingIfActive = function(objElement, settingName, settingValue, isAddonSettings) {
		
		var elementType = t.getElementType(objElement);
		if (elementType == "column")
			elementType = "col";

		if (elementType == "addon" && isAddonSettings !== true)
			elementType = "addon-container";

		var paneName = elementType + "-settings";
		var elementID = objElement.prop("id");
		
		t.g_panel.setSettingValueIfActive(paneName, elementID, settingName, settingValue);
	}
	
	
	
	/**
	 * map section children, run some func on them.
	 */
	this.mapSectionChildren = function(objRow, func) {
		
		var objContainers = t.getRowContainersAll(objRow);
		var objColumns = t.getAllRowCols(objRow);
		var objAddons = t.getElementAddons(objRow);

		objContainers.each(function(index, container) {
			var objContainer = jQuery(container);
			func(objContainer, "container");
		});

		objColumns.each(function(index, column) {
			var objCol = jQuery(column);
			func(objCol, "column");
		});

		objAddons.each(function(index, addon) {
			var objAddon = jQuery(addon);
			func(objAddon, "addon");
		});

	}

	/**
	 * play element animation
	 */
	this.playElementAnimation = function(objElement, data, includeDelay) {

		if (objElement.length == 0)
			return (true);

		if (data)
			var settings = data.settings;
		else
			var settings = t.getElementSettings(objElement);

		var animationType = g_ucAdmin.getVal(settings, "animation_type");
		if (!animationType)
			return (true);

		var animationDelay = g_ucAdmin.getVal(settings, "animation_delay");
		var animationDuration = g_ucAdmin
				.getVal(settings, "animation_duration");
		animationDuration = parseInt(animationDuration);

		if (!animationDuration)
			animationDuration = null;

		var delay = 0;
		if (includeDelay === true && animationDelay
				&& jQuery.isNumeric(animationDelay))
			delay = parseFloat(animationDelay) * 1000;

		setTimeout(
				function() {
					var classAnimated = "animated";

					var className = animationType + ' ' + classAnimated;

					objElement.removeClass(classAnimated).addClass(className);
					if (animationDuration)
						objElement.css("animation-duration", animationDuration
								+ "s");
					else
						objElement.css("animation-duration", "");

					objElement
							.one(
									'webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend',
									function() {
										jQuery(this).removeClass(classAnimated)
												.removeClass(animationType);
										objElement
												.css("animation-duration", "");
									});

				}, delay);

	}


	/**
	 * check if bg color changed of element
	 */
	this.isElementBGColorChange = function(oldSettings, newSettings) {

		var bgColorOld = t.getElementBGColor(null, oldSettings);
		var bgColorNew = t.getElementBGColor(null, newSettings);

		if (bgColorOld !== bgColorNew)
			return (true);
		else
			return (false);
	}


	/**
	 * undo action
	 */
	this.undo = function() {
		
		g_objHistory.undo();
		
	}
	
	
	/**
	 * show debug
	 */
	this.showDebug = function(text){
		
		t.g_pageBuilder.showDebug(text);		
	}
	
	/**
	 * hide debug
	 */
	this.hideDebug = function(){
		
		t.g_pageBuilder.hideDebug();				
	}
	
	this.____________PANEL_RELATED______________ = function() {}
	
	
	/**
	 * refresh the panel data by the grid
	 */
	this.refreshSettingsPanel = function(){
		
		var isVisible = t.g_panel.isVisible();
		if(isVisible == false){
			t.g_panel.clearClosedPanel();
			return(false);
		}
		
		var panelData = t.g_panel.getActivePaneData();
		var objectID = g_ucAdmin.getVal(panelData, "objectID");
		
		var objElement = null;
		
		if(objectID){
			
			objElement = jQuery("#"+objectID);
			
			//if objet not exists - exit
			if(objElement.length == 0){
				t.g_panel.close();
				return(false);
			}
			
		}
		
		t.openElementSettingsPanel(objElement, true);
				
	}
	
	
	/**
	 * open some element settings panel
	 */
	this.openElementSettingsPanel = function(objElement, isRefresh) {
		
		var type = t.getElementType(objElement);
		
		trace("open elements panel: "+type);
		
		switch (type) {
			case "row":
				t.openRowSettingsPanel(objElement, isRefresh);
			break;
			case "container":
				t.openContainerSettingsPanel(objElement, isRefresh);
			break;
			case "addon":
				t.openAddonSettingsPanel(objElement, null, isRefresh);
				break;
			case "column":
				t.openColSettingsPanel(objElement);
			break;
			case "addon-container":
				//openAddonContainerSettingsPanel	//make it work
			break;
		}

	}
	
	
	/**
	 * play section animation
	 */
	this.playPanelObjectSectionAnimation = function() {

		var data = t.g_panel.getActivePaneData();
		var objectID = data.objectID;

		var objElement = jQuery("#" + objectID);
		var type = t.getElementType(objElement);

		if (type != "row")
			objElement = t.getParentRow(objElement);

		if (objElement.length == 0)
			return (true);

		t.mapSectionChildren(objElement, function(objElement, type) {

			t.playElementAnimation(objElement, null, true);

		});

	}

	/**
	 * play panel object animation
	 */
	this.playPanelObjectAnimation = function() {

		var data = t.g_panel.getActivePaneData();
		var objectID = data.objectID;

		var objElement = jQuery("#" + objectID);

		if (objElement.length == 0)
			return (true);

		t.playElementAnimation(objElement, data);

	}
	
	
	this.____________ROW______________ = function() {}

	/**
	 * validate row
	 */
	this.validateRow = function(objRow) {

		if (!objRow)
			throw new Error("Empty Row Found: " + objRow);

		if (objRow.hasClass("uc-grid-row") == false
				&& objRow.hasClass("uc-grid-row-container") == false)
			throw new Error("Wrong Row or container: " + objRow);

	}

	/**
	 * get row html
	 */
	this.getHtmlRow = function(noContainer) {

		var rowID = t.getNewElementID("row");
		var iconSettings = t.getIcon("settings");
		var iconDuplicate = t.getIcon("duplicate");
		var iconMenu = t.getIcon("menu_closed");

		var html = "";
		html += "<div id='" + rowID + "' class='uc-grid-row'>";

		// add shape deviders

		html += "<div class='uc-grid-shape-devider uc-devider-top'></div>";
		html += "<div class='uc-grid-shape-devider uc-devider-bottom'></div>";

		html += "<div class='uc-grid-row-anchor'></div>";
		html += "<div class='uc-grid-row-anchor-indicator uc-grid-extras' title='"
				+ g_uctext.section_anchor + "'></div>";
		
		html += "<div class='uc-grid-row-hover uc-hover-top uc-grid-extras'></div>";
		html += "<div class='uc-grid-row-hover uc-hover-bottom uc-grid-extras'></div>";
		html += "<div class='uc-grid-row-hover uc-hover-left uc-grid-extras'></div>";
		html += "<div class='uc-grid-row-hover uc-hover-right uc-grid-extras'></div>";

		html += "<div class='uc-grid-element-hidden-overlay'>"
				+ g_uctext.hidden_row + "</div>";
		html += "<div class='uc-grid-element-background-overlay'></div>";

		// add row panel
		html += "	<div class='uc-grid-object-panel uc-grid-row-panel uc-grid-extras'>";
		html += "		<a href='javascript:void(0)' data-action='row_settings' data-actiontype='row' title='"
				+ g_uctext.settings
				+ "' class=\"uc-row-icon uc-grid-action-icon uc-tip\" ><i class=\""
				+ iconSettings + "\" aria-hidden=\"true\"></i></a> ";
		html += "		<a href='javascript:void(0)' data-action='duplicate_row' data-actiontype='row' title='"
				+ g_uctext.duplicate_section
				+ "' class=\"uc-row-icon uc-grid-action-icon uc-tip\" ><i class=\""
				+ iconDuplicate + "\" aria-hidden=\"true\"></i></a> ";
		html += "		<a href='javascript:void(0)' data-action='delete_row' data-actiontype='row' title='"
				+ g_uctext.delete_section
				+ "' class=\"uc-row-icon uc-grid-action-icon uc-tip\" ><i class=\""
				+ t.getIcon("delete") + "\" aria-hidden=\"true\"></i></a> ";
		html += "		<a href='javascript:void(0)' title='"
				+ g_uctext.move_section
				+ "' class=\"uc-row-icon uc-grid-icon-move uc-row-icon-move uc-tip\" ><i class=\""
				+ t.getIcon("move") + "\" aria-hidden=\"true\"></i></a> ";
		html += "		<a href='javascript:void(0)' class=\"uc-row-icon uc-row-icon-menu\" ><i class=\""
				+ iconMenu + "\" aria-hidden=\"true\"></i>";
		html += "			<span data-action='copy_row' data-actiontype='row' class=\"uc-grid-action-icon\" >"
				+ g_uctext.copy_section + "</span>";
		html += "			<span data-action='paste_row' data-actiontype='row' class=\"uc-grid-action-icon uc-icon-paste-row\" >"
				+ g_uctext.paste_section + "</span>";
		html += "			<span data-action='save_row_tolibrary' data-actiontype='row' class=\"uc-grid-action-icon\" >"
				+ g_uctext.save_to_library + "</span>";
		html += "			<span data-action='import_row_fromlibrary' data-actiontype='row' class=\"uc-grid-action-icon\" >Import From Library</span>";
		html += "</a>";
		html += "	</div>";

		// add buttons panel
		html += "	<div class='uc-grid-object-panel uc-row-addbuttons-panel uc-grid-extras'>";
		html += "		<a href='javascript:void(0)' data-action='add_row' data-actiontype='grid' title='Add Section' class=\"uc-row-icon uc-grid-action-icon uc-iconcolor-row uc-tip\" ><i class=\"fa fa-plus\" aria-hidden=\"true\"></i></a> ";
		html += "		<a href='javascript:void(0)' data-action='add_row_container' data-actiontype='row' title='Add Row' class=\"uc-row-icon uc-grid-action-icon uc-iconcolor-container uc-tip\" ><i class=\"fa fa-plus\" aria-hidden=\"true\"></i></a> ";
		html += "	</div>";

		// add container
		if (noContainer !== true) {

			html += t.getHtmlRowContainer();
		}

		html += "</div>";

		return (html);
	}

	/**
	 * get row title by index or options
	 */
	this.getRowTitle = function(objRow) {

		var rowIndex = objRow.index() + 1;
		var title = g_uctext["section"] + " " + rowIndex;

		return (title);
	}

	/**
	 * add empty row insertTo - before / after
	 */
	this.addEmptyRow = function(parentRow, insertTo, noContainer) {
		
		if (!insertTo)
			insertTo = "after";

		var html = t.getHtmlRow(noContainer);

		var objRow = jQuery(html);

		if (!parentRow) {
			t.g_objGrid.append(objRow);
		} else {
			if (insertTo == "after")
				objRow.insertAfter(parentRow);
			else
				objRow.insertBefore(parentRow);
		}

		t.triggerEvent(t.events.ROW_ADDED, objRow);
		t.triggerEvent(t.events.ROWS_UPDATED);

		// if added container trigger event
		if (noContainer !== true) {

			var objContainer = t.getRowContainer(objRow);

			t.triggerEvent(t.events.ROW_CONTAINER_ADDED, objContainer);
			t.triggerEvent(t.events.ROW_CONTAINERS_UPDATED, objRow);
		}
		
		return (objRow);
	}

	/**
	 * add row to the end
	 */
	this.addRow = function(parentRow) {

		var objRow = t.addEmptyRow(parentRow);

		t.addColumn(objRow);

	}

	/**
	 * get all rows
	 */
	this.getRows = function() {

		var objRows = t.g_objGrid.children(".uc-grid-row");

		return (objRows);
	}

	/**
	 * get number of rows
	 */
	this.getNumRows = function() {

		var objRows = t.getRows();

		var numRows = objRows.length;

		return (numRows);
	}

	/**
	 * get row bu number
	 */
	this.getRow = function(num) {

		if (!num)
			var num = 0;

		var objRows = t.getRows();

		if (num >= objRows.length)
			throw new Error("getRow error: Row " + num + " don't exists");

		var objRow = jQuery(objRows[num]);

		return (objRow);
	}

	/**
	 * get parent row
	 */
	this.getParentRow = function(objChild) {

		if (objChild.hasClass("uc-grid-row"))
			return (objChild);

		var objRow = objChild.parents(".uc-grid-row");

		return (objRow);
	}

	/**
	 * get row addons
	 */
	this.getRowAddons = function(objRow) {
		t.validateRow(objRow);

		var objAddons = t.getElementAddons(objRow);

		return (objAddons);
	}

	/**
	 * get number of row addons
	 */
	this.getNumRowAddons = function(objRow) {

		var objAddons = t.getRowAddons(objRow);
		return (objAddons.length);
	}

	/**
	 * delete row
	 */
	this.deleteRow = function(objRow) {

		t.validateRow(objRow);

		var rowID = objRow.prop("id");

		objRow.remove();

		var numRows = t.getNumRows();
		if (numRows == 0)
			t.addRow(); // triggers the updated event
		else
			t.triggerEvent(t.events.ROWS_UPDATED);

		t.triggerEvent(t.events.ELEMENT_REMOVED, rowID);
	}

	/**
	 * duplicate row
	 */
	this.duplicateRow = function(objRow) {

		var rowID = t.getNewElementID("row");

		// clear containers and columns sortable
		objRow.sortable("destroy");

		var objContainers = t.getRowContainersAll(objRow);
		jQuery.each(objContainers, function(index, container) {
			var objContainer = jQuery(container);

			objContainer.sortable("destroy");
		});

		var objRowCopy = objRow.clone(true, true);

		objRowCopy.attr("id", rowID);

		objRowCopy.insertAfter(objRow);

		t.triggerEvent(t.events.ROW_DUPLICATED, objRowCopy);

		t.triggerEvent(t.events.ROWS_UPDATED);

		// init original row sortable
		t.initSortableContainers(objRow);

		jQuery.each(objContainers, function(index, container) {
			var objContainer = jQuery(container);
			t.initSortableColumns(objContainer);
		});

	}

	/**
	 * copy row - put into container
	 */
	this.copyRow = function(objRow) {

		// get data
		var objRowCopy = objRow.clone(true, true);
		var rowID = objRow.prop("id");

		var dataRow = t.g_objGridData.getGridData_row(objRow, true);

		// store row
		if (g_objBuffer)
			g_objBuffer.store("row", rowID, objRowCopy, dataRow);
	}

	/**
	 * paste row
	 */
	this.pasteRow = function(objRow) {

		var errorMessage = "Stored row not found in the buffer";

		var type = g_objBuffer.getStoredType();
		if (type != "row") {
			trace(errorMessage);
			return (false);
		}
		
		var storedData = g_objBuffer.getStoredContent();

		if (!storedData) {
			trace(errorMessage);
			return (false);
		}

		var storedRowHtml = g_ucAdmin.getVal(storedData, "content_html");
		var storedRowObject = g_ucAdmin.getVal(storedData, "content_object");

		var rowID = t.getNewElementID("row");

		if (storedRowHtml) {
			storedRowHtml.attr("id", rowID);
			storedRowHtml.insertBefore(objRow);
			t.triggerEvent(t.events.ROW_DUPLICATED, storedRowHtml);
		} else {

			t.initByData_row(storedRowObject, objRow, "before");
		}

		g_objBuffer.clear();
		t.triggerEvent(t.events.ROWS_UPDATED);

	}
	
	
	/**
	 * change order of the rows by id's
	 */
	this.orderRowsByIDs = function(arrIDs){
		
		jQuery.each(arrIDs, function(index){
			
			var rowID = arrIDs[index];
			var objRow = jQuery("#"+rowID);
			
			g_ucAdmin.validateDomElement(objRow, "row with ID: "+rowID);
			
			t.g_objGrid.append(objRow);
			
		});
		
		t.triggerEvent(t.events.ROWS_UPDATED);
		
	}
	
	
	/**
	 * do action
	 */
	this.doRowAction = function(action, objRow) {

		switch (action) {
		case "add_row":
			t.addRow(objRow);
			break;
		case "delete_row":
			t.deleteRow(objRow);
			break;
		case "duplicate_row":
			t.duplicateRow(objRow);
			break;
		case "copy_row":
			t.copyRow(objRow);
			break;
		case "paste_row":
			t.pasteRow(objRow);
			break;
		case "save_row_tolibrary":
			t.openSaveSectionSettingsPanel(objRow);
			break;
		case "import_row_fromlibrary":
			t.importSectionFromLibrary(objRow);
			break;
		case "row_settings":
			t.openRowSettingsPanel(objRow);
			break;
		case "add_row_container":
			t.addRowContainer(objRow, true);
			break;
		default:
			trace("wrong row action: " + action);
			break;
		}

	}

	this.____________UPDATE_VISUAL_GENERAL______________ = function() {
	}

	/**
	 * get shape devider bg color
	 */
	this.getDividerBGColor = function(objElement, objSettings, elementType,
			position, checkParentParam) {

		var checkParent = true;
		if (elementType == "row" && !checkParentParam)
			checkParent = false;

		var defaultBG = "#ffffff";

		if (checkParent == false) { // for rows - check brothers

			if (position == "top")
				var objSourceElement = t.getPrevElement(objElement);
			else
				var objSourceElement = t.getNextElement(objElement);

			if (!objSourceElement)
				return (null);

			var bgColor = t.getElementBGColor(objSourceElement);
			if (!bgColor)
				bgColor = defaultBG;

			return (bgColor);

		} else { // check parent

			var objParentElement = t.getElementParent(objElement);

			if (!objParentElement)
				return (null);

			var bgColor = t.getElementBGColor(objParentElement);
			if (bgColor)
				return (bgColor);

			// if not found use recursion
			bgColor = t.getDividerBGColor(objParentElement, objSettings,
					elementType, position, true);
			return (bgColor);
		}

	}

	/**
	 * get current element bg color, settings - optional
	 */
	this.getElementBGColor = function(objElement, objSettings) {

		if (!objSettings) {

			if (!objElement)
				return (null);

			if (objElement.length == 0)
				return (null);

			var objSettings = t.getElementSettings(objElement);
		}

		var defaultBG = null;

		var enableBG = g_ucAdmin.getVal(objSettings, "bg_enable", true,
				g_ucAdmin.getvalopt.FORCE_BOOLEAN);

		if (enableBG == false)
			return (defaultBG);

		var bgColor = g_ucAdmin.getVal(objSettings, "bg_color", defaultBG);
		bgColor = jQuery.trim(bgColor);
		if (!bgColor)
			return (defaultBG);

		return (bgColor);
	}

	/**
	 * get element shape devider
	 */
	this.getElementShapeDivider = function(objSettings, position) {

		if (!objSettings)
			return (null);

		// keep old format if exists
		var enableShapeDivider = true;
		if (objSettings.hasOwnProperty("enable_shape_devider_" + position))
			var enableShapeDivider = g_ucAdmin.getVal(objSettings,
					"enable_shape_devider_" + position, false,
					g_ucAdmin.getvalopt.FORCE_BOOLEAN);

		if (enableShapeDivider == false) {
			return (null);
		}

		// new way - decide by shape
		var shape = g_ucAdmin.getVal(objSettings, "shape_devider_" + position
				+ "_type");

		return (shape);
	}

	/**
	 * on parent element color change, check if it could affect the shape if
	 * there is shape in this element, and autodetect is ok, then redraw shape
	 */
	this.onShapeDetectedColorParentChange = function(objElement, position) {

		var type = t.getElementType(objElement);

		var objSettings = objElement.data("settings");
		if (!objSettings)
			objSettings = {};

		var shape = t.getElementShapeDivider(objSettings, position);

		if (!shape)
			return (false);

		var isAutodetectColor = g_ucAdmin.getVal(objSettings, "shape_devider_"
				+ position + "_autodetect_color");
		isAutodetectColor = g_ucAdmin.strToBool(isAutodetectColor);

		if (isAutodetectColor == false)
			return (false);

		t.updateElementVisual_shapeDivider(objElement, objSettings, type,
				position);

	}

	/**
	 * update shape devider
	 */
	this.updateElementVisual_shapeDivider = function(objElement, objSettings,
			elementType, position) {

		var elementID = objElement.prop("id");

		var classDividerEnable = "uc-has-devider-" + position;

		var shape = t.getElementShapeDivider(objSettings, position);

		if (!shape) {
			objElement.removeClass(classDividerEnable);
			return (false);
		}

		var selectorDivider = ".uc-grid-shape-devider.uc-devider-" + position;
		var objDivider = objElement.children(selectorDivider);

		var settingNameColor = "shape_devider_" + position + "_color";

		var height = g_ucAdmin.getVal(objSettings, "shape_devider_" + position
				+ "_height");
		var shapeColor = g_ucAdmin.getVal(objSettings, settingNameColor,
				"#ffffff");
		var isflip = g_ucAdmin.getVal(objSettings, "shape_devider_" + position
				+ "_flip");
		var isAutodetectColor = g_ucAdmin.getVal(objSettings, "shape_devider_"
				+ position + "_autodetect_color");
		var placement = g_ucAdmin.getVal(objSettings, "shape_devider_"
				+ position + "_placement");
		var repeat = g_ucAdmin.getVal(objSettings, "shape_devider_" + position
				+ "_repeat", 1, g_ucAdmin.getvalopt.FORCE_NUMERIC);

		isAutodetectColor = g_ucAdmin.strToBool(isAutodetectColor);
		isflip = g_ucAdmin.strToBool(isflip);
		if (!repeat || repeat <= 0)
			repeat = 1;

		// autodetect color, and update it in settings as well
		if (isAutodetectColor == true) {

			var detectedColor = t.getDividerBGColor(objElement, objSettings,
					elementType, position);

			if (detectedColor && detectedColor != shapeColor) {
				shapeColor = detectedColor;

				if (t.g_temp.is_data_inited == true)
					t.updateElementSetting(objElement, settingNameColor,
							detectedColor);
			}

		}

		// if just installed - get content
		if (t.g_vars.shape_deviders.hasOwnProperty(shape) == false
				&& t.g_temp.is_data_inited == true) {

			var objBrowserShapes = t.g_pageBuilder.getObjBrowser("shape_deviders");
			var shapeData = objBrowserShapes.getAddonData(shape);
			var bgImage = g_ucAdmin.getVal(shapeData, "bgimage");

			if (bgImage) {

				bgImage = bgImage.replace("url(\"", "");
				bgImage = bgImage.replace("\")", "");
				t.g_vars.shape_deviders[shape] = bgImage;
			}

		}

		var shapeContent = g_ucAdmin.getVal(t.g_vars.shape_deviders, shape);

		if (!shapeContent) {
			objElement.removeClass(classDividerEnable);
			return (false);
		}

		objElement.addClass(classDividerEnable);

		var objDividerCss = {};

		// modify shape color
		if (shapeColor) {
			var svg = g_ucAdmin.base64_decode(shapeContent);
			svg = svg
					.replace('g fill="#ffffff"', 'g fill="' + shapeColor + '"');
			shapeContent = g_ucAdmin.base64_encode(svg);
		}

		var svgPrefix = "data:image/svg+xml;base64,";

		// change color in the content
		objDividerCss["background-image"] = "url('" + svgPrefix + shapeContent
				+ "')";

		// get repear percent
		var percentRepeat = 100;

		if (jQuery.isNumeric(repeat) && repeat > 0) {
			percentRepeat = 100 / repeat;

			var decimal = percentRepeat % 1;

			if (decimal > 0)
				percentRepeat = percentRepeat.toFixed(2);
		}

		if (height !== "") {

			height = g_ucAdmin.normalizeSizeValue(height);

			objDividerCss["background-size"] = percentRepeat + "% " + height;
			objDividerCss["height"] = height;
		}

		// flip the shape
		if (isflip == true) {
			if (position == "top")
				objDividerCss["transform"] = "rotateY(180deg)";
			else
				objDividerCss["transform"] = "rotateX(180deg) rotateY(180deg)";
		}

		if (placement == "beneath")
			objDividerCss["z-index"] = "0";

		// put mobile css
		var arrSettingsSizeRelated = {
			"height" : "shape_devider_" + position + "_height"
		};

		objDivider.removeAttr("style");
		objDivider.css(objDividerCss);

		// print mobile size
		var mobileSelector = "#" + elementID + " > " + selectorDivider;

		var cssSizeRelated = t.getSizeRelatedCss(arrSettingsSizeRelated,
				objSettings, mobileSelector, "!important", function(objSizeCss,
						cssAttribute, settingName, value, size) {

					if (value === "")
						return (objSizeCss);

					if (cssAttribute == "height") {
						objSizeCss["background-size"] = percentRepeat + "% "
								+ value + "px !important";
					}

					return (objSizeCss);
				});

		if (cssSizeRelated) {
			var styleContainerID = elementType + "_shape_devider_" + elementID
					+ "_sizes";

			g_ucAdmin.printCssStyle(cssSizeRelated, styleContainerID,
					g_objColStyleContainer);
		}

	}

	/**
	 * update object visual
	 */
	this.updateElementVisual = function(objElement, objCss, objSettings, prefix) {

		var elementID = objElement.prop("id");

		var settingsPrefix = "";
		if (prefix == "row")
			settingsPrefix = "row_";

		objCss = g_ucAdmin.addCssSetting(objSettings, objCss, settingsPrefix
				+ "padding_top", "padding-top", "px");
		objCss = g_ucAdmin.addCssSetting(objSettings, objCss, settingsPrefix
				+ "padding_bottom", "padding-bottom", "px");
		objCss = g_ucAdmin.addCssSetting(objSettings, objCss, settingsPrefix
				+ "padding_left", "padding-left", "px");
		objCss = g_ucAdmin.addCssSetting(objSettings, objCss, settingsPrefix
				+ "padding_right", "padding-right", "px");
		objCss = g_ucAdmin.addCssSetting(objSettings, objCss, settingsPrefix
				+ "margin_top", "margin-top", "px");
		objCss = g_ucAdmin.addCssSetting(objSettings, objCss, settingsPrefix
				+ "margin_bottom", "margin-bottom", "px");
		objCss = g_ucAdmin.addCssSetting(objSettings, objCss, settingsPrefix
				+ "margin_left", "margin-left", "px");
		objCss = g_ucAdmin.addCssSetting(objSettings, objCss, settingsPrefix
				+ "margin_right", "margin-right", "px");
		objCss = g_ucAdmin.addCssSetting(objSettings, objCss, settingsPrefix
				+ "text_align", "text-align");

		var cssBG = t.getBackgroundCss(objSettings);
		if (cssBG)
			jQuery.extend(objCss, cssBG);

		// remove style
		objElement.removeAttr("style");

		// add css
		var strAddCss = g_ucAdmin.getVal(objSettings, prefix + "_css", null);

		var addClass = g_ucAdmin.getVal(objSettings, prefix + "_class", null);

		// remove previous class
		var oldClass = objElement.data("addclass");
		if (oldClass && oldClass != addClass) {
			objElement.removeClass(oldClass);
			objElement.data("addclass", null);
		}

		// add the additional class
		if (addClass && addClass != oldClass) {
			objElement.addClass(addClass);
			objElement.data("addclass", addClass);
		}

		objElement.css(objCss);

		// update mobile
		var arrSettings = {
			"padding-top" : "padding_top",
			"padding-bottom" : "padding_bottom",
			"padding-left" : "padding_left",
			"padding-right" : "padding_right",
			"margin-top" : "margin_top",
			"margin-bottom" : "margin_bottom",
			"margin-left" : "margin_left",
			"margin-right" : "margin_right",
			"inline-css" : prefix + "_css"
		};

		// hide in devices

		var hideInMobile = g_ucAdmin.getVal(objSettings, "hide_in_mobile");
		hideInMobile = g_ucAdmin.strToBool(hideInMobile);
		if (hideInMobile == true)
			objElement.addClass("uc-hide-mobile");
		else
			objElement.removeClass("uc-hide-mobile");

		var hideInTablet = g_ucAdmin.getVal(objSettings, "hide_in_tablet");
		hideInTablet = g_ucAdmin.strToBool(hideInTablet);
		if (hideInTablet == true)
			objElement.addClass("uc-hide-tablet");
		else
			objElement.removeClass("uc-hide-tablet");

		var hideInDesktop = g_ucAdmin.getVal(objSettings, "hide_in_desktop");
		hideInDesktop = g_ucAdmin.strToBool(hideInDesktop);
		if (hideInDesktop == true)
			objElement.addClass("uc-element-hidden-desktop");
		else
			objElement.removeClass("uc-element-hidden-desktop");

		// put add css

		var container = g_objColStyleContainer;
		if (prefix == "row")
			container = g_objRowStyleContainer;

		if (strAddCss) {
			var styleContainerID = prefix + "_common_" + elementID + "_addcss";

			strAddCss = g_ucAdmin.removeLineBreaks(strAddCss);

			var cssAdditional = "#" + elementID + "{" + strAddCss + "}";

			g_ucAdmin.printCssStyle(cssAdditional, styleContainerID, container);
		}

		// put add css size related
		var cssSizeRelated = t.getSizeRelatedCss(arrSettings, objSettings, "#"
				+ elementID, "!important");

		if (cssSizeRelated) {

			var styleContainerID = prefix + "_common_" + elementID + "_sizes";

			g_ucAdmin
					.printCssStyle(cssSizeRelated, styleContainerID, container);
		}

		// shape deviders

		t.updateElementVisual_shapeDivider(objElement, objSettings, prefix,
				"top");
		t.updateElementVisual_shapeDivider(objElement, objSettings, prefix,
				"bottom");

		// background addons

		t.updateElementVisual_bgAddons(objElement, objSettings, prefix);
	}

	this.____________BACKGROUND_ADDONS______________ = function() {
	}

	/**
	 * update background addons
	 */
	this.updateElementVisual_bgAddons = function(objElement, options, prefix) {

		var classBGExists = "uc-has-bg-overlay";
		var enableBG = g_ucAdmin.getVal(options, "bg_enable", false,
				g_ucAdmin.getvalopt.FORCE_BOOLEAN);
		var objBGOverlay = objElement
				.children(".uc-grid-element-background-overlay");

		if (objBGOverlay.length == 0) {
			objElement.removeClass(classBGExists);
			return (false);
		}

		// get bg addon
		var existingAddonData = null;
		var objBGAddon = objBGOverlay.children(".uc-grid-bg-addon");
		if (objBGAddon.length == 0)
			objBGAddon = null;
		else {

			existingAddonData = objBGAddon.data("addon_data");
			var existingAddonName = existingAddonData.name;
		}

		var enableBGAddon = g_ucAdmin.getVal(options, "bg_addon_single_enable",
				false, g_ucAdmin.getvalopt.FORCE_BOOLEAN);

		if (enableBG == false || enableBGAddon == false) {
			if (objBGAddon)
				t.deleteBGAddon(objBGAddon);

			objElement.removeClass(classBGExists);
			return (false);
		}

		var settingsKey = "bg_addon_single";

		// get addon name
		var bgAddonName = g_ucAdmin.getVal(options, settingsKey);

		if (!bgAddonName) {
			if (objBGAddon)
				t.deleteBGAddon(objBGAddon);

			objElement.removeClass(classBGExists);
			return (false);
		}

		// get addon data, or create
		var addonData = g_ucAdmin.getVal(options, settingsKey + "_data");

		if (!addonData) {

			var extra = {};
			extra["title"] = bgAddonName;
			extra["url_icon"] = "";

			var addonData = {};
			addonData["name"] = bgAddonName;
			addonData["addontype"] = t.g_temp.addontype_bgaddon;
			addonData["extra"] = extra;
		}

		// compare addons data - existing and new
		if (existingAddonData) {
			
			var addonDataForCompare = t.g_objGridData.modifyAddonDataBeforeSave(addonData);
			var isEqual = g_objAddonConfig.isAddonsDataEqual(
					addonDataForCompare, existingAddonData);
			if (isEqual == true)
				return (false);
		}

		// if available existing bg addon - remove it

		if (objBGAddon)
			t.deleteBGAddon(objBGAddon);

		addonData = t.modifyAddonDataBeforeAdd(addonData);

		t.g_objAddonHTML.generateAddonHtml(addonData, function(htmlAddon) {
			
			objElement.addClass(classBGExists);
			objBGOverlay.html(htmlAddon);

			// store bg addon data
			var objBGAddon = objBGOverlay.children(".uc-grid-bg-addon");
			var addonDataSave = t.g_objGridData.modifyAddonDataBeforeSave(addonData);
			objBGAddon.data("addon_data", addonDataSave);

		});
	}

	/**
	 * delete background addon
	 */
	this.deleteBGAddon = function(objBGAddon) {

		if (!objBGAddon)
			return (false);

		t.validateColAddonElement(objBGAddon);

		var addonID = objBGAddon.prop("id");

		objBGAddon.remove();

		t.triggerEvent(t.events.ELEMENT_REMOVED, addonID);
	}

	this.____________ROW_UPDATE_VISUAL______________ = function() {
	}

	/**
	 * get background css
	 */
	this.getBackgroundCss = function(options) {

		var css = {};

		var oldColor = g_ucAdmin.getVal(options, "row_background_color");
		if (oldColor == true) {
			var enableBG = true;
		} else
			var enableBG = g_ucAdmin.getVal(options, "bg_enable", true,
					g_ucAdmin.getvalopt.FORCE_BOOLEAN);

		if (enableBG == false)
			return (css);

		// set color
		var color = g_ucAdmin.getVal(options, "bg_color");
		if (!color)
			color = oldColor;

		if (color)
			css["background-color"] = color;

		// set image
		var urlImage = g_ucAdmin.getVal(options, "bg_image_url");
		if (urlImage) {

			if (jQuery.isNumeric(urlImage))
				urlImage = g_ucAdmin.getVal(options, "bg_image_url_url");

			urlImage = g_ucAdmin.urlToFull(urlImage);

			var imageSize = g_ucAdmin.getVal(options, "bg_image_size");
			var imagePosition = g_ucAdmin.getVal(options, "bg_image_position");
			var imageRepeat = g_ucAdmin.getVal(options, "bg_image_repeat");
			var imageBlend = g_ucAdmin.getVal(options, "bg_image_blend");
			var imageParallax = g_ucAdmin.getVal(options, "bg_image_parallax",
					false, g_ucAdmin.getvalopt.FORCE_BOOLEAN);

			css["background-image"] = "url('" + urlImage + "')";

			if (imageSize)
				css["background-size"] = imageSize;

			if (imagePosition)
				css["background-position"] = imagePosition;

			if (imageRepeat)
				css["background-repeat"] = imageRepeat;

			if (imageBlend && imageBlend != "normal")
				css["background-blend-mode"] = imageBlend;

			if (imageParallax === true)
				css["background-attachment"] = "fixed";

		}

		// set gradient
		var enableGradient = g_ucAdmin.getVal(options, "bg_gradient_enable",
				false, g_ucAdmin.getvalopt.FORCE_BOOLEAN);
		if (enableGradient == true) {

			var gradientReverse = g_ucAdmin.getVal(options,
					"bg_gradient_reverse", false,
					g_ucAdmin.getvalopt.FORCE_BOOLEAN);

			if (gradientReverse == true) {
				var gradientColor2 = g_ucAdmin.getVal(options,
						"bg_gradient_color1");
				var gradientColor1 = g_ucAdmin.getVal(options,
						"bg_gradient_color2");
			} else {
				var gradientColor1 = g_ucAdmin.getVal(options,
						"bg_gradient_color1");
				var gradientColor2 = g_ucAdmin.getVal(options,
						"bg_gradient_color2");
			}

			var gradientStartPos = g_ucAdmin.getVal(options,
					"bg_gradient_start_pos");
			var gradientEndPos = g_ucAdmin.getVal(options,
					"bg_gradient_end_pos");
			var gradientLinearDir = g_ucAdmin.getVal(options,
					"bg_gradient_linear_direction");
			var gradientRadialDir = g_ucAdmin.getVal(options,
					"bg_gradient_radial_direction");

			var gradientType = g_ucAdmin.getVal(options, "bg_gradient_type");

			var strGradient = "";
			strGradient += gradientType + "-gradient(";
			if (gradientType == "linear") {
				strGradient += gradientLinearDir + "deg, ";
			} else
				strGradient += "circle at " + gradientRadialDir + ", ";

			strGradient += gradientColor1 + " " + gradientStartPos + "%, ";
			strGradient += gradientColor2 + " " + gradientEndPos + "%";

			strGradient += ")";

			var bgImageContent = g_ucAdmin.getVal(css, "background-image");
			if (bgImageContent)
				bgImageContent += ", ";

			bgImageContent += strGradient;

			css["background-image"] = bgImageContent;

		}

		return (css);
	}

	/**
	 * update cols and addons css of the row or container
	 */
	this.updateVisualCss_updateColsAndAddons = function(objSettings, parentID,
			selectorParent, objStyleContainer) {

		// ------ child columns ----

		var cssCol = {};
		cssCol = g_ucAdmin.addCssSetting(objSettings, cssCol, "col_gutter",
				"padding-left", "px");
		cssCol = g_ucAdmin.addCssSetting(objSettings, cssCol, "col_gutter",
				"padding-right", "px");

		var colSelector = selectorParent + " .uc-grid-col";
		var strCss = g_ucAdmin.arrCssToStrCss(cssCol, colSelector);

		// ------ child addons ----

		var addonBoxStyle = "";
		var spaceBetweenAddons = g_ucAdmin.getVal(objSettings,
				"space_between_addons", null);

		if (spaceBetweenAddons) {
			spaceBetweenAddons = g_ucAdmin
					.normalizeSizeValue(spaceBetweenAddons);
			addonBoxStyle += "margin-top:" + spaceBetweenAddons + ";";
		}

		var addonsSelector = selectorParent + " .uc-grid-col .uc-grid-col-addon + .uc-grid-col-addon";
		var selectorFirstAddon = selectorParent + " .uc-grid-col .uc-grid-overlay-empty + .uc-grid-col-addon";

		if (addonBoxStyle) {
			if (strCss)
				strCss += "\n";

			strCss += addonsSelector + "{" + addonBoxStyle + "}";
			strCss += selectorFirstAddon + "{margin-top:0px;}";
		}

		g_ucAdmin.printCssStyle(strCss, parentID + "_children",
				objStyleContainer);

		// mobile columns
		var arrSettings = {
			"padding-left" : "col_gutter",
			"padding-right" : "col_gutter"
		};

		var sizeRelatedCss = "";

		sizeRelatedCss += t.getSizeRelatedCss(arrSettings, objSettings,
				colSelector);

		// mobile addons
		var arrSettings = {
			"margin-top" : "space_between_addons"
		};

		sizeRelatedCss += t.getSizeRelatedCss(arrSettings, objSettings,
				addonsSelector);

		g_ucAdmin.printCssStyle(sizeRelatedCss, parentID + "_size",
				g_objRowStyleContainer);

	}

	/**
	 * update row visual css
	 */
	this.updateRowVisual_css = function(objRow, objSettings) {

		var rowID = objRow.prop("id");
		var selectorRow = g_gridID + " .uc-grid-row#" + rowID;
		var selectorContainer = selectorRow + " .uc-grid-row-container";

		// back color
		var cssRow = {};
		var cssRowMobile = {};
		var cssRowTablet = {};

		// output common objects
		var cssRow = {};
		t.updateElementVisual(objRow, cssRow, objSettings, "row");

		// ------ row anchor ----
		var strAnchorID = g_ucAdmin.getVal(objSettings, "row_id");
		var objAnchor = objRow.children(".uc-grid-row-anchor");
		var objAnchorIndicator = objRow
				.children(".uc-grid-row-anchor-indicator");

		if (strAnchorID) {
			objAnchor.prop("id", strAnchorID);
			objRow.addClass("uc-has-anchor");
			objAnchorIndicator.text("#" + strAnchorID);
		} else {
			objAnchor.prop("id", "");
			objRow.removeClass("uc-has-anchor");
			objAnchorIndicator.find("span").text("");
		}

		t.updateVisualCss_updateColsAndAddons(objSettings, rowID, selectorRow,
				g_objRowStyleContainer);

		var strCss = "";

		// ----------- Container

		var strStyleContainer = "";
		var containerWidth = g_ucAdmin.getVal(objSettings,
				"row_container_width", null);
		if (containerWidth) {
			containerWidth = g_ucAdmin.normalizeSizeValue(containerWidth);
			strStyleContainer += "max-width:" + containerWidth + ";";
		}

		// add container css
		var containerAddCss = g_ucAdmin.getVal(objSettings,
				"row_container_css", null);
		if (containerAddCss) {
			containerAddCss = g_ucAdmin.removeLineBreaks(containerAddCss);
			strStyleContainer += containerAddCss;
		}

		if (strStyleContainer) {
			strCss += "\n" + selectorContainer + "{" + strStyleContainer + "}";
		}

		// ------ print inner objects css ----

		g_ucAdmin.printCssStyle(strCss, rowID, g_objRowStyleContainer);

	}

	/**
	 * check if section intersect with row or addon
	 */
	this.updateRowVisual_buttons_getIntersect = function(objRow) {

		// var objContainers = t.getRowContainersAll(objRow);
		// var objContainer = objContainers.last();
		var intersectType = null;

		var rowPaddingBottom = t.getRowSetting(objRow, "row_padding_bottom", true);
		if (rowPaddingBottom >= 30)
			return (null);

		intersectType = "container";

		return (intersectType);
	}

	/**
	 * check container panel if intersects with row panel
	 */
	this.updateRowVisual_buttons_firstContainerPanel = function(objRow) {

		if (objRow.length == 0)
			return (false);

		var objContainer = t.getRowContainer(objRow);

		var offsetRow = objRow.offset();
		var offsetContainer = objContainer.offset();

		var diffTop = offsetContainer.top - offsetRow.top;
		if (diffTop > 30) {
			objRow.removeClass("uc-container-panel-intersect");
			return (false);
		}

		var diffLeft = offsetContainer.left - offsetRow.left;
		if (diffLeft > 30) {
			objRow.removeClass("uc-container-panel-intersect");
			return (false);
		}

		// if there is intersection add class
		objRow.addClass("uc-container-panel-intersect");

	}

	/**
	 * check buttons intersection
	 */
	this.updateRowVisual_buttons = function(objRow) {

		t.validateRow(objRow);

		// updateRowVisual_buttons_bottomButton(objRow);

		// try to update visual twice, for speed first time and more reliability
		// second time
		setTimeout(function() {
			t.updateRowVisual_buttons_firstContainerPanel(objRow);
		}, 500);

		setTimeout(function() {
			t.updateRowVisual_buttons_firstContainerPanel(objRow);
		}, 2000);

	}

	/**
	 * update row css
	 */
	this.updateRowVisual = function(objRow, settings) {

		var objSettings = settings || objRow.data("settings");

		t.updateRowVisual_css(objRow, objSettings);
		t.updateRowVisual_buttons(objRow, objSettings);

	}

	this.____________ROW_SETTINGS______________ = function() {
	}

	/**
	 * update row settings placeholders
	 */
	this.updateRowSettingsPlaceholders = function(objRowSettings, objOptions) {

		if (!objOptions)
			var objOptions = t.getCombinedOptions();

		var objRowPlaceholders = jQuery.extend({}, objOptions);

		var arrSettingNames = jQuery.extend([], t.g_temp.settings_placeholders);
		arrSettingNames.push("noprefix_col_gutter");
		arrSettingNames.push("noprefix_space_between_addons");
		arrSettingNames.push("noprefix_shape_devider_top_height");

		objRowPlaceholders = t.getObjSizePlaceholders(arrSettingNames,
				objRowPlaceholders, objRowSettings, objOptions, "row", "0");

		if (t.g_panel)
			t.g_panel.updatePlaceholders("row-settings", objRowPlaceholders);
	}

	/**
	 * check shape deviders if color changed
	 */
	this.checkRowUpdateRelatedElements = function(objRow, oldSettings, newSettings) {

		if (t.g_temp.is_data_inited == false)
			return (false);
		
		var isBGChanged = t.isElementBGColorChange(oldSettings, newSettings);

		if (isBGChanged == false)
			return (false);

		// --- check rows

		var prevRow = t.getPrevElement(objRow, "row");
		if (prevRow)
			t.onShapeDetectedColorParentChange(prevRow, "bottom");

		var nextRow = t.getNextElement(objRow, "row");
		if (nextRow)
			t.onShapeDetectedColorParentChange(nextRow, "top");

		// --- check columns
		var objCols = t.getCols(objRow, true);
		if (objCols.length == 0)
			return (false);

		var objCols = t.getAllRowCols(objRow);

		objCols.each(function(index, col) {

			var objCol = jQuery(col);

			t.onShapeDetectedColorParentChange(objCol, "top");
			t.onShapeDetectedColorParentChange(objCol, "bottom");
		});

	}

	/**
	 * set row settings, update css
	 */
	this.updateRowSettings = function(objRow, objSettings){
		
		// check bg color change
		var oldSettings = objRow.data("settings");
		
		objRow.data("settings", objSettings);

		t.updateRowVisual(objRow, objSettings);
		
		if (t.g_panel && t.g_panel.isVisible())
			t.updateRowSettingsPlaceholders(objSettings);
		
		t.triggerEvent(t.events.ROW_SETTINGS_UPDATED, objRow);

		t.checkRowUpdateRelatedElements(objRow, oldSettings, objSettings);

	}
	
	/**
	 * get row settings, combined with grid settings
	 */
	this.getRowSettings = function(objRow, addGridOptions, arrFilter) {

		var objRowSettings = objRow.data("settings");

		var objSettings = {};

		if (objRowSettings)
			jQuery.extend(objSettings, objRowSettings);

		// old Color fix
		var oldColor = g_ucAdmin.getVal(objSettings, "row_background_color");
		if (oldColor)
			objSettings["bg_color"] = oldColor;

		// add grid options
		if (addGridOptions === true) {
			var options = t.getCombinedOptions();
			jQuery.each(options, function(key, value) {
				var rowOption = g_ucAdmin.getVal(objSettings, key);
				if (rowOption === "")
					objSettings[key] = value;
			});
		}

		if (arrFilter)
			objSettings = g_ucAdmin.filterObjectByKeys(objSettings, arrFilter);

		return (objSettings);
	}

	/**
	 * get row setting
	 */
	this.getRowSetting = function(objRow, settingName, isNumeric) {

		var objSettings = t.getRowSettings(objRow, true);

		var getValOpt = null;
		if (isNumeric === true)
			getValOpt = g_ucAdmin.getvalopt.FORCE_NUMERIC;

		var value = g_ucAdmin.getVal(objSettings, settingName, 0, getValOpt);

		return (value);
	}

	
	/**
	 * open row settings dialog
	 */
	this.openRowSettingsPanel = function(objRow, isRefresh) {
		
		var objSettingsData = t.getRowSettings(objRow);
		var rowID = objRow.prop("id");
		
		var numRowAddons = t.getNumRowAddons(objRow);
		var isEmptyRow = (numRowAddons == 0);

		var params = {};
		if (isEmptyRow)
			params["add_title_prefix"] = g_uctext["empty"];
		else {
			var rowTitle = t.getRowTitle(objRow);

			var rowIndex = objRow.index() + 1;
			params["replace_title"] = rowTitle + " " + g_uctext["settings"];
		}
		
		if(isRefresh === true)
			params["refresh"] = true;
				
		if (t.g_panel)
			t.g_panel.open("row-settings", objSettingsData, rowID, null, params);

		t.updateRowSettingsPlaceholders(objSettingsData);
	}
	
	
	/**
	 * get column settings object
	 */
	this.getColSettings = function(objCol) {
		t.validateCol(objCol);

		var objSettings = objCol.data("settings");

		if (!objSettings)
			objSettings = {};

		return (objSettings);
	}

	/**
	 * apply row settings from panel
	 */
	this.applyRowSettings = function(params) {

		var isInstant = g_ucAdmin.getVal(params, "is_instant");
		var paramName = g_ucAdmin.getVal(params, "name");

		if (isInstant == true) { // check delay oriented params

			var isSettingDelayed = (jQuery.inArray(paramName,
					t.g_temp.settings_delay_row) !== -1);

			if (isSettingDelayed == true)
				return (false);
		}

		var data = t.g_panel.getPaneData("row-settings");
		var rowID = data.objectID;
		var objRow = jQuery("#" + rowID);

		if (objRow.length == 0)
			return (false);

		t.updateRowSettings(objRow, data.settings);
	}

	/**
	 * import from library
	 */
	this.importSectionFromLibrary = function(objRow) {

		if (!g_objBrowserSections)
			throw new Error("Sections browser not inited");

		g_objBrowserSections.openAddonsBrowser(null, function(data) {

			var layoutID = g_ucAdmin.getVal(data, "id");
			if (!layoutID)
				throw new Error("empty layout id for import");

			var dataSend = {};
			dataSend["layoutid"] = layoutID;

			t.g_panel.close();

			g_ucAdmin.ajaxRequest("get_grid_import_layout_data", dataSend,
					function(response) {

						var gridData = response["grid_data"];
						var objData = jQuery.parseJSON(gridData);

						var rows = g_ucAdmin.getVal(objData, "rows");

						jQuery.each(rows, function(index, row) {

							t.initByData_row(row, objRow, "before");

						});

					});

		});

	}

	/**
	 * open save section settings panel
	 */
	this.openSaveSectionSettingsPanel = function(objRow) {

		var rowID = objRow.prop("id");

		if (!t.g_panel)
			return (false);

		var title = t.getRowTitle(objRow);
		var objSectionData = t.g_objGridData.getGridData_row(objRow, false, true);

		var strData = g_ucAdmin.encodeObjectForSave(objSectionData);

		var objSettings = {};
		objSettings["section_title"] = title;
		objSettings["section_data"] = strData;

		var params = {
			"disable_trigger_change" : true
		};

		t.g_panel.open("save-section", objSettings, rowID, null, params);
	}

	this.____________MULTIPLE_ROWS______________ = function(){}

	/**
	 * update all rows visual
	 */
	this.updateAllRowsVisual = function(){
		var rows = t.getRows();

		jQuery.each(rows, function(index, row) {
			var objRow = jQuery(row);
			t.updateRowVisual(objRow);
		});
	}

	/**
	 * update rows icons, enable / disable relevant icons
	 */
	this.updateRowsIcons = function(){
		
		// disable paste icons
		if (!g_objBuffer)
			return (false);

		var storedBufferType = g_objBuffer.getStoredType();

		if (storedBufferType == "row") {
			t.g_objGrid.addClass("uc-inbuffer-row");
		} else {
			t.g_objGrid.removeClass("uc-inbuffer-row");
		}

	}

	this.____________ROW_CONTAINER______________ = function(){}

	/**
	 * get row container position - can be first - default or "last"
	 * 
	 * @param objRow
	 */
	this.getRowContainer = function(objRow, position) {
		
		if (objRow.hasClass("uc-grid-row-container"))
			return (objRow);

		var objContainer = objRow.find(".uc-grid-row-container");

		if (objContainer.length > 1) {
			if (position == "last")
				objContainer = objContainer.last();
			else
				objContainer = jQuery(objContainer[0]);
		}
		
		g_ucAdmin.validateDomElement(objContainer, "Row Container");
		return (objContainer);
	}

	/**
	 * validate if the object is row container
	 */
	this.validateRowContainer = function(objContainer) {
		
		if (!objContainer || objContainer.length == 0) {
			console.trace();
			throw new Error("row container not found");
		}

		if (typeof objContainer.hasClass != "function") {
			console.trace();
			trace(objContainer);
			throw new Error("row container is not jquery object");
		}

		if (objContainer.hasClass("uc-grid-row-container") == false) {
			trace(objContainer);
			throw new Error("Wrong row container object");
		}
	}

	/**
	 * get parent row
	 */
	this.getParentRowContainer = function(objChild) {

		var objContainer = objChild.parents(".uc-grid-row-container");

		return (objContainer);
	}

	/**
	 * get row container html
	 */
	this.getHtmlRowContainer = function() {

		var containerID = t.getNewElementID("container");

		html = "";
		html += "	<div id='" + containerID
				+ "' class='uc-grid-row-container unite-clearfix'>";

		// add hover lines

		html += "	<div class='uc-grid-container-hover uc-hover-top uc-grid-extras'></div>";
		html += "	<div class='uc-grid-container-hover uc-hover-bottom uc-grid-extras'></div>";
		html += "	<div class='uc-grid-container-hover uc-hover-left uc-grid-extras'></div>";
		html += "	<div class='uc-grid-container-hover uc-hover-right uc-grid-extras'></div>";

		// add container panel
		html += "	<div class='uc-grid-object-panel uc-grid-container-panel uc-grid-extras'>";
		html += "			<a href='javascript:void(0)' data-action='container_settings' data-actiontype='container' title='Row Settings' class=\"uc-container-icon uc-grid-action-icon uc-tip\" ><i class=\""
				+ t.getIcon("settings") + "\" aria-hidden=\"true\"></i></a> ";
		html += "			<a href='javascript:void(0)' data-action='duplicate_container' data-actiontype='container' title='Duplicate Row' class=\"uc-container-icon uc-grid-action-icon uc-tip\" ><i class=\""
				+ t.getIcon("duplicate") + "\" aria-hidden=\"true\"></i></a> ";
		html += "			<a href='javascript:void(0)' data-action='delete_container' data-actiontype='container' title='Delete Row' class=\"uc-container-icon uc-grid-action-icon uc-tip\" ><i class=\""
				+ t.getIcon("delete") + "\" aria-hidden=\"true\"></i></a> ";
		html += "			<a href='javascript:void(0)' title='Move Row' class=\"uc-container-icon uc-grid-icon-move uc-container-icon-move uc-tip\" ><i class=\""
				+ t.getIcon("move") + "\" aria-hidden=\"true\"></i></a> ";
		html += "			<a href='javascript:void(0)' data-action='add_row_container' data-actiontype='container' title='Add Row' class=\"uc-container-icon uc-grid-action-icon uc-tip\" ><i class=\"fa fa-plus\" aria-hidden=\"true\"></i></a> ";

		/*
		 * html += " <a href='javascript:void(0)'
		 * data-action='container_columns_layout' data-actiontype='container'
		 * title='Row Columns Layout' class=\"uc-container-icon
		 * uc-grid-action-icon uc-tip\" ><i class=\"fal fa-th-list\"
		 * aria-hidden=\"true\"></i></a> ";
		 */

		html += "	</div>"; // container panel end

		// add another container
		/*
		 * html += " <a href='javascript:void(0)'
		 * data-action='add_row_container' data-actiontype='container'
		 * title='Add Row' class='uc-btn-wrap uc-grid-action-icon
		 * uc-button-addcontainer-wrapper uc-grid-extras'><span class='uc-btn
		 * uc-btn-square'><i class='fa fa-plus' aria-hidden='true'></i></span></a>";
		 */

		html += "</div>"; // row container end

		return (html);
	}

	
	/**
	 * add row container
	 */
	this.addEmptyRowContainer = function(objElement, insertTo){
		
		if(!insertTo)
			var insertTo = "after";
		
		var html = t.getHtmlRowContainer();
		var objContainer = jQuery(html);

		var type = t.getElementType(objElement);

		switch (type) {
		case "row":
			objElement.append(objContainer);
		break;
		case "container":
			if(insertTo == "after")
				objContainer.insertAfter(objElement);
			else
				objContainer.insertBefore(objElement);				
			break;
		}
		
		var objRow = t.getParentRow(objElement);
		
		t.triggerEvent(t.events.ROW_CONTAINER_ADDED, objContainer);
		t.triggerEvent(t.events.ROW_CONTAINERS_UPDATED, objRow);

		return (objContainer);
	}
	
	
	/**
	 * add row container
	 */
	this.addRowContainer = function(objContainer, isFromRow) {

		if (isFromRow === true) // if the element is row passed
			objContainer = t.getRowContainer(objContainer, "last");

		var objContainer = t.addEmptyRowContainer(objContainer);
		t.addColumn(objContainer);
	}

	/**
	 * get last row container
	 */
	this.getRowContainersAll = function(objRow) {
		
		var objContainers = objRow.find(".uc-grid-row-container");

		return (objContainers);
	}

	/**
	 * get number of row containers
	 */
	this.getNumContainers = function(objRow) {
		
		var objContainers = t.getRowContainersAll(objRow);

		return (objContainers.length);
	}

	/**
	 * delete row container
	 */
	this.deleteRowContainer = function(objContainer) {
		
		t.validateRow(objContainer);

		var objRow = t.getParentRow(objContainer);

		var numContainers = t.getNumContainers(objRow);
		if (numContainers <= 1) {
			t.deleteRow(objRow);
			return (false);
		}

		var containerID = objContainer.prop("id");

		objContainer.remove();

		t.triggerEvent(t.events.ROW_CONTAINERS_UPDATED, objRow);
		t.triggerEvent(t.events.ELEMENT_REMOVED, containerID);

	}

	/**
	 * duplicate container
	 */
	this.duplicateRowContainer = function(objContainer) {
		
		// get container in case that row passed
		objContainer = t.getRowContainer(objContainer);

		objContainer.sortable("destroy");

		var containerID = t.getNewElementID("container");

		var objContainerCopy = objContainer.clone(true, true);

		objContainerCopy.attr("id", containerID);

		objContainerCopy.insertAfter(objContainer);

		var objRow = t.getParentRow(objContainer);

		t.triggerEvent(t.events.ROW_CONTAINER_DUPLICATED, objContainerCopy);
		t.triggerEvent(t.events.ROW_CONTAINERS_UPDATED, objRow);

		// make sortable the initial container again

		t.initSortableColumns(objContainer);
	}
	
	
	/**
	 * change order of the rows by id's
	 */
	this.orderRowContainersByIDs = function(arrIDs){
		
		var objParentRow = null;
		
		if(_.isEmpty(arrIDs))
			return(false);
		
		jQuery.each(arrIDs, function(index){
			
			var containerID = arrIDs[index];
			var objContainer = jQuery("#"+containerID);
			
			g_ucAdmin.validateDomElement(objContainer, "container with ID: "+containerID);
			
			if(!objParentRow)
				objParentRow = t.getParentRow(objContainer);
			
			objParentRow.append(objContainer);
			
		});
		
		t.triggerEvent(t.events.ROW_CONTAINERS_UPDATED, objParentRow);
		
	}
	
	
	/**
	 * row layout chosen
	 */
	this.changeColumnsLayout = function(objContainer, layoutType) {

		var objLayout = jQuery(this);

		var columnsSize = layoutType.split('-');
		var numCols = t.getNumCols(objContainer);
		var newNumCols = columnsSize.length;

		// even number, just resize
		if (numCols == newNumCols) {

			columnsSize.forEach(function(size, index) {
				var objCol = t.getCol(objContainer, index);
				t.setColSize(objCol, size);
			});

			return (false);
		}

		// add more columns, add new
		if (newNumCols > numCols) {

			columnsSize.forEach(function(size, index) {

				if (index < numCols) {
					var objCol = t.getCol(objContainer, index);
					t.setColSize(objCol, size);
				} else
					t.addLayoutColumn(objContainer, size);
			});

			return (false);
		}

		// reduce number of columns - put the rest of addons to the last column

		var isOK = true;
		var numAddons = t.getNumRowAddons(objContainer);
		if (numAddons > 1)
			isOK = confirm("Replace Addons?");

		if (isOK == false)
			return (false);

		for ( var i = numCols - 1; i >= 0; i--) {
			var objCol = t.getCol(objContainer, i);
			if (i < columnsSize.length)
				t.setColSize(objCol, columnsSize[i]);
			else {
				var objAddons = t.g_objGridData.getColAddonsData(objCol);
				var lastObjCol = t.getCol(objContainer, columnsSize.length - 1);
				if (objAddons.length)
					objAddons.forEach(function(item, index) {
						t.addColAddon(lastObjCol, item, true);
					});
				t.deleteCol(objCol);
			}
		}

	}

	/**
	 * do row container related action
	 */
	this.doContainerAction = function(action, objContainer) {

		t.validateRowContainer(objContainer);

		switch (action) {
		case "delete_container":
			t.deleteRowContainer(objContainer);
			break;
		case "duplicate_container":
			t.duplicateRowContainer(objContainer);
			break;
		case "container_settings":
			t.openContainerSettingsPanel(objContainer);
			break;
		case "add_row_container":
			t.addRowContainer(objContainer);
			break;
		default:
			throw new Error("Wrong container action: " + action);
			break;
		}

	}

	this.____________CONTAINER_SETTINGS______________ = function(){}

	/**
	 * update placeholders
	 */
	this.updateRowContainerSettingsPlaceholders = function(objContainer, objContainerSettings){
		
		var objRow = t.getParentRow(objContainer);
		var objRowSettings = t.getRowSettings(objRow, true);

		var arrSettingNames = jQuery.extend([], t.g_temp.settings_placeholders);
		arrSettingNames.push("noprefix_col_gutter");
		arrSettingNames.push("noprefix_space_between_addons");

		arrSettingNames.push("row_container_width");
		arrSettingNames.push("col_gutter");
		arrSettingNames.push("space_between_addons");

		var rowSettingsFiltered = g_ucAdmin.filterObjectByKeys(objRowSettings,
				arrSettingNames);
		var objContainerPlaceholders = jQuery.extend({}, rowSettingsFiltered);

		objContainerPlaceholders = t.getObjSizePlaceholders(arrSettingNames,
				objContainerPlaceholders, objContainerSettings, objRowSettings,
				null, "0");

		t.g_panel.updatePlaceholders("container-settings", objContainerPlaceholders);
	}

	/**
	 * open row settings dialog
	 */
	this.openContainerSettingsPanel = function(objContainer, isRefresh) {

		// in case of row pass
		objContainer = t.getRowContainer(objContainer);

		var objSettingsData = t.getRowContainerSettings(objContainer);

		var containerID = objContainer.prop("id");
		
		var params = {};
		if(isRefresh == true)
			params.refresh = true;
		
		t.g_panel.open("container-settings", objSettingsData, containerID, null, params);

		t.updateRowContainerSettingsPlaceholders(objContainer, objSettingsData);
	}

	/**
	 * get container settings
	 */
	this.getRowContainerSettings = function(objContainer) {
		
		var objSettings = objContainer.data("settings");
		if (!objSettings)
			var objSettings = {};

		// add columns layout
		objSettings["col_layout"] = t.getColSizesString(objContainer);

		return (objSettings);
	}

	/**
	 * update container visual settings
	 */
	this.updateRowContainerVisual = function(objContainer) {
		
		var containerID = objContainer.prop("id");
		var selectorContainer = g_gridID + " .uc-grid-row-container#" + containerID;
		
		var objSettings = t.getRowContainerSettings(objContainer);
		var objCss = {};

		// update defaults
		var cssContainer = {};

		// get css object
		var containerWidth = g_ucAdmin.getVal(objSettings,
				"row_container_width");

		if (containerWidth !== "")
			cssContainer["max-width"] = g_ucAdmin
					.normalizeSizeValue(containerWidth);

		t.updateElementVisual(objContainer, cssContainer, objSettings, "container");
		
		// update columns and addons
		t.updateVisualCss_updateColsAndAddons(objSettings, containerID, selectorContainer, g_objColStyleContainer);

	}

	/**
	 * update container settings
	 */
	this.updateContainerSettings = function(objContainer, objSettings) {

		objContainer.data("settings", objSettings);

		t.updateRowContainerVisual(objContainer, objSettings);

		if (t.g_panel && t.g_panel.isVisible())
			t.updateRowContainerSettingsPlaceholders(objContainer, objSettings);

		t.triggerEvent(t.events.ROW_CONTAINER_SETTINGS_UPDATED, objContainer);
	}
	
	
	/**
	 * apply row container settings
	 */
	this.applyRowContainerSettings = function(params) {
		
		var isInstant = g_ucAdmin.getVal(params, "is_instant");
		var paramName = g_ucAdmin.getVal(params, "name");
		var paramValue = g_ucAdmin.getVal(params, "value");

		if (isInstant == true) { // check delay oriented params
			var isSettingDelayed = (jQuery.inArray(paramName,
					t.g_temp.settings_delay_row) !== -1);

			if (isSettingDelayed == true)
				return (false);
		}

		var data = t.g_panel.getPaneData("container-settings");
		var containerID = data.objectID;
		var objSettings = g_ucAdmin.getVal(data, "settings");
		
		var objContainer = jQuery("#" + containerID);
		if (objContainer.length == 0)
			return (false);

		// update col layout, if was a click on layout row
		if (paramName == "col_layout") {
			t.changeColumnsLayout(objContainer, paramValue);
			return (false);
		}

		t.updateContainerSettings(objContainer, objSettings);
	}

	this.____________COLUMN______________ = function(){}

	/**
	 * validate that the object is column
	 */
	this.validateCol = function(objCol) {

		g_ucAdmin.validateDomElement(objCol, "column");

		if (objCol.hasClass("uc-grid-col") == false)
			throw new Error("The object is not column type");
	}
	
	
	/**
	 * get html columns
	 */
	this.getHtmlColumn = function(){
		
		var colID = t.getNewElementID("col");
		
		var html = "";
		html += "<div id='" + colID + "' class=\"uc-grid-col "+t.g_vars.class_empty+" \">";
		
		// add shape deviders

		html += "<div class='uc-grid-shape-devider uc-devider-top'></div>";
		html += "<div class='uc-grid-shape-devider uc-devider-bottom'></div>";

		html += "<div class=\"uc-grid-box-wrapper\">";

		html += "		<div class=\"uc-grid-col-addons\">";
				
		html += "		</div>"; // col addons

		html += "	</div>"; // box wrapper end

		// hidden overlay
		html += "<div class='uc-grid-element-hidden-overlay'>"
				+ g_uctext.hidden_column + "</div>";
		
		html += "<div class='uc-grid-element-empty-placeholder'>+</div>";

		// top panel
		html += "		<div class = \"uc-grid-object-panel uc-grid-col-panel uc-grid-extras\">";
		html += "			<a href='javascript:void(0)' title='"
				+ g_uctext.move_column
				+ "' class=\"uc-col-icon uc-grid-icon-move uc-col-icon-move uc-tip\"><i class=\""
				+ t.getIcon("move") + "\" aria-hidden=\"true\"></i></a> ";
		html += "			<a href='javascript:void(0)' data-action='duplicate' data-actiontype='col' title='"
				+ g_uctext.duplicate_column
				+ "' class=\"uc-col-icon uc-grid-action-icon uc-tip\"><i class=\""
				+ t.getIcon("duplicate") + "\" aria-hidden=\"true\"></i></a> ";
		html += "			<a href='javascript:void(0)' data-action='delete' data-actiontype='col' title='"
				+ g_uctext.delete_column
				+ "' class=\"uc-col-icon uc-grid-action-icon uc-tip\"><i class=\""
				+ t.getIcon("delete") + "\" aria-hidden=\"true\"></i></a> ";
		html += "			<a href='javascript:void(0)' data-action='settings' data-actiontype='col' title='"
				+ g_uctext.column_settings
				+ "' class=\"uc-col-icon uc-grid-action-icon uc-tip\"><i class=\""
				+ t.getIcon("settings") + "\" aria-hidden=\"true\"></i></a> ";
		html += "			<a href='javascript:void(0)' data-action='addcol_after' data-actiontype='col' title='"
				+ g_uctext.add_column
				+ "' class=\"uc-col-icon uc-grid-action-icon uc-icon-addcol uc-tip\"><i class=\"fa fa-plus\" aria-hidden=\"true\"></i></a> ";
		html += "		</div>";

		html += "<div class='uc-grid-col-hover uc-hover-left uc-grid-extras'></div>";
		html += "<div class='uc-grid-col-hover uc-hover-top uc-grid-extras'></div>";
		html += "<div class='uc-grid-col-hover uc-hover-right uc-grid-extras'></div>";
		html += "<div class='uc-grid-col-hover uc-hover-bottom uc-grid-extras'></div>";

		/*
		 * html += " <a href='javascript:void(0)' data-action='addcol_before'
		 * data-actiontype='col' title='"+g_uctext.add_column+"'
		 * class=\"uc-icon-addcol uc-addcol-before uc-grid-action-icon\" ><span
		 * class='uc-btn uc-btn-square uc-btn-column uc-grid-extras'><i
		 * class=\"fa fa-plus\" aria-hidden=\"true\"></i></span></a> "; html += "
		 * <a href='javascript:void(0)' data-action='addcol_after'
		 * data-actiontype='col' title='"+g_uctext.add_column+"'
		 * class=\"uc-icon-addcol uc-addcol-after uc-grid-action-icon\" ><span
		 * class='uc-btn uc-btn-square uc-btn-column uc-grid-extras'><i
		 * class=\"fa fa-plus\" aria-hidden=\"true\"></i></span></a>";
		 */

		html += "</div>"; // col;

		return (html);
	}

	/**
	 * get all row cols from all the containers
	 */
	this.getAllRowCols = function(objRow) {
		
		var objCols = objRow.find(".uc-grid-col");

		return (objCols);
	}

	/**
	 * get columns in row
	 */
	this.getCols = function(objRow, novalidate) {

		var objContainer = t.getRowContainer(objRow);

		var objCols = objContainer.children(".uc-grid-col");

		if (objCols.length == 0 && !novalidate)
			throw new Error("getCols error - row should have at least 1 column");

		return (objCols);
	}

	/**
	 * get column by number
	 */
	this.getCol = function(objRow, numCol) {

		var objCols = t.getCols(objRow);
		if (numCol >= objCols.length)
			throw new Error("There is no col number: " + numCol + " in the row");

		var objCol = jQuery(objCols[numCol]);

		return (objCol);
	}

	/**
	 * change order of the rows by id's
	 */
	this.orderColumnsByIDs = function(arrIDs){
		
		var objParentContainer = null;
		
		if(_.isEmpty(arrIDs))
			return(false);
		
		jQuery.each(arrIDs, function(index){
			
			var colID = arrIDs[index];
			var objCol = jQuery("#"+colID);
			
			g_ucAdmin.validateDomElement(objCol, "col with ID: "+colID);
			
			if(!objParentContainer){
				objParentContainer = t.getParentRowContainer(objCol);
			}
			
			objParentContainer.append(objCol);
		});
		
		
		t.triggerEvent(t.events.ROW_COLUMNS_UPDATED, objParentContainer);
		
	}
	
	
	/**
	 * get parent column
	 */
	this.getParentCol = function(objChild) {
		
		var objCol = objChild.parents(".uc-grid-col");

		return (objCol);
	}

	
	/**
	 * get number of columns in row or container
	 */
	this.getNumCols = function(objRow) {

		t.validateRow(objRow);

		var objCols = t.getCols(objRow, true);
		
		var numCols = objCols.length;

		return (numCols);
	}

	/**
	 * get addons wrapper
	 */
	this.getColAddonsWrapper = function(objCol) {

		var objAddonsWrapper = objCol.find(".uc-grid-col-addons");

		g_ucAdmin.validateDomElement(objAddonsWrapper, "col addons wrapper");

		return (objAddonsWrapper);
	}

	/**
	 * check if it's first column
	 */
	this.isFirstCol = function(objCol) {
		
		var isFirst = objCol.hasClass(t.g_vars.class_first_col);

		return isFirst;
	}

	/**
	 * check if it's last column
	 */
	this.isLastCol = function(objCol) {
		var isLast = objCol.hasClass("uc-col-last");

		return isLast;
	}

	/**
	 * get column size from the column
	 */
	this.getColSize = function(objCol) {

		t.validateCol(objCol);

		var arrClasses = objCol[0].className.match(/\buc-colsize\S+/ig);

		if (arrClasses.length == 0)
			return (null);

		var className = arrClasses[0];
		var size = className.replace("uc-colsize-", "");

		return (size);
	}

	/**
	 * get row container row sizes string
	 */
	this.getColSizesString = function(objContainer) {

		var objColumns = t.getCols(objContainer, true);
		if (objColumns.length == 0)
			return ("");

		var strSizes = "";
		jQuery.each(objColumns, function(index, col) {

			var objCol = jQuery(col);
			var size = t.getColSize(objCol);

			if (strSizes.length)
				strSizes += "-";

			strSizes += size;
		});

		return (strSizes);
	}
	
	this.____________COLUMN_ACTIONS______________ = function(){}

	
	/**
	 * add empty column. row can be row container as well the mode can be:
	 * empty, before, after
	 */
	this.addColumn = function(objRow, objCol, mode) {

		if (!objRow) {

			if (objCol)
				var objContainer = t.getParentRowContainer(objCol);
			else {
				var objRow = t.getRow();
				var objContainer = t.getRowContainer(objRow);
			}
		} else {
			var objContainer = t.getRowContainer(objRow);
		}

		t.validateRowContainer(objContainer);

		// check the limits
		var numCols = t.getNumCols(objContainer);

		if (numCols == t.g_vars.max_cols)
			return (false);

		// add the column
		var htmlCol = t.getHtmlColumn();

		var objNewCol = jQuery(htmlCol);

		// insert before or after column
		objNewCol.hide();

		if (objCol) {

			switch (mode) {
			case "before":
				objNewCol.insertBefore(objCol);
				break;
			case "after":
				objNewCol.insertAfter(objCol);
				break;
			default:
				break;
			}

		} else { // insert last column

			objContainer.append(objNewCol);
		}

		t.triggerEvent(t.events.ROW_COLUMNS_UPDATED, objContainer);
		t.triggerEvent(t.events.COL_ADDED, objNewCol);

		// show the column after transition
		setTimeout(function() {
			objNewCol.show();
		}, 550);

		return (objNewCol);
	}

	/**
	 * add columns for layout
	 */
	this.addLayoutColumn = function(objRow, colClass) {
		
		if (!objRow)
			return (false);

		var objContainer = t.getRowContainer(objRow);

		var numCols = t.getNumCols(objContainer);
		var htmlCol = t.getHtmlColumn();
		var objNewCol = jQuery(htmlCol).addClass("uc-col-empty uc-col-last uc-colsize-" + colClass + " uc-col-trans");
		
		objNewCol.hide();

		var objCol = t.getCol(objContainer, numCols - 1);
		jQuery(objCol).removeClass("uc-col-last");

		objNewCol.insertAfter(objCol);
		t.updateColOperationButtons(objContainer);

		setTimeout(function() {
			objNewCol.show();
		}, 350);

	}

	/**
	 * duplicate column
	 */
	this.duplicateCol = function(objCol) {

		var objContainer = t.getParentRowContainer(objCol);

		// check the limits
		var numCols = t.getNumCols(objContainer);

		if (numCols == t.g_vars.max_cols)
			return (false);

		var objColCopy = objCol.clone(true, true);
		objColCopy.hide();

		var colID = t.getNewElementID("col");
		objColCopy.attr("id", colID);

		objColCopy.insertAfter(objCol);

		t.triggerEvent(t.events.ROW_COLUMNS_UPDATED, objContainer);

		// show the column after transition

		setTimeout(function() {
			objColCopy.show();

			t.triggerEvent(t.events.COL_ADDED, objColCopy);
			t.triggerEvent(t.events.COL_DUPLICATED, objColCopy);
		}, 500);

	}

	/**
	 * 
	 * @param objCol
	 */
	this.deleteCol = function(objCol) {

		var objContainer = t.getParentRowContainer(objCol);
		var numCols = t.getNumCols(objContainer);
		var colID = objCol.prop("id");

		if (numCols <= 1) {
			t.deleteRowContainer(objContainer);
			return (false);
		}

		objCol.remove();

		t.triggerEvent(t.events.ROW_COLUMNS_UPDATED, objContainer);
		t.triggerEvent(t.events.ELEMENT_REMOVED, colID);
	}

	/**
	 * update row columns classes
	 */
	this.updateColsClasses = function(objContainer) {
		
		var objCols = t.getCols(objContainer);

		var numCols = objCols.length;

		var colWidth = 1; // temp value, num cells that it take

		var classColSize = t.g_vars.class_size_prefix + colWidth + "_" + numCols;

		objCols.each(function(num, col) {

			var isFirst = (num == 0);
			var isLast = (num == numCols - 1);

			var objCol = jQuery(col);
			var isEmpty = objCol.hasClass(t.g_vars.class_empty);
			
			// set class
			var classCol = t.g_vars.class_col;

			if (isFirst)
				classCol += " " + t.g_vars.class_first_col;

			if (isLast)
				classCol += " " + t.g_vars.class_last_col;

			if (isEmpty)
				classCol += " " + t.g_vars.class_empty;
			
			classCol += " " + classColSize;

			classCol += " uc-col-trans";
						
			col.className = classCol;

		});

	}

	/**
	 * set the add col icon active / not active
	 */
	this.activateAddColIcon = function(objIcon, isActivate) {

		if (isActivate) {
			objIcon.addClass("uc-icon-active");
		} else {
			objIcon.removeClass("uc-icon-active");
		}

	}

	/**
	 * check column operations buttons when
	 */
	this.updateColOperationButtons = function(objContainer) {

		t.validateRowContainer(objContainer);

		var numCols = t.getNumCols(objContainer);

		if (numCols >= t.g_vars.max_cols) { // hide

			objContainer.addClass("uc-max-cols");

		} else { // not hide
			objContainer.removeClass("uc-max-cols");

		}

	}
	
	

	/**
	 * set column empty state loading
	 */
	this.setColEmptyStateLoading = function(objCol, isLoading) {
		
		t.validateCol(objCol);

		var objOverlayEmptyContent = objCol.find(".uc-grid-overlay-empty .uc-overlay-empty-content");

		if (isLoading == true)
			objOverlayEmptyContent.addClass("uc-state-loading");
		else
			objOverlayEmptyContent.removeClass("uc-state-loading");

	}

	/**
	 * set row hover second mode
	 */
	this.setColHoverSecondMode = function(event, objCol) {

		if (!objCol)
			var objCol = jQuery(this);

		if (objCol.hasClass("uc-grid-col") == false)
			objCol = t.getParentCol(objCol);

		objCol.addClass("uc-over-mode2");

	}

	/**
	 * unset row hover second mode
	 */
	this.unsetColHoverSecondMode = function(event, objCol) {

		if (!objCol)
			var objCol = jQuery(this);

		if (objCol.hasClass("uc-grid-col") == false)
			objCol = t.getParentCol(objCol);

		objCol.removeClass("uc-over-mode2");
	}

	
	/**
	 * update row layout columns classes
	 */
	this.setColSize = function(objCol, size) {

		t.validateCol(objCol);

		var className = objCol[0].className;

		objCol[0].className = className.replace(/\buc-colsize\S+/ig,"uc-colsize-" + size);
		
	}

	/**
	 * do column action
	 */
	this.doColAction = function(objCol, action) {

		switch (action) {
		case "delete":

			t.deleteCol(objCol);

			break;
		case "duplicate":

			t.duplicateCol(objCol);

			break;
		case "addcol_before":

			t.addColumn(null, objCol, "before");

			break;
		case "addcol_after":

			t.addColumn(null, objCol, "after");

			break;
		case "settings":
			t.openColSettingsPanel(objCol);
			break;
		case "add_col_addon":
			t.openAddonsBrowser(objCol);
			break;
		default:
			trace("wrong col action: " + action);
			break;
		}

	}

	this.______COLUMN_SETTINGS_______ = function() {}

	/**
	 * set column settings placeholders
	 */
	this.setColumnPlaceholders = function(objCol, colSettings) {
		
		var objPlaceholders = {};

		var objRow = t.getParentRow(objCol);
		var filter = [ "space_between_addons", "col_gutter" ];
		var rowSettings = t.getRowSettings(objRow, true, filter);

		objPlaceholders["col_space_between_addons"] = g_ucAdmin.getVal(
				rowSettings, "space_between_addons");

		var colPaddingSides = g_ucAdmin.getVal(rowSettings, "col_gutter");
		objPlaceholders["padding_left"] = colPaddingSides;
		objPlaceholders["padding_right"] = colPaddingSides;

		// set mobile placeholders

		var arrSettingNames = jQuery.extend([], t.g_temp.settings_placeholders);
		arrSettingNames.push("col_space_between_addons");

		objPlaceholders = t.getObjSizePlaceholders(arrSettingNames,
				objPlaceholders, colSettings, objPlaceholders, null, "0");

		if (t.g_panel)
			t.g_panel.updatePlaceholders("col-settings", objPlaceholders);
	}

	/**
	 * open col settings panel
	 */
	this.openColSettingsPanel = function(objCol) {

		t.validateCol(objCol);
		var colID = objCol.prop("id");
		var colSettings = t.getColSettings(objCol);

		// set column placeholders
		t.setColumnPlaceholders(objCol, colSettings);

		t.g_panel.open("col-settings", colSettings, colID);
	}

	/**
	 * update column visual
	 */
	this.updateColVisual = function(objCol, objSettings) {
		
		if (!objSettings)
			var objSettings = objCol.data("settings");

		var cssCol = {};
		t.updateElementVisual(objCol, cssCol, objSettings, "col");

		var strCss = "";

		// --- column addons elements

		var colID = objCol.prop("id");
		var objRow = t.getParentRow(objCol);
		var rowID = objRow.prop("id");
		var selectorAddons = g_gridID + " .uc-grid-row#" + rowID + " .uc-grid-col#" + colID + " .uc-grid-col-addon + .uc-grid-col-addon";
		var selectorFirstAddon = g_gridID + " .uc-grid-row#" + rowID + " .uc-grid-col#" + colID + " .uc-grid-overlay-empty + .uc-grid-col-addon";

		var spaceBetweenAddons = g_ucAdmin.getVal(objSettings,
				"col_space_between_addons", null);
		if (spaceBetweenAddons) {
			strCss += selectorAddons + "{";
			strCss += "margin-top:"
					+ g_ucAdmin.normalizeSizeValue(spaceBetweenAddons);
			strCss += "}";

			strCss += selectorFirstAddon + "{";
			strCss += "margin-top:0px";
			strCss += "}";

		}

		g_ucAdmin.printCssStyle(strCss, colID, g_objColStyleContainer);

		// ------- size related space between addons

		var arrSettings = {
			"margin-top" : "col_space_between_addons"
		};

		var sizeRelatedCss = t.getSizeRelatedCss(arrSettings, objSettings,
				selectorAddons);

		g_ucAdmin.printCssStyle(sizeRelatedCss, colID + "_size",
				g_objColStyleContainer);

	}

	/**
	 * update column settings
	 */
	this.updateColSettings = function(objCol, settings) {
		
		t.validateCol(objCol);

		objCol.data("settings", settings);
		
		t.updateColVisual(objCol, settings);

		t.triggerEvent(t.events.COL_SETTINGS_UPDATED, objCol);
		
	}
	

	/**
	 * apply column settings
	 */
	this.applyColSettings = function(){

		var data = t.g_panel.getPaneData("col-settings");
		var colID = data.objectID;
		var objCol = jQuery("#" + colID);

		if (objCol.length == 0)
			return (false);

		t.updateColSettings(objCol, data.settings);
		
		if (t.g_panel.isVisible())
			t.setColumnPlaceholders(objCol, data.settings);
	}

	

	this.____________COL_ADDON_ELEMENT______________ = function() {}

	/**
	 * validate that the column element is col addon type
	 */
	this.validateColAddonElement = function(objElement) {

		// validate type
		var type = t.getElementType(objElement);
		if (type != "addon" && type != "bgaddon")
			throw new Error("The element must be addons type");

		// validate single
		if (objElement.length > 1) {
			trace(objElement);
			throw new Error("The addon element should be sinlge");
		}

	}

	/**
	 * get col addons data
	 */
	this.getColAddons = function(objCol) {
		
		t.validateCol(objCol);

		var objAddonsWrapper = t.getColAddonsWrapper(objCol);

		var objAddons = t.getElementAddons(objAddonsWrapper);

		return (objAddons);
	}

	/**
	 * get number of col addons
	 */
	this.getNumColAddons = function(objCol) {

		t.validateCol(objCol);
		
		var objAddons = t.getColAddons(objCol);
		var numAddons = objAddons.length;
		
		return(numAddons);
	}

	/**
	 * get parent row
	 */
	this.getParentAddonElement = function(objChild) {

		var objAddon = objChild.parents(".uc-grid-col-addon");

		g_ucAdmin.validateDomElement(objAddon, "addon holder");

		return (objAddon);
	}

	/**
	 * show/hide move icon when number addons = 1,columns = 1,rows = 1
	 */
	this.isSingleAddonInGrid = function(objAddon) {

		var objCol = t.getParentCol(objAddon);

		var numRows = t.getNumRows();

		if (numRows > 1)
			return (false);

		var objRow = t.getParentRow(objCol);

		var numColums = t.getNumCols(objRow);

		if (numColums > 1)
			return (false);

		var numAddons = t.getNumColAddons(objCol);

		if (numAddons > 1)
			return (false);

		return (true);
	}

	
	/**
	 * delete column addon
	 */
	this.deleteColAddon = function(objAddon) {
		
		t.validateColAddonElement(objAddon);

		var addonID = objAddon.prop("id");

		var objCol = t.getParentCol(objAddon);

		objAddon.remove();
		
		t.triggerEvent(t.events.COL_ADDONS_UPDATED, objCol);
		t.triggerEvent(t.events.ELEMENT_REMOVED, addonID);
	}

	/**
	 * duplicate col addon
	 */
	this.duplicateColAddon = function(objAddon) {

		t.validateColAddonElement(objAddon);

		var objAddonCopy = objAddon.clone(true, true);

		// hide overlay
		// showAddonOverlay(objAddonCopy, false);

		// insert new addon
		objAddonCopy.insertAfter(objAddon);
		t.triggerEvent(t.events.ADDON_DUPLICATED, objAddonCopy);

		var objCol = t.getParentCol(objAddon);
		t.triggerEvent(t.events.COL_ADDONS_UPDATED, objCol);
	}

	
	/**
	 * do addon element related action
	 */
	this.doAddonAction = function(objAddon, action) {

		t.validateColAddonElement(objAddon);
		
		switch (action) {
		case "edit_addon":
			t.openAddonsBrowser(objAddon);
			break;
		case "addon_container_settings":
			t.openAddonContainerSettingsPanel(objAddon);
			break;
		case "delete_addon":
			t.deleteColAddon(objAddon);
			break;
		case "duplicate_addon":
			t.duplicateColAddon(objAddon);
			break;
		case "add_col_addon":
			var parentCol = t.getParentCol(objAddon);
			t.openAddonsBrowser(parentCol, objAddon);
			break;
		default:
			throw new Error("Wrong addon action: " + action);
			break;
		}

	}

	this.____________ADDONS_SETTINGS______________ = function() {
	}

	/**
	 * update addon container visual
	 */
	this.updateAddonContainerVisual = function(objAddon, objSettings) {

		if (!objSettings)
			var objSettings = objAddon.data("addon_settings");

		var cssAddon = {};
		t.updateElementVisual(objAddon, cssAddon, objSettings, "addon");
	}

	/**
	 * update addon container settings
	 */
	this.updateAddonContainerSettings = function(objAddon, settings, isGridInit) {

		t.validateColAddonElement(objAddon);

		objAddon.data("addon_settings", settings);

		t.updateAddonContainerVisual(objAddon, settings);

		t.triggerEvent(t.events.ADDON_CONTAINER_SETTINGS_UPDATED, [ objAddon, {
			"is_grid_init" : isGridInit
		} ]);

	}

	/**
	 * get space between addons of the column
	 */
	this.getColSpaceBetweenAddons = function(objColSettings, objAddon) {

		var index = objAddon.index();
		if (index == 1)
			return (0);

		if (!objColSettings)
			objColSettings = {};

		if (objColSettings.hasOwnProperty("col_space_between_addons"))
			return (objColSettings["col_space_between_addons"]);

		var objContainer = t.getParentRowContainer(objAddon);

		var settingsContainer = t.getRowContainerSettings(objContainer);
		if (settingsContainer.hasOwnProperty("col_space_between_addons"))
			return (settingsContainer["col_space_between_addons"]);

		var objRow = t.getParentRow(objContainer);
		var spaceBetweenAddons = t.getRowSetting(objRow, "space_between_addons",
				true);

		return (spaceBetweenAddons);
	}

	/**
	 * update placeholders
	 */
	this.updateAddonContainerSettingsPlaceholders = function(objColSettings, objAddon) {

		var arrSettingNames = jQuery.extend([], t.g_temp.settings_placeholders);

		// get space between addons
		var spaceBetweenAddons = t.getColSpaceBetweenAddons(objColSettings,
				objAddon);

		var objPlaceholders = {};
		objPlaceholders["margin_top"] = spaceBetweenAddons;

		objPlaceholders = t.getObjSizePlaceholders(arrSettingNames,
				objPlaceholders, objColSettings, null, null, "0");

		if (t.g_panel)
			t.g_panel.updatePlaceholders("addon-container-settings", objPlaceholders);
	}

	/**
	 * apply column settings
	 */
	this.applyAddonContainerSettings = function() {
		
		var data = t.g_panel.getPaneData("addon-container-settings");

		var addonID = data.objectID;
		var objAddon = jQuery("#" + addonID);

		if (objAddon.length == 0)
			return (false);

		t.updateAddonContainerSettings(objAddon, data.settings);

		if (t.g_panel.isVisible() == true)
			t.updateAddonContainerSettingsPlaceholders(data.settings, objAddon);
	}

	/**
	 * open addon container settings
	 */
	this.openAddonContainerSettingsPanel = function(objAddon) {
		
		var addonID = objAddon.prop("id");

		var addonContainerSettings = objAddon.data("addon_settings");
		
		t.g_panel.open("addon-container-settings", addonContainerSettings, addonID);

		t.updateAddonContainerSettingsPlaceholders(addonContainerSettings, objAddon);
	}

	/**
	 * apply addon settings
	 */
	this.applyAddonSettings = function(params) {
		
		var isInstant = params["is_instant"];
		var addonID = params["object_id"];
		var objAddon = jQuery("#" + addonID);
		if (objAddon.length == 0)
			return (false);

		var fieldName = g_ucAdmin.getVal(params, "name");

		// update instant field if available

		// try instant update
		if (isInstant == true) {

			var objField = objAddon.find("span[data-uc_font_field=" + fieldName + "]");
			if (objField.length) {
				var value = g_ucAdmin.getVal(params, "value");
				objField.html(value);
			}

			return (false);
		}

		var data = t.g_panel.getPaneData("addon-settings");

		var settings = data.settings;

		var addonDataNew = g_objAddonConfig.getAddonDataFromSettingsValues(settings);
		
		var addonData = t.g_objGridData.getColAddonData(objAddon);
		
		addonData = g_objAddonConfig.setNewAddonData(addonData, addonDataNew);
		
		t.saveAddonElementData(objAddon, addonData);
		
		t.redrawAddon(objAddon);
	}
	
	
	/**
	 * redraw addon by it's data
	 */
	this.redrawAddonByData = function(objAddon, addonDataNew){
		
		var addonData = t.g_objGridData.getColAddonData(objAddon);
		
		addonDataNew.extra = addonData.extra;
		
		t.saveAddonElementData(objAddon, addonDataNew);
				
		t.redrawAddon(objAddon);
				
	}
	
	
	/**
	 * open addon settings panel from addon object the command will send command
	 * to later
	 */
	this.openAddonSettingsPanel = function(objAddon, command, isRefresh) {
				
		t.validateColAddonElement(objAddon);

		var addonData = t.g_objGridData.getColAddonData(objAddon);

		var title = g_objAddonConfig.getAddonTitle(addonData);
		// title = g_uctext["edit_addon"] + ": "+title;

		var sendData = g_objAddonConfig.getSendDataFromAddonData(addonData);
		
		var addonID = objAddon.prop("id");

		var data = {};
		if (command)
			data.command = command;
		
		if(isRefresh === true)
			data.refresh = true;
		
		var panelData = g_objAddonConfig.getPanelData(addonData);
		if (panelData)
			jQuery.extend(data, panelData);
				
		t.g_panel.open("addon-settings", sendData, addonID, title, data);
	}
	

	this.____________ADDONS______________ = function(){}


	/**
	 * generate serial ID from data and serial
	 */
	this.generateAddonSerialID = function(addonData) {

		var name = addonData.name;
		t.g_vars.serial++;

		var serialID = name + "_" + t.g_vars.serial;

		return (serialID);
	}

	/**
	 * modify addon data before adding addon
	 */
	this.modifyAddonDataBeforeAdd = function(addonData) {

		var extra = g_ucAdmin.getVal(addonData, "extra", {});
		if (!extra)
			extra = {};

		extra["serial_id"] = t.generateAddonSerialID(addonData);
		extra["serial_contentid"] = t.g_vars.addon_conetentid_prefix + "_"+ addonData.name + "_" + t.g_vars.serial;
		
		addonData.extra = extra;

		return (addonData);
	}


	/**
	 * save addon element data
	 */
	this.saveAddonElementData = function(objAddon, addonData, isGridInit) {

		t.validateColAddonElement(objAddon);

		var newAddonData = jQuery.extend({}, addonData);

		var addonName = newAddonData.name;

		objAddon.data("addon_name", addonName);
		objAddon.data("addon_data", newAddonData);

		var options = g_ucAdmin.getVal(addonData, "options");
		if (options)
			objAddon.data("addon_settings", options);

		t.triggerEvent(t.events.ADDON_SETTINGS_UPDATED, [ objAddon, {
			"is_grid_init" : isGridInit
		} ]);
		
		
		
	}

	/**
	 * get addon error message html
	 */
	this.getHtmlAddonError = function(message, addonData) {

		var title = addonData.name;
		var extra = g_ucAdmin.getVal(addonData, "extra");
		var extraTitle = g_ucAdmin.getVal(extra, "title");
		if (extraTitle)
			title = extraTitle;

		var html = "<div class='uc-grid-addon-error'>";
		html += "Error in " + title + " addon: <br>";
		html += message;
		html += "</div>";

		return (html);
	}

	/**
	 * update column with addon data
	 */
	this.addColAddon = function(objCol, addonData, isGridInit, placeholderID, objAddonBefore, addDirection){
		
		if (jQuery.isArray(addonData))
			return (false);

		addonData = t.modifyAddonDataBeforeAdd(addonData);

		var objAddonsWrapper = t.getColAddonsWrapper(objCol);

		// save data
		t.g_objAddonHTML.generateAddonHtml(addonData, function(htmlAddon) {
			
			var objHtml = jQuery(htmlAddon);

			t.saveAddonElementData(objHtml, addonData, isGridInit);

			var errorReturned = null;

			try {

				var objAddon = jQuery(objHtml);

				// update container settings
				var addonOptions = g_ucAdmin.getVal(addonData, "options");
				if (addonOptions)
					t.updateAddonContainerSettings(objAddon, addonOptions, isGridInit);
				
				// put addon in it's place
				if (placeholderID) {
					
					var objPlaceholder = jQuery("#" + placeholderID);
					if (objPlaceholder.length == 0)
						throw new Error("t.addColAddon: Placeholder not found: " + placeholderID);
					
					objPlaceholder.replaceWith(objHtml);

				} else {
					if (objAddonBefore && objAddonBefore.length) {
						
						if(addDirection == "before")
							objAddon.insertBefore(objAddonBefore);
						else
							objAddon.insertAfter(objAddonBefore);
						
						
					} else
						objAddonsWrapper.append(objHtml);
				}

				t.triggerEvent(t.events.ADDON_ADDED, {
					"addon" : objAddon,
					"is_grid_init" : isGridInit
				});

			} catch(error) {
				
				var htmlErrorMessage = t.getHtmlAddonError(error, addonData);
				objHtml.append(htmlErrorMessage);

				trace("js error in " + addonData.name + " addon: " + error);

				errorReturned = "Javascript Error Occured: " + error;
			}

			t.triggerEvent(t.events.COL_ADDONS_UPDATED, [ objCol, {
				"is_grid_init" : isGridInit
			} ]);

			if (errorReturned)
				throw errorReturned;

		});

	}

	
	/**
	 * redraw addon
	 */
	this.redrawAddon = function(objAddon) {

		t.validateColAddonElement(objAddon);

		// remove the output
		var addonData = t.g_objGridData.getColAddonData(objAddon);
		addonData["output"] = null;

		// no modify serial id before redraw
		// addonData = t.modifyAddonDataBeforeAdd(addonData);

		t.saveAddonElementData(objAddon, addonData);

		// generate html
		t.g_objAddonHTML.generateAddonHtml(addonData, function(htmlAddon) {
			
			objAddon.html(htmlAddon);

		}, true);

	}

	/**
	 * update obj addon with new one
	 */
	this.updateColAddon = function(objAddon, addonData) {

		g_ucAdmin.validateNotEmpty(addonData, "addon data");
		var addonID = objAddon.prop("id");

		t.validateColAddonElement(objAddon);

		addonData = t.modifyAddonDataBeforeAdd(addonData);

		// generate html
		t.g_objAddonHTML.generateAddonHtml(addonData, function(htmlAddon) {

			var objAddonNew = jQuery(htmlAddon);

			objAddon.replaceWith(objAddonNew);

			t.saveAddonElementData(objAddonNew, addonData);

			// previous addon element removed
			t.triggerEvent(t.events.ELEMENT_REMOVED, addonID);

		});

	}

	/**
	 * update addon data without redraw the actual addon run after content
	 * editing
	 */
	this.updateAddonDataSettingNoRedraw = function(objAddon, settingName, settingValue) {
		
		t.validateColAddonElement(objAddon);
		var addonData = t.g_objGridData.getColAddonData(objAddon);

		addonData = g_objAddonConfig.updateAddonDataSetting(addonData,
				settingName, settingValue);

		t.saveAddonElementData(objAddon, addonData);

		var isItems = false;
		if (settingName.indexOf("uc_items_attribute") === 0)
			isItems = true;
		
		if (isItems == true) {
			t.updatePanelSettingIfActive(objAddon, "uc_items_editor", addonData.items, true);

		} else {
			t.updatePanelSettingIfActive(objAddon, settingName, settingValue, true);
		}

	}

	/**
	 * open addon browser
	 */
	this.openAddonsBrowser = function(objElement, objAddonBefore) {
		
		if (!g_objBrowser) {
			trace("browser not available");
			return (false);
		}

		var isNew = true;
		var addonData = null;

		var type = t.getElementType(objElement);

		if (type == "addon") { // edit addon

			t.openAddonSettingsPanel(objElement);
			return (false);
		}

		g_objBrowser.openAddonsBrowser(addonData, function(newAddonData) {

			t.setColEmptyStateLoading(objElement, true);

			newAddonData.return_output = t.g_temp.is_live_view;

			g_objAddonConfig.loadNewAddonData(newAddonData,
					function(addonData) {

						t.setColEmptyStateLoading(objElement, false);

						t.addColAddon(objElement, addonData, false, null,objAddonBefore);

					});

		});

	}

	
	
	this.____________GRID_SETTINGS______________ = function(){}
	
	/**
	 * update grid settings placeholders
	 */
	this.updateGridSettingsPlaceholders = function() {
		
		var objOptions = t.getCombinedOptions();

		var arrSettings = jQuery.extend([], t.g_temp.settings_placeholders_grid);

		var objPlaceholders = {};
		objPlaceholders = t.getObjSizePlaceholders(arrSettings,
				objPlaceholders, arrSettings, objOptions, null, "0");

		if (t.g_panel)
			t.g_panel.updatePlaceholders("grid-settings", objPlaceholders);
	}

	/**
	 * open grid settings panel
	 */
	this.openGridSettingsPanel = function() {
		
		t.g_panel.open("grid-settings", null);

		t.updateGridSettingsPlaceholders();
	}

	/**
	 * open page params panel
	 */
	this.openPageParamsPanel = function() {
		
		// update settings
		t.g_panel.open("page-params", t.g_temp.page_params);
	}

	/**
	 * get grid option
	 */
	this.getGridOption = function(name) {

		var gridOptions = t.getCombinedOptions();

		var value = g_ucAdmin.getVal(gridOptions, name);

		return (value);
	}

	/**
	 * get combined options
	 */
	this.getCombinedOptions = function() {

		if (!t.g_optionsCustom)
			t.g_optionsCustom = {};

		var objOptions = {};
		jQuery.extend(objOptions, t.g_options, t.g_optionsCustom);

		return (objOptions);
	}

	/**
	 * get font css from object
	 */
	this.getFontCss = function(objFont, selector) {
		
		var arrStyle = {};
		var cssMobileSize;
		var customStyles = "";

		for ( var styleName in objFont) {
			var styleValue = objFont[styleName];

			if (!styleValue)
				continue;

			if (styleValue == "not_chosen")
				continue;

			switch (styleName) {
			case "font-family":
				if (styleValue.indexOf(" ") != -1
						&& styleValue.indexOf(",") == -1)
					arrStyle[styleName] = "'" + styleValue + "'";
				else
					arrStyle[styleName] = styleValue;

				// add google font
				if (t.g_temp.google_fonts.hasOwnProperty(styleValue)) {
					var urlGoogleFont = "https://fonts.googleapis.com/css?family="
							+ styleValue;
					g_ucAdmin.loadIncludeFile("css", urlGoogleFont, {
						"norand" : true
					});
				}

				break;
			case "font-weight":
			case "line-height":
			case "text-decoration":
			case "color":
			case "font-style":
				arrStyle[styleName] = styleValue;
				break;
			case "font-size":
				styleValue = g_ucAdmin.normalizeSizeValue(styleValue);
				arrStyle[styleName] = styleValue;
				break;
			case "mobile-size":
				styleValue = g_ucAdmin.normalizeSizeValue(styleValue);
				cssMobileSize = selector + "{font-size:" + styleValue + "}";
				cssMobileSize = g_ucAdmin.wrapCssInMobile(cssMobileSize);
				break;
			case "custom":
				customStyles = styleValue;
				break;
			}
		}

		// make this function
		var css = g_ucAdmin.arrCssToStrCss(arrStyle);

		if (customStyles)
			css += customStyles;

		css = selector + "{" + css + "}\n";

		if (cssMobileSize)
			css += cssMobileSize + "\n";

		return (css);
	}

	/**
	 * get size related css
	 */
	this.getSizeRelatedCss = function(arrSettingsNames, objOptions, selector, suffix,funcModify){
		
		var br = "\n";
		var tab = "	    ";

		var css = "";
		var arrSizes = t.g_temp.sizes;
		for ( var indexSize in arrSizes) {
			var size = arrSizes[indexSize];

			var objSizeCss = {};

			for ( var cssAttribute in arrSettingsNames) {

				var settingName = arrSettingsNames[cssAttribute];

				var settingNameSize = settingName + "_" + size;

				var value = g_ucAdmin.getVal(objOptions, settingNameSize);

				if (value !== "") {
					objSizeCss[cssAttribute] = g_ucAdmin
							.normalizeSizeValue(value);

					if (suffix && cssAttribute != "inline-css")
						objSizeCss[cssAttribute] += " " + suffix;

				}

				if (funcModify && typeof funcModify == "function")
					objSizeCss = funcModify(objSizeCss, cssAttribute,
							settingName, value, size);
			}

			if (jQuery.isEmptyObject(objSizeCss) == true)
				continue;

			var cssSize = g_ucAdmin.arrCssToStrCss(objSizeCss, selector, true);

			cssSize = g_ucAdmin.wrapCssInMobile(br + cssSize, size);

			css += br + cssSize;
		}

		return (css);
	}

	/**
	 * put css based on the options
	 */
	this.putGeneratedCss = function() {
		
		var br = "\n";
		var tab = "	    ";
		var objOptions = t.getCombinedOptions();

		var css = "";
		
		g_ucAdmin.validateObjProperty(objOptions, [ "col_gutter", "page_css",
				"row_padding_top", "row_padding_bottom",
				"row_padding_top_mobile", "row_padding_bottom_mobile",
				"row_container_width" ], "grid options");

		// row css
		var selectorRow = g_gridID + " .uc-grid-row";

		css += selectorRow + "{" + br;
		css += tab + "padding-top:" + objOptions.row_padding_top + "px;" + br;
		css += tab + "padding-bottom:" + objOptions.row_padding_bottom + "px;"
				+ br;
		css += "}" + br + br;

		// row container css
		css += g_gridID + " .uc-grid-row .uc-grid-row-container{" + br;

		css += tab + "max-width:"
				+ g_ucAdmin.normalizeSizeValue(objOptions.row_container_width)
				+ ";" + br;

		css += "}" + br + br;

		// column css
		var selectorCol = g_gridID + " .uc-grid-row .uc-grid-col";

		css += selectorCol + "{" + br;

		// add gutter
		css += tab + "padding-left:" + objOptions.col_gutter + "px;" + br;
		css += tab + "padding-right:" + objOptions.col_gutter + "px;" + br;
		css += "}" + br;

		// column addons
		var selectorAddons = g_gridID + " .uc-grid-row .uc-grid-col .uc-grid-col-addon + .uc-grid-col-addon";
		var selectorFirstAddons = g_gridID + " .uc-grid-row .uc-grid-col .uc-grid-overlay-empty + .uc-grid-col-addon";
		
		var spaceBetweenAddons = g_ucAdmin.getVal(objOptions, "space_between_addons", null);
		if (spaceBetweenAddons) {

			spaceBetweenAddons = g_ucAdmin
					.normalizeSizeValue(spaceBetweenAddons);
			var addonsStyle = "margin-top:" + spaceBetweenAddons + ";";
			css += selectorAddons + "{" + addonsStyle + "}" + br;

			css += selectorFirstAddons + "{margin-top:0px}" + br;
		}

		// put font styles
		var objPageFonts = g_ucAdmin.getVal(objOptions, "page_fonts", null);
		if (objPageFonts) {
			for ( var fontName in objPageFonts) {
				var objFontData = objPageFonts[fontName];
				var fontSelector = g_gridID + " .uc-page-font-" + fontName;
				if (fontName == "page")
					fontSelector = g_gridID + " .uc-grid-col-addon-html";

				var cssFont = t.getFontCss(objFontData, fontSelector);

				css += "\n" + cssFont;
			}

		}

		t.g_objGrid.removeAttr("style");

		// body background
		var cssBG = t.getBackgroundCss(objOptions);
		if (cssBG)
			t.g_objGrid.css(cssBG);

		// add page css
		var pageCss = objOptions.page_css;
		pageCss = jQuery.trim(pageCss);
		if (pageCss)
			css += pageCss;

		var sizeRelatedCss = "";

		// row mobile
		var arrSettings = {
			"padding-top" : "row_padding_top",
			"padding-bottom" : "row_padding_bottom"
		};

		sizeRelatedCss += t.getSizeRelatedCss(arrSettings, objOptions,
				selectorRow);

		// columns mobile
		var arrSettings = {
			"padding-left" : "col_gutter",
			"padding-right" : "col_gutter"
		};

		sizeRelatedCss += t.getSizeRelatedCss(arrSettings, objOptions,
				selectorCol);

		// addons mobile
		var arrSettings = {
			"margin-top" : "space_between_addons"
		};
		sizeRelatedCss += t.getSizeRelatedCss(arrSettings, objOptions,
				selectorAddons);

		// size related inline css
		var arrSettings = {
			"inline-css" : "page_css"
		};

		sizeRelatedCss += t.getSizeRelatedCss(arrSettings, objOptions, "");

		css += sizeRelatedCss;

		t.g_objStyle.html(css);

		// apply the scroll
		var isScroll = g_ucAdmin.getVal(objOptions, "enable_smooth_scroll");
		isScroll = g_ucAdmin.strToBool(isScroll);

		if (isScroll == true)
			t.g_objGrid.addClass("uc-smooth-scroll");
		else
			t.g_objGrid.removeClass("uc-smooth-scroll");

	}
	
	/**
	 * modify options before update
	 */
	this.modifyGridValuesUpdate = function(objValues){
		
		var objModifiedSettings = {};
		
		jQuery.each(objValues, function(option, val) {
			if (!val || jQuery.trim(val) == "")
				return (true);

			// convert to int
			if (typeof val == "string" && jQuery.isNumeric(val))
				val = parseInt(val);

			objModifiedSettings[option] = val;
		});
		
		return(objModifiedSettings);
	}
	
	/**
	 * update options from settings dialog
	 */
	this.updateOptionsFromGridSettings = function(){
		
		if (!t.g_panel)
			return (false);

		var objValues = t.g_panel.getSettingsValues("grid-settings");
		
		// update custom options, skip empty values
		t.g_optionsCustom = t.modifyGridValuesUpdate(objValues);
		
	}
	
	/**
	 * apply grid settings from options
	 */
	this.applyGridSettingsFromData = function(objValues){
		
		if(objValues)
			t.g_optionsCustom = t.modifyGridValuesUpdate(objValues);
		
		t.putGeneratedCss();
		t.updateAllRowsVisual();
		t.updateGridSettingsPlaceholders();
		
		t.triggerEvent(t.events.GRID_SETTINGS_UPDATED);
		
	}
	
	/**
	 * apply grid options
	 */
	this.applyGridSettings = function() {
		
		t.updateOptionsFromGridSettings();
		t.applyGridSettingsFromData();
	}

	/**
	 * save page params to object
	 */
	this.applyPageParams = function() {
		
		var objParams = t.g_panel.getSettingsValues("page-params");
		t.g_temp.page_params = objParams;
		
		t.triggerEvent(t.events.PAGE_PARAMS_UPDATED);
	}

	/**
	 * init grid settings related, style, options, dialogs
	 */
	this.initGridSettings = function() {
		
		// init style object
		t.g_objStyle = t.g_objWrapper.children("style");
		g_ucAdmin.validateDomElement(t.g_objStyle, "style tag");

		// init options
		t.g_options = t.g_objGrid.data("options");
		if (!t.g_options)
			throw new Error("Should be passed some options!");

		t.g_objGrid.removeAttr("data-options"); // remove attribute for not
												// interfere
	}

	this.____________EVENTS______________ = function(){}
	
	/**
	 * trigger event that some grid change happened, based on main event
	 */
	this.checkTriggerChangeEvent = function(eventName, options) {

		// skip if grid not inited
		if (t.g_temp.is_data_inited == false)
			return (false);

		// skip grid init related events
		if (options) {
			if (jQuery.isArray(options))
				var objEventParams = options[1];
			else
				var objEventParams = options;

			var isGridInit = g_ucAdmin.getVal(objEventParams, "is_grid_init");
			if (isGridInit === true)
				return (false);

		}

		// trigger change event
		t.g_objGrid.trigger(t.events.CHANGE_TAKEN, [ eventName, options ]);

	}

	/**
	 * grigger event
	 */
	this.triggerEvent = function(eventName, options) {

		t.g_objGrid.trigger(eventName, options);

		t.checkTriggerChangeEvent(eventName, options);

	}

	/**
	 * on some event
	 */
	this.onEvent = function(eventName, func) {
		
		t.g_objGrid.on(eventName, func);

	}

	/**
	 * on rows updated happends on add / update / delete / reorder row
	 */
	this.onRowsUpdated = function(event) {
		
		// update row classes
		var numRows = t.getNumRows();
		if(numRows <= 1)
			t.g_objGrid.addClass("uc-single-row");
		else
			t.g_objGrid.removeClass("uc-single-row");

		t.updateRowsIcons();
	}

	/**
	 * on add column
	 */
	this.onRowColumnsUpdated = function(event, container) {
		
		var objContainer = jQuery(container);
		t.validateRowContainer(objContainer);

		t.updateColsClasses(objContainer);

		t.updateColOperationButtons(objContainer);

		var objRow = t.getParentRow(objContainer);
		t.updateRowVisual_buttons(objRow);
	}

	/**
	 * on col addons updated function show / hide empty visual if no addons
	 */
	this.onColAddonsUpdated = function(event, objCol) {
				
		objCol = jQuery(objCol);

		var numAddons = t.getNumColAddons(objCol);
		
		if(numAddons <= 0)
			objCol.addClass(t.g_vars.class_empty);
		else
			objCol.removeClass(t.g_vars.class_empty);
			
	}

	/**
	 * on col or row action icon click
	 */
	this.onActionIconClick = function() {
		
		var objIcon = jQuery(this);

		if (g_ucAdmin.isButtonEnabled(objIcon) == false)
			return (false);

		var action = objIcon.data("action");
		var actionType = objIcon.data("actiontype");

		if (!action || action == "")
			throw new Error("wrong icon action");

		switch (actionType) {
		case "grid":
			var objRow = t.getParentRow(objIcon);
			t.doGridAction(action, objRow);
			break;
		case "col":
			var objAddon = t.getParentCol(objIcon);
			t.doColAction(objAddon, action);
			break;
		case "row":
			var objRow = t.getParentRow(objIcon);
			t.doRowAction(action, objRow);
			break;
		case "addon":
			var objAddon = t.getParentAddonElement(objIcon);

			t.doAddonAction(objAddon, action);
			break;
		case "container":
			var objContainer = t.getParentRowContainer(objIcon);
			t.doContainerAction(action, objContainer);
			break;
		default:
			throw new Error("Wrong action type: " + actionType);
			break;
		}

	}

	/**
	 * on addon placeholder click - open addons browser to insert new addon
	 */
	this.onAddonPlaceholderClick = function(){
		
		var objElement = jQuery(this);
		
		var objCol = t.getParentCol(objElement);
		
		t.openAddonsBrowser(objCol);
		
	}
	
	/**
	 * init new row events
	 */
	this.onNewRowAdded = function(event, row) {
		
		var objRow = jQuery(row);

		t.initSortableContainers(objRow);
			
		// show settings if the panel is visible
		if (t.g_temp.is_data_inited == true && t.g_panel && t.g_panel.isVisible() == true)
			t.openRowSettingsPanel(objRow);

	}


	/**
	 * on new container added, init new container events
	 */
	this.onNewContainerAdded = function(event, container) {
		
		var objContainer = jQuery(container);

		t.initSortableColumns(objContainer);
	}

		
	/**
	 * on row duplicated - update all columns and row visual
	 */
	this.onRowDuplicated = function(event, row) {
		
		var objRow = jQuery(row);

		t.initSortableContainers(objRow);

		t.updateRowVisual(objRow);

		// send event to all containers

		var objContainers = t.getRowContainersAll(objRow);

		objContainers.each(function(index, container) {
			var objContainer = jQuery(container);

			// update IDs
			var containerID = t.getNewElementID("container");
			objContainer.attr("id", containerID);

			t.triggerEvent(t.events.ROW_CONTAINER_DUPLICATED, objContainer);
		});

	}

	/**
	 * on container duplicated, send event to all cols
	 */
	this.onRowContainerDuplicated = function(event, container) {
		
		var objContainer = jQuery(container);

		t.initSortableColumns(objContainer);

		t.updateRowContainerVisual(objContainer);

		var objCols = t.getCols(objContainer);

		jQuery(objCols).each(function(index, col) {
			var objCol = jQuery(col);

			// update IDs
			var colID = t.getNewElementID("col");
			objCol.attr("id", colID);

			t.triggerEvent(t.events.COL_DUPLICATED, objCol);
		});

	}

	/**
	 * on column duplicated, update all addons and col visual
	 */
	this.onColDuplicated = function(event, col) {
		
		var objCol = jQuery(col);

		t.updateColVisual(objCol);

		var objAddons = t.getElementAddons(objCol);

		jQuery(objAddons).each(function(index, addon) {
			var objAddon = jQuery(addon);
			t.triggerEvent(t.events.ADDON_DUPLICATED, objAddon);
		});

	}

	/**
	 * on panel settings change
	 */
	this.onPanelSettingsChange = function(event, params) {
		
		var settingsType = params["object_name"];

		switch (settingsType) {
		case "grid-settings":
			t.applyGridSettings(params);
			break;
		case "page-params":
			t.applyPageParams(params);
			break;
		case "row-settings":
			t.applyRowSettings(params);
			break;
		case "col-settings":
			t.applyColSettings(params);
			break;
		case "addon-container-settings":
			t.applyAddonContainerSettings(params);
			break;
		case "addon-settings":
			t.applyAddonSettings(params);
			break;
		case "container-settings":
			t.applyRowContainerSettings(params);
			break;
		default:
			// console.trace();
			throw new Error("t.onPanelSettingsChange: Wrong settings type: "+ settingsType);
			break;
		}

	}

	/**
	 * on panel html settings loaded from ajax save settings in addon data
	 */
	this.onPanelHtmlSettingsLoaded = function(event, data) {

		var addonID = data["object_id"];

		var objAddon = jQuery("#" + addonID);
		if (objAddon.length == 0)
			return (true);

		var htmlSettings = data["html_settings"];
		if (!htmlSettings)
			return (true);

		var addonData = t.g_objGridData.getColAddonData(objAddon);

		addonData = g_objAddonConfig.setHtmlSettingsInAddonData(addonData, htmlSettings);

		t.saveAddonElementData(objAddon, addonData);
	}

	
	/**
	 * calling after delete or update addon
	 */
	this.onElementRemoved = function(event, elementID) {

		if (t.g_temp.is_live_view == true)
			t.g_objAddonHTML.checkRemoveLiveModeIncludes();
		
		if (elementID && t.g_panel)
			t.g_panel.hideIfActive(elementID);
		
	}
	
	
	/**
	 * on addon duplicated, modify content
	 */
	this.onAddonDuplicated = function(event, addonCopy) {
		
		var objAddonCopy = jQuery(addonCopy);

		// modify data
		var addonData = t.g_objGridData.getColAddonData(objAddonCopy);

		addonData = t.modifyAddonDataBeforeAdd(addonData);

		addonData = g_objAddonConfig.clearDuplicatedAddonData(addonData);

		t.saveAddonElementData(objAddonCopy, addonData);

		t.g_objAddonHTML.setDuplicatedAddonHtml(objAddonCopy, addonData);
	}

	/**
	 * trigger after new addon added
	 */
	this.onAddonAdded = function(event, data) {
		
		var objAddon = data["addon"];
		var isGridInit = data["is_grid_init"];
		
		//t.initEvents_sortableAddon(objAddon);
		
		if (isGridInit === true)
			return(true);

		var addonData = t.g_objGridData.getColAddonData(objAddon);
		var command = g_objAddonConfig.getPanelCommand("add_addon", addonData);

		// if(!command)
		// return(true);
		
		t.openAddonSettingsPanel(objAddon, command);
		
	}

	/**
	 * on containers updated, set row and containre classes related to number of
	 * container
	 */
	this.onContainersUpdated = function(event, row) {
		
		var objRow = jQuery(row);

		t.validateRow(objRow);

		var objContainers = t.getRowContainersAll(objRow);
		var numContainers = objContainers.length;

		if (numContainers == 0)
			return (false);

		var firstContainer = objContainers.first();
		var lastContainer = objContainers.last();

		// set first container class
		if (numContainers > 1) {
			objContainers.not(firstContainer).removeClass("uc-first-container");
			objContainers.not(lastContainer).removeClass("uc-last-container");
		}

		firstContainer.addClass("uc-first-container");
		lastContainer.addClass("uc-last-container");

	}

	/**
	 * on install addon if svg - add to svg content
	 */
	this.onInstallAddon = function(event, data) {

		var addonType = g_ucAdmin.getVal(data, "addontype");
		if (addonType != "shape_devider")
			return (true);

		var shapeContent = g_ucAdmin.getVal(data, "shape_content");
		if (!shapeContent)
			throw new Error("Wrong shape content");

		t.g_vars.shape_deviders[data.name] = shapeContent;

	}

	/**
	 * on contenteditable field change event, update parent addon and the panel
	 * field if available
	 */
	this.onEditableFieldChange = function(event) {

		var objStyle = jQuery(this);

		var settingName = objStyle.data("uc_font_field");
		var objAddon = objStyle.parents(".uc-grid-col-addon");
		var value = objStyle.html();

		// clear content if pasted
		var isJustPasted = objStyle.data("just_pasted");

		// on paste - clear html tags
		if (isJustPasted === true) {

			var valueBeforePaste = objStyle.data("value_before_paste");

			var diffText = g_ucAdmin.getTextDiff(valueBeforePaste, value);

			var newText = g_ucAdmin.stripTagsKeepFormatting(diffText);

			value = value.replace(diffText, newText);

			objStyle.html(value);

			var posStart = value.indexOf(newText);
			var posEnd = posStart + newText.length;

			g_ucAdmin.setCursorPosition(objStyle, posEnd);

			objStyle.data("just_pasted", null);
			objStyle.data("value_before_paste", null);
		}

		t.updateAddonDataSettingNoRedraw(objAddon, settingName, value);
	}

	/**
	 * on paste, clear the html data
	 */
	this.onEditableFieldPaste = function(event) {

		var objStyle = jQuery(this);

		var value = objStyle.html();

		objStyle.data("just_pasted", true);
		objStyle.data("value_before_paste", value);

	}

	/**
	 * keydown event
	 */
	this.onKeyDown = function(event){
		
		if(event.hasOwnProperty("keyCode") == false)
			return(true);
		
		var keyCode = event.keyCode;
		var ctrlKey = event.ctrlKey;
		
		if(keyCode == 90 && ctrlKey == true)
			t.undo();
		
	}
	
	this.____________HOVER_EVENTS______________ = function(){}

	/**
	 * on panel head mouseover show borders of relative element
	 */
	this.onPanelHeadMouseOver = function(event, data) {

		var objectID = data.objectID;

		t.setElementAlwaysHoverMode(objectID, true);
	}

	/**
	 * on panel head mouseover hide borders of relative element
	 */
	this.onPanelHeadMouseOut = function(event, data) {

		var objectID = data.objectID;

		t.setElementAlwaysHoverMode(objectID, false);
	}

	/**
	 * on panel mouse over blink with active element
	 */
	this.onPanelMouseOver = function(event, data) {

		// var objectID = data.objectID;
		// t.setElementAlwaysHoverMode(objectID, true, true);
	}

	/**
	 * set hover mode to the element
	 */
	this.setElementAlwaysHoverMode = function(elementID, set, isTimeout) {
		
		if (typeof elementID == "string") {

			var objElement = jQuery("#" + elementID);
			if (objElement.length == 0)
				return (false);

		} else
			var objElement = elementID;

		var type = t.getElementType(objElement);

		switch (type) {
		case "row":
		case "column":
		case "addon":
		case "container":
			break;
		default:
			return (false);
			break;
		}

		if (set == true) {
			objElement.addClass("uc-grid-always-hover");
			if (isTimeout == true) {
				setTimeout(function() {
					objElement.removeClass("uc-grid-always-hover");
				}, 800);
			}
		} else
			objElement.removeClass("uc-grid-always-hover");

	}

	/**
	 * remove all element hover modes
	 */
	this.removeAllElementHoverMode = function() {

		t.g_objGrid.find("uc-grid-always-hover").removeClass(
				"uc-grid-always-hover");

	}

	/**
	 * check some element hover mode, set mode2 if there is some other hover
	 * elements children
	 */
	this.checkElementHoverMode = function(objElement) {
		
		var hoverChildren = objElement.find("."+t.g_vars.class_element_hover);
		if (hoverChildren.length == 0)
			objElement.removeClass("uc-over-mode2");
		else
			objElement.addClass("uc-over-mode2");

	}

	/**
	 * check parent hover mode
	 */
	this.checkParentsHoverMode = function(objElement) {
		
		// check column
		var objCol = t.getParentCol(objElement);
		if (objCol.length)
			t.checkElementHoverMode(objCol);

		// check container
		var objContainer = t.getParentRowContainer(objElement);
		if (objContainer.length)
			t.checkElementHoverMode(objContainer);

		// check row
		var objRow = t.getParentRow(objElement);
		if (objRow.length)
			t.checkElementHoverMode(objRow);

	}
	
	/**
	 * on element mouse over
	 */
	this.onElementMouseOver = function() {

		var objElement = jQuery(this);

		objElement.addClass(t.g_vars.class_element_hover);

		t.checkParentsHoverMode(objElement);
	}

	/**
	 * on element mouse out
	 */
	this.onElementMouseOut = function() {

		var objElement = jQuery(this);

		objElement.removeClass(t.g_vars.class_element_hover);

		t.checkParentsHoverMode(objElement);
	}

	/**
	 * on element double click
	 */
	this.onElementDoubleClick = function(event) {
		
		event.stopPropagation();
		event.stopImmediatePropagation();

		var objElement = jQuery(this);

		t.openElementSettingsPanel(objElement);

	}

	/**
	 * on sort addons start
	 */
	this.onSortElementStart = function() {
		
		t.g_objGrid.addClass("uc-drag-mode");
	}

	/**
	 * on sort stop
	 */
	this.onSortElementStop = function() {
		
		t.g_objGrid.removeClass("uc-drag-mode");
	}

	
	this.____________________GET_DATA________________ = function() {}

	/**
	 * get grid data
	 */
	this.getGridData = function() {

		var data = {};
		data.rows = t.g_objGridData.getGridData_rows();

		if (t.g_optionsCustom)
			data.options = t.g_optionsCustom;

		return (data);
	}


	/**
	 * get page params
	 */
	this.getPageParams = function() {
		
		if (!t.g_temp.page_params)
			return (null);

		return (t.g_temp.page_params);
	};

	
	this.____________INIT______________ = function() {}
	
	/**
	 * init the events
	 */
	this.initEvents = function() {
		
		t.onEvent(t.events.ROW_COLUMNS_UPDATED, t.onRowColumnsUpdated);
		t.onEvent(t.events.ROWS_UPDATED, t.onRowsUpdated);
		t.onEvent(t.events.ROW_ADDED, t.onNewRowAdded);
		t.onEvent(t.events.ROW_CONTAINER_ADDED, t.onNewContainerAdded);
		t.onEvent(t.events.COL_ADDONS_UPDATED, t.onColAddonsUpdated);
		t.onEvent(t.events.ELEMENT_REMOVED, t.onElementRemoved);
		t.onEvent(t.events.ADDON_ADDED, t.onAddonAdded);
		t.onEvent(t.events.ROW_DUPLICATED, t.onRowDuplicated);
		t.onEvent(t.events.ROW_CONTAINER_DUPLICATED, t.onRowContainerDuplicated);
		t.onEvent(t.events.COL_DUPLICATED, t.onColDuplicated);
		t.onEvent(t.events.ADDON_DUPLICATED, t.onAddonDuplicated);
		t.onEvent(t.events.ROW_CONTAINERS_UPDATED, t.onContainersUpdated);
		
		
		t.g_objGrid.on("click", ".uc-grid-action-icon", t.onActionIconClick);
		
		t.g_objGrid.on("click", ".uc-grid-element-empty-placeholder", t.onAddonPlaceholderClick);
		
		// init sortable rows
		t.g_objGrid.sortable({
			handle : ".uc-row-icon-move",
			axis : "y",
			update : function() {
				t.triggerEvent(t.events.ROWS_UPDATED);
			},
			start : t.onSortElementStart,
			stop : t.onSortElementStop
		});
		
		// hover events
		var hoverSelectors = ".uc-grid-row, .uc-grid-row-container, .uc-grid-col, .uc-grid-col-addon";

		t.g_objGrid.on("mouseenter", hoverSelectors, t.onElementMouseOver);
		t.g_objGrid.on("mouseleave", hoverSelectors, t.onElementMouseOut);

		t.g_objGrid.on("dblclick", hoverSelectors, t.onElementDoubleClick);

		// init buffer events
		if (g_objBuffer)
			g_objBuffer.onEvent(g_objBuffer.events.UPDATED, t.updateRowsIcons);

		jQuery(window).bind("beforeunload", function() {

			if (t.g_pageBuilder) {
				var showMessage = t.g_pageBuilder.onBeforeUnload();
				if (showMessage == true)
					return ("Are you sure?");
			}

		});

		// send body click event to top window
		jQuery("body").click(function() {
			t.triggerEvent(t.events.BODY_CLICK);
		});

		// on contenteditable field change
		t.g_objGrid.on("input", ".uc-font-editable-field", t.onEditableFieldChange);
		t.g_objGrid.on("paste", ".uc-font-editable-field", t.onEditableFieldPaste);
		
		jQuery(document).on("keydown", t.onKeyDown);
		
	}

	/**
	 * init tipsy
	 */
	this.initTipsy = function() {

		if (typeof jQuery("body").tipsy != "function")
			return (false);

		var tipsyOptions = {
			html : true,
			gravity : "s",
			delayIn : 1000,
			selector : ".uc-tip"
		};

		t.g_objGrid.tipsy(tipsyOptions);

	}

	/**
	 * init side panel
	 */
	this.initPanel = function() {

		if (!t.g_pageBuilder) {

			t.g_panel = null;
			return (false);
		}

		t.g_panel = new UniteCreatorGridPanel();
		t.g_panel = t.g_pageBuilder.getSidePanel();

		// init panel events

		t.g_panel.onEvent(t.g_panel.events.SETTINGS_CHANGE, t.onPanelSettingsChange);
		t.g_panel.onEvent(t.g_panel.events.SETTINGS_HTML_LOADED,t.onPanelHtmlSettingsLoaded);
		t.g_panel.onEvent(t.g_panel.events.HEAD_MOUSEOVER, t.onPanelHeadMouseOver);
		t.g_panel.onEvent(t.g_panel.events.HEAD_MOUSEOUT, t.onPanelHeadMouseOut);
		t.g_panel.onEvent(t.g_panel.events.PANEL_MOUSEOVER, t.onPanelMouseOver);
		t.g_panel.onEvent(t.g_panel.events.PANEL_MOUSEOUT, t.removeAllElementHoverMode);
		
	}
	
	
	/**
	 * init sortable columns
	 */
	this.initSortableColumns = function(objContainer) {

		// init columns sortable
		objContainer.sortable({

			items : ".uc-grid-col",
			handle : ".uc-col-icon-move",
			cursor : "move",
			axis : "x",
			update : function(event, ui) {
				var objCol = ui.item;
				var objContainer = t.getParentRowContainer(objCol);

				t.triggerEvent(t.events.ROW_COLUMNS_UPDATED, objContainer);
			},
			start : t.onSortElementStart,
			stop : t.onSortElementStop
		});

	}

	/**
	 * init row sortable
	 */
	this.initSortableContainers = function(objRow) {
		
		// init container sortable
		objRow.sortable({
			items : ".uc-grid-row-container",
			handle : ".uc-container-icon-move",
			cursor : "move",
			axis : "y",
			update : function(event, ui) {
				var objContainer = ui.item;
				var objRow = t.getParentRow(objContainer);
				
				t.triggerEvent(t.events.ROW_CONTAINERS_UPDATED, objRow);
			},
			start : t.onSortElementStart,
			stop : t.onSortElementStop
		});
		
	}

	
	/**
	 * init by column data
	 */
	this.initByData_col = function(col, objContainer, objCol, mode){
		
		if(!objCol)
			var objCol = null;
		
		if(!mode)
			var mode = null;
		
		var objCol = t.addColumn(objContainer, objCol, mode);
		var addonsData = col.addon_data;
		
		if (jQuery.isArray(addonsData)) {
			
			// add addons placeholders
			var arrPlaceholders = null;
			if (addonsData.length > 1) {
				arrPlaceholders = [];
				var objAddonsWrapper = t.getColAddonsWrapper(objCol);
				
				jQuery.each(addonsData, function(addonIndex, addonData) {
					var placeholderID = "uc_placeholder_" + g_ucAdmin.getRandomString();
					var htmlPlaceholder = "<span id='" + placeholderID + "'></span>";
					arrPlaceholders.push(placeholderID);
					objAddonsWrapper.append(htmlPlaceholder);
				});
			}
			
			// add addons
			jQuery.each(addonsData, function(addonIndex, addonData) {
				try {
					var placeholderID = null;
					if (arrPlaceholders)
						placeholderID = arrPlaceholders[addonIndex];
					
					t.addColAddon(objCol, addonData, true, placeholderID);

				} catch (Error) {
					// skip error
					trace(Error);
				}

			});

		} else {
			// single - old way
			try {
				if (addonsData)
					t.addColAddon(objCol, addonsData, true);

			} catch (Error) {
			}

		}
		
		var settings = g_ucAdmin.getVal(col, "settings");
		if (settings)
			t.updateColSettings(objCol, settings);
		
	}
	
	
	// init columns from data inside container
	this.initByData_cols = function(cols, objContainer) {
		
		jQuery.each(cols, function(colIndex, col) {
			
			t.initByData_col(col, objContainer);
			
		}); // end add columns

		// set custom sizes, update visual
		var objCols = t.getCols(objContainer);
		
		jQuery.each(objCols, function(index, col) {

			var objCol = jQuery(col);
			var colData = cols[index];
			
			var size = g_ucAdmin.getVal(colData, "size");
			if (size)
				t.setColSize(objCol, size);

		});

	}
	
	/**
	 * init by data - container
	 */
	this.initByData_container = function(container, objRow, insertTo){
		
		var objContainer = t.addEmptyRowContainer(objRow, insertTo);
				
		//---- add columns
		
		g_ucAdmin.validateObjProperty(container, "cols");
		
		var cols = container.cols;
		t.initByData_cols(cols, objContainer);
		
		// update container visual
		if (container.hasOwnProperty("settings") && typeof container.settings == "object")
			t.updateContainerSettings(objContainer, container.settings);
		
	}
	
	
	/**
	 * init containers by data
	 */
	this.initByData_containers = function(containers, objRow){
		
		t.validateRow(objRow);
		
		jQuery.each(containers, function(indexContainer, container) {
			
			t.initByData_container(container, objRow);
			
		});
		
	}
	
	
	/**
	 * init row by data
	 */
	this.initByData_row = function(row, objParentRow, insertTo) {
		
		var containers = g_ucAdmin.getVal(row, "containers");
		
		var numContainers = containers.length;
		if (numContainers == 0)
			return(false);
				
		var objRow = t.addEmptyRow(objParentRow, insertTo, true);
				
		//add containers
		t.initByData_containers(containers, objRow);
		
		// update row visual
		if (row.hasOwnProperty("settings") && typeof row.settings == "object") {
			t.updateRowSettings(objRow, row.settings);
		}
		
	}
	
	
	/**
	 * init rows
	 */
	this.initByData_rows = function(rows) {

		jQuery.map(rows, function(row, rowIndex) {

			t.initByData_row(row);
		});

	}

	/**
	 * init options by data
	 */
	this.initByData_options = function(options) {

		t.g_optionsCustom = options;

		if (!t.g_panel)
			return (false);
		
		t.g_panel.setSettingsValues("grid-settings", t.g_optionsCustom);
		
		if (t.g_panel.isVisible())
			t.updateGridSettingsPlaceholders();
	}

	/**
	 * init the builder by data
	 */
	this.initByData = function(initData){
		
		try {

			g_ucAdmin.validateObjProperty(initData, "rows");

			// init options
			if (initData.hasOwnProperty("options"))
				t.initByData_options(initData.options);

			// init rows
			if (initData.hasOwnProperty("rows"))
				t.initByData_rows(initData.rows);

		} catch (error) {
			var errorText = "Error in grid init: " + error
					+ " <br><br>Save disabled, Please move to box view";
			t.showErrorMessage(errorText);

			if (t.g_pageBuilder)
				t.g_pageBuilder.hideSaveButton();

			throw (error);

		}

	}
	
	/**
	 * do grid action
	 */
	this.doAction = function(action, params) {
		t.doGridAction(action, null, params);
	};

	/**
	 * init builder options
	 */
	this.initBuilderOptions = function() {
		
		var builderOptions = t.g_objGrid.data("builder-options");
		t.g_temp.indev = builderOptions["indev"];
		t.g_temp.google_fonts = builderOptions["google_fonts"];
		t.g_temp.icons = builderOptions["icons"];
		t.g_temp.page_params = builderOptions["params"];

		t.g_objGrid.removeAttr("data-builder-options"); // remove attribute for
														// not interfere
	}

	/**
	 * init page builder - top window class
	 */
	this.initPageBuilder = function() {
		
		if (window.top == window) {
			trace("the grid is not inside iframe");
			return (false);
		}
		
		if (typeof window.top.g_objPageBuilder == "undefined") {

			t.g_temp.init_counter++;
			if (t.g_temp.init_counter < 60) // 30 seconds
				setTimeout(t.initPageBuilder, 500);
			else
				alert("The page builder is not inited, please write support ticket, we'll help you with this issue");

			return (false);
		}

		if (!window.top.g_objPageBuilder) {
			
			t.g_temp.init_counter++;
			if (t.g_temp.init_counter < 60) // 30 seconds
				setTimeout(t.initPageBuilder, 500);
			else
				alert("page builder in parent frame not inited. please write support ticket, we'll help you with this issue");

			return (false);
		}
		
		// after page builder inited
		t.g_pageBuilder = window.top.g_objPageBuilder;
		
		t.g_objPageBuilderWrapper = t.g_pageBuilder.getPageBuilderWrapper();
		
		window.top.g_objPageBuilder.initGridBuilder(t);
		
		g_objBuffer = t.g_pageBuilder.getObjBuffer();
		
		// init browser
		g_objBrowser = t.g_pageBuilder.getObjBrowser();
		g_objBrowserSections = t.g_pageBuilder.getObjBrowser("sections");

		t.g_objBodyParent = t.g_pageBuilder.getDomBody();
		
		t.initPanel();
		
		t.g_objDragDrop.init(t);
		
		// init events
		g_ucAdmin.onEvent("install_addon", t.onInstallAddon, t.g_objBodyParent);
		
	}

	/**
	 * check duplicated IDs and trace if available
	 */
	this.checkDuplicateIDs = function() {
		
		var arrIDs = {};
		var allElements = t.g_objWrapper.find("div,span");
		var numIDs = 0;

		jQuery.each(allElements, function(index, element) {
			var objElement = jQuery(element);
			var id = objElement.prop("id");
			if (!id)
				return (true);

			if (arrIDs.hasOwnProperty(id)) {
				trace("duplicate ID Found: " + id);
				numIDs++;
			}

			arrIDs[id] = true;
		});

		if (numIDs == 0)
			trace("duplicate ID not found");
		else
			trace("duplicate ID found");

		setTimeout(function() {
			alert("duplicate ID check finished, please remove it: " + numIDs);
		}, 2000);

	}

	/**
	 * init shape deviders
	 */
	this.initShapeDividers = function() {
		
		if (typeof g_ucShapeDividersJson == "undefined") {

			t.g_vars.shape_deviders = [];

			// window.g_ucShapeDividersJson = [];
			// throw new Error("shape deviders object not found");
			// return(false);
		} else
			t.g_vars.shape_deviders = JSON.parse(g_ucShapeDividersJson);

	}

	/**
	 * get temp vars
	 */
	this.getTempVars = function() {
		
		return t.g_temp;
	};

	/**
	 * init grid
	 */
	this.init = function(gridID) {

		try {
			
			t.g_objGrid = jQuery(gridID);
			if (t.g_objGrid.length == 0)
				throw new Error("grid object: " + gridID + " not found");

			g_gridID = gridID;
			
			// init parent frame
			t.initPageBuilder();

			t.g_objWrapper = t.g_objGrid.parents(".uc-grid-builder-wrapper");
			
			t.initBuilderOptions();

			// init live view
			var isLiveView = t.g_objGrid.data("liveview");
			t.g_temp.is_live_view = g_ucAdmin.strToBool(isLiveView);
			
			//init grid data object
			t.g_objGridData.init(t);
			
			//init addon html object
			t.g_objAddonHTML.init(t);
						
			t.initShapeDividers();

			// init panel
			t.initEvents();

			t.initGridSettings();

			if (g_objBuffer)
				g_objBuffer.run();

			g_objRowStyleContainer = t.g_objWrapper.children(".uc-grid-row-styles");
			g_objColStyleContainer = t.g_objWrapper.children(".uc-grid-col-styles");
			
			// add the data
			var initData = t.g_objGrid.data("init");
			if (initData) {
				t.initByData(initData);
				t.g_objGrid.removeAttr("data-init"); // remove attribute for not
													// interfere
			} else {
				t.addRow();
			}
			
			t.triggerEvent(t.events.DATA_INITED);
			t.g_temp.is_data_inited = true;

			// put the css by the options
			t.putGeneratedCss();

			// init smooth scroll
			g_objScroll = new SmoothScroll('.uc-smooth-scroll a[href*="#"]');

			g_objHistory.initGrid(this);
			
			
			// t.initTipsy();

		} catch (error) {

			t.showErrorMessage(error);

		}
	};

}