<?php
/**
 * @package Blox Page Builder
 * @author UniteCMS.net
 * @copyright (C) 2017 Unite CMS, All Rights Reserved. 
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * */
defined('_JEXEC') or die('Restricted access');

class UniteJoomlaK2ItemUC extends UniteElementsBaseUC{
	
	const TABLE_ITEMS = "#__k2_items";
	
	private $id;
	private $record;
	private $data;
	private $params;
	
	
	/**
	 * validate that the article is inited
	 */
	private function validateInited(){
		if(empty($this->id))
			UniteFunctionsUC::throwError("The article is not inited");
	}
	
	
	/**
	 * get article title
	 */
	public function getTitle($specialChars = false){
		$this->validateInited();
		
		$title = UniteFunctionsUC::getVal($this->record, "title");
		if($specialChars == true)
			$title = htmlspecialchars($title);
			
		return($title);
	}
	
	/**
	 * get content
	 */
	public function getContent(){
		
		$intro = $this->record["introtext"];
		$full = $this->record["fulltext"];
		
		$content = $full;
		if(empty($content))
			$content = $intro;
		
		return($content);
	}
	
	/**
	 * get image
	 */
	private function getImage(){
		
		$urlImage = JPATH_SITE.DIRECTORY_SEPARATOR.'media'.DIRECTORY_SEPARATOR.'k2'.DIRECTORY_SEPARATOR.'items'.DIRECTORY_SEPARATOR.'cache'.DIRECTORY_SEPARATOR.md5("Image".$this->id).'_L.jpg';
		if(file_exists($urlImage))
			return($urlImage);
		
		return("");
	}
	
	
	/**
	 * get image
	 */
	private function getThumb(){
	
		$urlThumb = JPATH_SITE.DIRECTORY_SEPARATOR.'media'.DIRECTORY_SEPARATOR.'k2'.DIRECTORY_SEPARATOR.'items'.DIRECTORY_SEPARATOR.'cache'.DIRECTORY_SEPARATOR.md5("Image".$this->id).'_S.jpg';
		if(file_exists($urlThumb))
			return($urlThumb);
		
		return("");
	}
	
	
	/**
	 * get data from the item
	 */
	public function getData(){
		
		if(!empty($this->data))
			return($this->data);
		
		$this->validateInited();
		
		$data = array();
		$data["id"] = UniteFunctionsUC::getVal($this->record, "id");
		$data["title"] = $this->getTitle();
		$data["alias"] = UniteFunctionsUC::getVal($this->record, "alias");
		$data["intro"] = UniteFunctionsUC::getVal($this->record, "introtext");
		$data["content"] = $this->getContent();
		$data["image"] = $this->getImage();
		$data["thumb"] = $this->getThumb();
				
		$this->data = $data;
			
		return($data);
	}
	
	
	/**
	 * init article by db record
	 */
	public function initByRecord($record){
		
		$this->record = $record;
		
		$this->params = $record["params"];
		$this->params = UniteFunctionsUC::jsonDecode($this->params);
				
		$this->id = $record["id"];
	}
	
	
	/**
	 * init by id
	 */
	public function initByID($id){
		
		$id = (int)$id;
		if(empty($id))
			UniteFunctionsUC::throwError("Empty K2 Item ID");
		
		try{
			
			$record = $this->db->fetchSingle(self::TABLE_ITEMS,"id=".$id);
					
		}catch(Exception $e){
			$message = "k2 item with id: $id not found";
			UniteFunctionsUC::throwError($message);
		}
		
		$this->initByRecord($record);
		
	}
	
}

