<?php
/**
 * @package Blox Page Builder
 * @author UniteCMS.net
 * @copyright (C) 2017 Unite CMS, All Rights Reserved. 
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * */
defined('_JEXEC') or die('Restricted access');

class UniteCreatorParamsProcessor extends UniteCreatorParamsProcessorWork{
	
	
	/**
	 * get content selector data
	 */
	protected function getContentSelectorData($value, $name, $processType){
	   	
	    $arrValues = $value;
	    
	    if(is_array($arrValues) == false)
	        return($value);
	    
	    $type = UniteFunctionsUC::getVal($arrValues, $name); 
	    	    
	    switch($type){
	        case "custom":
	            $value = UniteFunctionsUC::getVal($arrValues, $name."_content");
	        break;
	        case "post":
	            $value = UniteFunctionsUC::getVal($arrValues, $name."_postid");
	            
	            $data = $this->getPostData($value);
	            if($data == null)
	                $value = "";
	            
	            $value = UniteFunctionsUC::getVal($data, "content");
	            	            
	        break;
	        case "k2item":
	            $itemID = UniteFunctionsUC::getVal($arrValues, $name."_k2item");
	            
	            $value = UniteFunctionJoomlaUC::getK2ItemContent($itemID);
	            
	        break;
	        case "module":
	            $moduleID = UniteFunctionsUC::getVal($arrValues, $name."_moduleid");
	            
	            $value = $this->getModuleParamData($moduleID, $processType);
	        break;
	        default:
	          	$value = $type;
	        break;
	    }
	    
	    
	    return($value);
	}
	
	
	
	
	/**
	 * get processe param data, function with override
	 */
	protected function getProcessedParamData($data, $value, $param, $processType){
		
		$type = UniteFunctionsUC::getVal($param, "type");
		$name = UniteFunctionsUC::getVal($param, "name");
		
		//special params
		switch($type){
			
			case UniteCreatorDialogParam::PARAM_JOOMLA_MODULE:
				$data[$name] = $this->getModuleParamData($value, $processType);
			break;
			case UniteCreatorDialogParam::PARAM_JOOMLA_K2_ITEM:
			    			    
				$data[$name] = $this->getK2ArticleData($value, $param, $processType);
			break;
			case UniteCreatorDialogParam::PARAM_CONTENT:
			    
			    $data[$name] = $this->getContentSelectorData($value, $name, $processType);
			    
			break;
			case UniteCreatorDialogParam::PARAM_POSTS_LIST:
			    $data[$name] = $this->getArticlesListData($value, $name, $processType);
			break;
			default:
				$data = parent::getProcessedParamData($data, $value, $param, $processType);
			break;
		}
		
			
		return($data);
	}
	
	
	/**
	 * return if param value is array
	 */
	protected function isParamValueIsArray($paramType){
		
	    switch($paramType){
	        case UniteCreatorDialogParam::PARAM_POSTS_LIST:
	        case UniteCreatorDialogParam::PARAM_MENU:
	        case UniteCreatorDialogParam::PARAM_CONTENT:
	        	return(true);
	        break;
	    }
		
	    return parent::isParamValueIsArray($paramType);
	}
	
	
	
	private function a________K2______(){}
	
	
	/**
	 * get k2 article data
	 */
	protected function getK2ArticleData($value, $param, $processType){
		
		$name = UniteFunctionsUC::getVal($param, "name");
		
		try{
			$itemID = $value;
			
			$k2Item = new UniteJoomlaK2ItemUC();
			$k2Item->initByID($itemID);
			$dataK2 = $k2Item->getData();
			
			return($dataK2);
			
			/*
			$data[$name] = $dataK2["content"];
			$data[$name."_content"] = $dataK2["content"];
			$data[$name."_id"] = $dataK2["id"];
			$data[$name."_title"] = $dataK2["title"];
			$data[$name."_alias"] = $dataK2["alias"];
			$data[$name."_image"] = $dataK2["image"];
			$data[$name."_thumb"] = $dataK2["thumb"];
			*/
			
		}catch(Exception $e){
			$data = array();
			$data["content"] = $e->getMessage();
		}
		
		
		return($data);
	}
	
	
	private function a________MODULE______(){}
	
	
	/**
	 * get module output for backend
	 */
	private function getModuleBackendOutput($arrModule){
		
		$title = UniteFunctionsUC::getVal($arrModule, "title");
		$module = UniteFunctionsUC::getVal($arrModule, "module");
		
		$textJoomlaModule = __("Joomla Module", BLOXBUILDER_TEXTDOMAIN);
		$textDesc = "Module: $title ($module)";
		$iconJoomla = GlobalsUC::$url_provider."assets/images/icon_joomla.png";
		
		$html = "<div class=\"uc-joomla-module-box\">";
		$html .= "<img src=\"{$iconJoomla}\">";
		$html .= "<span class=\"uc-grid-addon-title\">{$textJoomlaModule}</span>";
		$html .= "<span class=\"uc-grid-addon-description\">{$textDesc}</span>";
		$html .= "</div>";
		
		return($html);
	}
	
	/**
	 * get module param data
	 */
	private function getModuleParamData($moduleID, $processType){

		//processs only those types
		
		switch($processType){
			case self::PROCESS_TYPE_OUTPUT:
			case self::PROCESS_TYPE_OUTPUT_BACK:
			break;
			default:
				return($moduleID);
			break;
		}
		
		
		try{
			
			$arrModule = UniteFunctionJoomlaUC::getModuleByID($moduleID);
			
			switch($processType){
				case self::PROCESS_TYPE_OUTPUT:
					$arrModule = (object)$arrModule;
					$output = JModuleHelper::renderModule($arrModule);
				break;
				case self::PROCESS_TYPE_OUTPUT_BACK:
					$output = $this->getModuleBackendOutput($arrModule);
				break;
				default:
					$output = "wrong process type: $processType";
				break;
			}
										
		}catch(Exception $e){
			$output = __("Module with ID: $moduleID not found", BLOXBUILDER_TEXTDOMAIN);
		}
		
				
		return($output);
	}
	
	
	private function a________ARTICLES______(){}
	
	
	/**
	 * get article data
	 */
	private function getArticleData(UniteJoomlaArticleUC $article){
		
		$output = array();
		
		$data = $article->getData();
		$intro = UniteFunctionsUC::getVal($data, "introtext");
		$fulltext = UniteFunctionsUC::getVal($data, "fulltext");
		if(empty($fulltext))
			$fulltext = $intro;
		
		$output = array();
		$output["id"] = UniteFunctionsUC::getVal($data, "id");
		$output["title"] = UniteFunctionsUC::getVal($data, "title");
		$output["alias"] = UniteFunctionsUC::getVal($data, "alias");
		$output["intro"] = $intro;
		$output["content"] = $fulltext;
		$output["link"] = UniteFunctionsUC::getVal($data, "link");
		$output["date"] = UniteFunctionsUC::getVal($data, "datestamp");
		
		
		$urlImage = $article->getUrlImageIntro();
		$output = $this->getImageFields($output, "image", $urlImage);
		
		
		return($output);
	}
	
	
	/**
	 * get post data
	 */
	protected function getPostData($postID){
		
		if(empty($postID))
			return(null);
		
		try{
			
			$article = new UniteJoomlaArticleUC();
			$article->initByID($postID);
			$output = $this->getArticleData($article);
			
		}catch(Exception $e){
			return(null);
		}
		
		return($output);
	}
	
	
	/**
	 * get articles list data
	 */
	private function getArticlesListData($value, $name, $processType){
		
		if(empty($value))
			return(array());
		
		if(is_array($value) == false)
			return(array());
		
		if($processType != self::PROCESS_TYPE_OUTPUT && $processType != self::PROCESS_TYPE_OUTPUT_BACK)
			return(null);
		
		
		$filters = array();	
		$filters["category"] = UniteFunctionsUC::getVal($value, "{$name}_category");
		
		$limit = UniteFunctionsUC::getVal($value, "{$name}_maxitems");
		
		$limit = (int)$limit;
		if($limit <= 0)
			$limit = 100;
		
		if($limit > 1000)
			$limit = 1000;
		
		$filters["limit"] = $limit;
		$filters["orderby"] = UniteFunctionsUC::getVal($value, "{$name}_orderby");
		$filters["orderdir"] = UniteFunctionsUC::getVal($value, "{$name}_orderdir1");
		$filters["orderby2"] = UniteFunctionsUC::getVal($value, "{$name}_orderby2");
		$filters["orderdir2"] = UniteFunctionsUC::getVal($value, "{$name}_orderdir2");
		
		$arrRecords = UniteFunctionJoomlaUC::getArticlesRecords($filters);
		
		//get data output
		$arrData = array();
		foreach($arrRecords as $record){
			
			$article = new UniteJoomlaArticleUC();
			$article->initByRecord($record);
			
			$data = $this->getArticleData($article);
			$arrData[] = $data;
		}
		
		
		return($arrData);
	}
	
	
	
}