<?php

/**
 * @package Blox Page Builder
 * @author UniteCMS.net
 * @copyright (C) 2017 Unite CMS, All Rights Reserved. 
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * */
defined('APP_INC') or die('Restricted access');

class BloxMainAdmin_StandAlone extends UniteProviderAdminUC{

	protected $loginError;
	
	
    /**
     * constructor
     */
    public function __construct(){
    	
        if(GlobalsStandAlone::$isInstalled == true)
            GlobalsStandAlone::initLoggedInUser();           
        
    	$this->initBefore();
    	
    	HelperStandAlone::validateIsInstalled();
    	
    	parent::__construct();
    	  
    }
    
    /**
     * on php error
     */
    public function onPHPError($errno , $errstr="", $errfile="", $errline="", $errcontext=""){
    	
    	dmp($errno);
    	dmp($errstr);
    	exit();
    }

    /**
     * on php error
     */
    public function onPHPException(Error $e){
    	    	    	
    	HelperHtmlUC::outputError($e);
    	
    }
    
    /**
     * on ajax action
     */
    public function onStandAloneAjaxAction($found, $action, $data){
    	
    	$actions = new ActionsStandAlone();
    	$actions->doAjaxActions($action, $data);
    	    	    	
    	return($found);
    }
    
    /**
     * modify layout preview
     */
    private function modifyLayoutPreview($params){
    	
    	dmp("modify");
    	dmp($params);
    	//exit();
    	
    }
    
    
    /**
     * modify view url
     */
    public function getLayoutPreviewByID($urlPreview, $layoutID){
    	
    	if(empty($layoutID))
    		return($urlPreview);
    	
    	//return template
    	if($layoutID == "[page]"){
    		$urlTemplate = GlobalsStandAlone::$urlFront."[page]";
    		return($urlTemplate);
    	}
    	
    	$objLayout = new UniteCreatorLayout();
    	$objLayout->initByID($layoutID);
    	
    	$objLayoutType = $objLayout->getObjLayoutType();
    	
    	//preview by name
    	if($objLayoutType->isBasicType == true){
    		
    		$name = $objLayout->getName();
    		
    		if(!empty($name)){
    			$layoutID = $name;
    			
	    		if(GlobalsStandAlone::$isRewriteUrl == true){
	    			$urlPreview = GlobalsStandAlone::$urlFront.$name;
	    			return($urlPreview);
    			}
    			
    		}
    	}
    	
    	$urlPreview = UniteFunctionsUC::addUrlParams(GlobalsStandAlone::$urlFront, "page=$layoutID");
    	
    	return($urlPreview);
    }
    
    
    /**
     * init admin before
     */
    private function initBefore(){
    	
    	//set_error_handler(array($this, "onPHPError"));
    	set_exception_handler( array($this, "onPHPException"));
    	
    	$pluginBase = new UniteCreatorPluginBase();
    	$pluginBase->registerAdminViewPath(GlobalsStandAlone::$pathViews);
    	$pluginBase->registerAdminAjaxActionFunction(array($this, "onStandAloneAjaxAction"));
    	
    	$this->addFilter("get_layout_preview_byid", "getLayoutPreviewByID",10,2);
    	
    	//$link = UniteProviderFunctionsUC::applyFilters(UniteCreatorFilters::FILTER_MODIFY_URL_VIEW, $link, $viewName, $params, $isBlankWindow, $isFront);
    	
    }
    
    
    /**
     * check forgot pass action
     */
    private function checkForgotPassAction(){
    	    	
    	$forgotAction = UniteFunctionsUC::getPostVariable("forgot_pass_action", "", UniteFunctionsUC::SANITIZE_TEXT_FIELD);
    	$forgotAction = UniteFunctionsUC::strToBool($forgotAction);
    	
    	if($forgotAction == false)
    		return(false);
    	
    	if(GlobalsStandAlone::$isLoggedIn == true)
    		return(false);
    	
    	try{
    	
    		$email = UniteFunctionsUC::getPostVariable("reset_pass_email", "", UniteFunctionsUC::SANITIZE_TEXT_FIELD);
    		$isValid = UniteFunctionsUC::isEmailValid($email);
    		
    		if($isValid == false)
    			UniteFunctionsUC::throwError("The email not valid, try again");
    		
    		$objUsers = new UniteCreatorUsers();
    		$arrUser = $objUsers->getUserByEmail($email);
    		
    		$password  = $arrUser["password"];
    		
    		$message = "Hello, your password for $email is: {$password}";
    		$subject = __("Blox Password Reminder", BLOXBUILDER_TEXTDOMAIN);
    		
    		UniteProviderFunctionsUC::sendEmail($email, $subject, $message);
    		
    		$urlRedirect = HelperUC::getViewUrl(GlobalsStandAlone::VIEW_STANDALONE_RESET_PASSWORD,"sent=true&email={$email}");
    		
    		
    	}catch(Exception $e){
    		
    		$message = $e->getMessage();
    		
    		$urlRedirect = HelperUC::getViewUrl(GlobalsStandAlone::VIEW_STANDALONE_RESET_PASSWORD,"error={$message}&email=$email");
    		
    	}
    	
    	
    	UniteFunctionsUC::redirectToUrl($urlRedirect);
    	exit();
    }
    
    
    /**
     * check login action
     */
    private function checkLoginAction(){
    	
    	$loginAction = UniteFunctionsUC::getPostVariable("login_action", "", UniteFunctionsUC::SANITIZE_TEXT_FIELD);
    	$loginAction = UniteFunctionsUC::strToBool($loginAction);
    	
    	
    	if($loginAction == false)
    		return(false);
    	
    	if(GlobalsStandAlone::$isLoggedIn == true)
    		return(false);
    	
    	//if submitted - check user/pass
    	
    	$username = UniteFunctionsUC::getPostVariable("username", "", UniteFunctionsUC::SANITIZE_TEXT_FIELD);
    	$password = UniteFunctionsUC::getPostVariable("password", "", UniteFunctionsUC::SANITIZE_NOTHING);
    	
    	$objUsers = new UniteCreatorUsers();
    	    	
    	$arrUser = $objUsers->getUserBYUserPass($username, $password);
    	    	
    	if(empty($arrUser))
    		UniteFunctionsUC::throwError(__("Wrong username or password", BLOXBUILDER_TEXTDOMAIN));
    		
    	HelperStandAlone::doLoginUser($arrUser);
    	
    	
    	HelperStandAlone::redirectToAdmin();
    }
    
    
    /**
     * check logout action
     */
    private function checkLogoutAction(){
    	
    	$logoutAction = UniteFunctionsUC::getGetVar("logout_action", "", UniteFunctionsUC::SANITIZE_KEY);
    	$logoutAction = UniteFunctionsUC::strToBool($logoutAction);
    	
    	if($logoutAction == false)
    		return(false);
    	
    	HelperStandAlone::doLogout();
    	
    	HelperStandAlone::redirectToAdmin();
    }
    
    
    /**
     * init stand alone view
     */
    private function initStandAloneView(){
                
    	if(GlobalsStandAlone::$isLoggedIn == false && self::$view != GlobalsStandAlone::VIEW_STANDALONE_RESET_PASSWORD){
    		
    		self::$view = GlobalsStandAlone::VIEW_STANDALONE_LOGIN;
    		
    		try{
    			
    			$this->checkLoginAction();
    			$this->checkForgotPassAction();
    			
    		}catch(Exception $e){
    			
    			$this->loginError = $e->getMessage();
    			
    		}
    		
    	}else{
    			$this->checkLogoutAction();
    	}
    	
    }
    
    
    /**
     * on admin pages
     */
   	 public function adminPages(){
   	 	
   	 	$this->initStandAloneView();
   	 	
   	 	GlobalsStandAlone::$currentAdminView = self::$view;
   	 	
   	 	
   	 	switch(self::$view){		//views without master
   	 	      	 	    
   	 		case GlobalsStandAlone::VIEW_STANDALONE_RESET_PASSWORD:
   	 		case GlobalsStandAlone::VIEW_STANDALONE_LOGIN:
   	 		    
   	 		    HelperStandAlone::validateIsInstalled();
   	 		    HelperStandAlone::validateTablesExists();
   	 		    HelperStandAlone::validateUserExists();
   	 		    
   	 			$viewFilepath = GlobalsStandAlone::$pathViews.self::$view.".php";
   	 			require $viewFilepath; 
   	 			
   	 		break;
   	 		default:
   	 			
   	 			if(GlobalsUC::$blankWindowMode == true)
   	 				$viewFilepath = GlobalsStandAlone::$pathViews."master_stand_alone_blank.php";
   	 			else
   	 				$viewFilepath = GlobalsStandAlone::$pathViews."master_stand_alone.php";
   	 			
   	 			require $viewFilepath; 
   	 			
   	 			//parent::adminPages();
   	 		break;
   	 	}
   	 	
   	 }
	
   	 
}

?>