<?php

/**
 * @package Blox Page Builder
 * @author UniteCMS.net
 * @copyright (C) 2017 Unite CMS, All Rights Reserved. 
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * */
defined('APP_INC') or die('Restricted access');

class BloxMainFront_StandAlone{

	/**
	 * output home page
	 */
	private function outputHomePage(){
		
    	$homePageID = HelperUC::getGeneralSetting("home_page_id");
    	if(is_numeric($homePageID) == false)
    		return(false);
		
    	$homePageID = (int)$homePageID;
    		
    	try{
    		$objLayout = new UniteCreatorLayout();
    		$objLayout->initByID($homePageID);
    		
    	}catch(Exception $e){
    		
    		return(false);
    	}
    		
    	HelperUC::outputLayout($objLayout, false, true);
    	
    	return(true);
	}
	
	
	/**
	 * output layout by redirect url
	 */
	private function outputByRedirectURL(){
    	    	
		$redirectURL = UniteFunctionsUC::getVal($_SERVER, "REDIRECT_URL");
    	if(empty($redirectURL))
    		return(false);
    	
		$scriptURL = UniteFunctionsUC::getVal($_SERVER, "SCRIPT_URL");
		if(!empty($scriptURL))
			$redirectURL = $scriptURL;
    	
		if($redirectURL == "/index.php")
			return(false);
     	
		// there is also: REQUEST_URI
    	
    	$info = pathinfo($redirectURL);
    	$pageName = UniteFunctionsUC::getVal($info, "filename");
    		
    	$objLayout = new UniteCreatorLayout();
    	$objLayout->initByName($pageName, GlobalsUC::ADDON_TYPE_LAYOUT_GENERAL);
    		
    	HelperUC::outputLayout($objLayout, false, true);
    	
    	return(true);
	}
	
	/**
	 * check ajax action front
	 */
	private function checkAjaxAction(){
		
		$action = UniteFunctionsUC::getPostGetVariable("action", "", UniteFunctionsUC::SANITIZE_KEY);

		if($action != "bloxbuilder_ajax_action")
			return(false);
		
		$actions = new UniteCreatorActions();
		$actions->onAjaxFrontAction();
				
	}
	
	
    /**
     * run front
     */
    public function run(){
    	
    	HelperStandAlone::validateIsInstalled();
    	
    	$this->checkAjaxAction();
    	
    	$layoutID = UniteFunctionsUC::getGetVar("page", "", UniteFunctionsUC::SANITIZE_TEXT_FIELD);
    	
    	//meanwhile numeric only
    	$layoutID = (int)$layoutID;
    	    	
    	if(!empty($layoutID)){
    		HelperUC::outputLayout($layoutID, false, true);
    		return(false);
    	}
    	
    	//--- check url rewrite
    	
		$isOutput = $this->outputByRedirectURL();
    	if($isOutput == true)
    		return(false);
    	
    	//--- check home page
    	
    	$isOutput = $this->outputHomePage();
    		if($isOutput == true)
    			return(false);

    	$textNothingHere = __("You have no homepage defined", BLOXBUILDER_TEXTDOMAIN);
    	$textNothingHere .= "<br>";
    	$textNothingHere .= __("Please go to general settings and choose the layout you want to show as the homepage", BLOXBUILDER_TEXTDOMAIN);

    	echo $textNothingHere;
    	
    }

}

?>