<?php
/**
 * @package Blox Page Builder
 * @author UniteCMS.net
 * @copyright (C) 2017 Unite CMS, All Rights Reserved. 
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * */
defined('BLOXBUILDER_INC') or die('Restricted access');

class UniteCreatorUsers extends UniteElementsBaseUC{
	
	/**
	 * get default user record
	 */
	public function getCurrentUserRecord(){
		
		$user = GlobalsStandAlone::$loggedInUser;
		
		if(empty($user))
			UniteFunctionsUC::throwError("Current user not found");	
		
		$userID = UniteFunctionsUC::getVal($user, "id");
		
		$record = $this->getUserByID($userID);
		
		return($record);
	}
	
	/**
	 * get current user id
	 */
	public function getCurrentUserID(){
		
		$record = $this->getCurrentUserRecord();
		
		$userID = UniteFunctionsUC::getVal($record, "id");
		
		return($userID);
	}
	
	
	/**
	 * get number of users
	 */
	public function getTotalUsers(){
		
		$total = $this->db->getTotalRows(GlobalsStandAlone::$tableUsers);
		
		return($total);
	}
	
	/**
	 * insert user
	 */
	public function insertUser($username, $password, $email){
	
        $arrInsert = Array();
        $arrInsert["username"] = $username;
        $arrInsert["password"] = $password;
        $arrInsert["email"] = $email;
        $arrInsert["date_added"] = time();
        $arrInsert["role"] = 'admin';		
        $arrInsert["params"] = '';		
        
        $this->db->insert(GlobalsStandAlone::$tableUsers, $arrInsert);
        
	}
		
	
	/**
	 * update admin profile user
	 */
	public function updateAdminProfileUser($arrUpdate){
		
		$userID = $this->getCurrentUserID();
		$userID = (int)$userID;
				
		$this->db->update(GlobalsStandAlone::$tableUsers, $arrUpdate, "id={$userID}");
	}
	
	
	/**
	 * update profile user
	 */
	public function updateAdminProfileUserBYData($data){
		
		$arrValues = UniteFunctionsUC::getVal($data, "user_values");
		UniteFunctionsUC::validateNotEmpty($arrValues, __("user values",BLOXBUILDER_TEXTDOMAIN));
		
		//get username
		$username = UniteFunctionsUC::getVal($arrValues, "username");
		UniteFunctionsUC::validateNotEmpty($username, __("User Name",BLOXBUILDER_TEXTDOMAIN));
		UniteFunctionsUC::validateAlphaNumeric($username, __("User Name",BLOXBUILDER_TEXTDOMAIN));

		$fullname = UniteFunctionsUC::getVal($arrValues, "fullname");
		$fullname = HelperProviderUC::sanitizeTextField($fullname);
		$fullname = trim($fullname);
		
		$email = UniteFunctionsUC::getVal($arrValues, "email");
		UniteFunctionsUC::validateEmail($email);
		
		$password = UniteFunctionsUC::getVal($arrValues, "password");
		$password = trim($password);
		
		$arrUpdate = array();
		$arrUpdate["username"] = $username;
		$arrUpdate["fullname"] = $fullname;
		$arrUpdate["email"] = $email;
		
		if(!empty($password))
			$arrUpdate["password"] = $password;
		
		$this->updateAdminProfileUser($arrUpdate);
	}
	
	
	/**
	 * get user by ID
	 */
	public function getUserByID($userID){
		
		$userID = (int)$userID;
		
		$response = $this->db->fetchSingle(GlobalsStandAlone::$tableUsers, "id={$userID}");
		
		return($response);
	}
	
	/**
	 * get user by email
	 */
	public function getUserByEmail($email){
		
		$response = $this->db->fetch(GlobalsStandAlone::$tableUsers, array("email"=>$email));
		
		if(empty($response))
			UniteFunctionsUC::throwError("User with email: $email not found", BLOXBUILDER_TEXTDOMAIN);
		
		$arrUser = $response[0];
		
		return($arrUser);
	}
	
	/**
	 * get user by username / password
	 */
	public function getUserBYUserPass($username, $password){
		
		if(empty($username))
			return(null);
		
		if(empty($password))
			return(null);
			
		$where = array();
		$where["username"] = $username;
		$where["password"] = $password;
		
		
		$response = $this->db->fetch(GlobalsStandAlone::$tableUsers, $where);
		
		if(empty($response))
			return(null);
		
		if(count($response) != 1)
			return(null);
		
		$arrUser = $response[0];
		
		return($arrUser);
	}
	
	
}	