<?php

define("IS_INSTALLER", true);

ini_set("display_errors","on");

class BloxInstaller{
	
	const STEP_INSTALL_DB = "install_db";
	const STEP_INSTALL_USER = "install_user";
	const STEP_INSTALLED = "installed";
	const STEP_INSTALL_TABLES_AGAIN = "install_tables_again";
	
	private $step;
	
	/**
	 * set step
	 */
	public function setStep($step=false){
		
	    if(!empty($step)){
	        
	        $this->step = $step;
	        return(false);
	    }
	    
	    if(!empty($this->step))
	        return(false);
	    
		//check if db installed
		
		$isInstalled = HelperInstaller::isDBInstalled();
		if($isInstalled == false){
			$this->step = self::STEP_INSTALL_DB;
			return(false);
		}
		
		$isTablesExists = HelperStandAlone::isTablesExists();
		if($isTablesExists == false){
		    $this->step = self::STEP_INSTALL_TABLES_AGAIN;
		    return(false);
		}
		    
		
		//check if user installed
		$isUserInstalled = HelperInstaller::isUserInstalled();
		if($isUserInstalled == false)
			$this->step = self::STEP_INSTALL_USER;
		else
			$this->step = self::STEP_INSTALLED;
		
	}
	
	
	/**
	 * run install db step
	 */
	private function runInstallDBStep(){
		
		HelperInstaller::checkStep1Submit();
		
		$view = HelperInstaller::getPathView("step_1");;
		
        require $view;
	}
	
	
	/**
	 * install user step
	 */
	private function runInstallUserStep(){
		
		HelperInstaller::checkStep2Submit();
		
		$view = HelperInstaller::getPathView("step_2");
				
		require $view;		
	}
	
	/**
	 * write step installed
	 */
	private function runInstalledStep(){
		
		$view = HelperInstaller::getPathView("confirm");
		require $view;
		
	}
	
	/**
	 * install tables again
	 */
	private function runInstallTablesAgain(){
	    
	    dmp("one or more tables didn't installed. Installing again...");
	    
	    require_once "inc_php/helper_installer.class.php";
	    
	    HelperInstaller::installDBOnly();
	    
	    dmp("db tables installed, checking...");
	    
	    $isExists = HelperStandAlone::isTablesExists();
	    
	    if($isExists == true)
	        dmp("all tables installed, please refresh the page");
	    else
	        dmp("some tables are not installed. please turn to developers");
	      	    
	}
	
	/**
	 * run the installer
	 */
	private function runTheInstaller(){
	    
		$this->setStep();
		
		
		switch($this->step){
			
			case self::STEP_INSTALL_DB:
				
				$this->runInstallDBStep();
				
			break;
			case self::STEP_INSTALL_USER:
				
				$this->runInstallUserStep();
				
			break;
			case self::STEP_INSTALLED:
				
				$this->runInstalledStep();
				
			break;
			case self::STEP_INSTALL_TABLES_AGAIN:
			    
			    $this->runInstallTablesAgain();
			    
			break;
			default:
				UniteFunctionsUC::throwError("Wrong install step: ".$this->step);
			break;
		}
		
	}
	
	
	/**
	 * run the installer
	 */
	public function run(){
		
		try{
			
			$this->runTheInstaller();
			
		}catch(Exception $e){
			
			$error = $e->getMessage();
			$trace = $e->getTraceAsString();
			
			switch($this->step) {
				case self::STEP_INSTALL_DB:
					$viewError = HelperInstaller::getPathView("step1_error");
			        require $viewError;
			    break;
				case self::STEP_INSTALL_USER:
					$viewStep2 = HelperInstaller::getPathView("step_2");
			        require $viewStep2;
				break;
				default:
				    
					HelperHtmlUC::outputException($e, "Blox Installer");
				break;
			}
			
		}//catch
		
	}//run function
	
}

//run the installer

try{

	require "includes.php";
	
	$objInstaller = new BloxInstaller();
	$objInstaller->run();
	
}catch(Exception $e){
	
    $isTablesExists = HelperStandAlone::isTablesExists();
    
    if($isTablesExists == false){
        
        $objInstaller = new BloxInstaller();
        $objInstaller->setStep(BloxInstaller::STEP_INSTALL_TABLES_AGAIN);
        $objInstaller->run();
        
        exit();
           
    }else{
        
        dmp("check tables not exists");
        
        
        echo "<b>Error Occured</b>";
        echo "<pre>";
        
        print_r($e->getMessage());
        
        echo "<br><br>";
        
        $message = "Please delete config.php file and try to install again";
        print_r($message);
        
    }
    
	
	//print_r($e->getTraceAsString());
}
